@@ -8,21 +8,21 @@
  *	separately.  This module computes the extra CIF that may be
  *	needed.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cif/CIFhier.c,v 1.2 2008/12/04 16:21:43 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cif/CIFhier.c,v 1.3 2010/06/24 12:37:15 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -82,7 +82,7 @@ static CIFLayer *CurCifLayer;
     (dst)->r_ybot = (src)->r_ybot * scale; \
     (dst)->r_xtop = (src)->r_xtop * scale; \
     (dst)->r_ytop = (src)->r_ytop * scale;
-
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -112,7 +112,7 @@ CIFInitCells()
     CIFTotalDef = DBCellLookDef("__CIF__");
     if (CIFTotalDef == (CellDef *) NULL)
     {
-	CIFTotalDef = DBCellNewDef ("__CIF__",(char *) NULL);
+	CIFTotalDef = DBCellNewDef("__CIF__");
 	ASSERT(CIFTotalDef != (CellDef *) NULL, "cifMakeCell");
 	DBCellSetAvail(CIFTotalDef);
 	CIFTotalDef->cd_flags |= CDINTERNAL;
@@ -124,7 +124,7 @@ CIFInitCells()
     CIFComponentDef = DBCellLookDef("__CIF2__");
     if (CIFComponentDef == (CellDef *) NULL)
     {
-	CIFComponentDef = DBCellNewDef ("__CIF2__",(char *) NULL);
+	CIFComponentDef = DBCellNewDef("__CIF2__");
 	ASSERT(CIFComponentDef != (CellDef *) NULL, "cifMakeCell");
 	DBCellSetAvail(CIFComponentDef);
 	CIFComponentDef->cd_flags |= CDINTERNAL;
@@ -148,7 +148,7 @@ CIFInitCells()
     CIFDummyUse = DBCellNewUse(CIFTotalDef, (char *) NULL);
     DBSetTrans (CIFDummyUse, &GeoIdentityTransform);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -198,7 +198,7 @@ cifHierCleanup()
 
     SigEnableInterrupts();
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -235,10 +235,7 @@ cifHierCopyFunc(tile, cxp)
     Rect sourceRect, targetRect;
     int pNum;
     CellDef *def = (CellDef *) cxp->tc_filter->tf_arg;
-
-#ifdef NONMANHATTAN
     int dinfo = 0;
-#endif
 
     /* Ignore tiles in vendor GDS, unless this is specifically	*/
     /* overridden by the "see-vendor" option.			*/
@@ -253,15 +250,12 @@ cifHierCopyFunc(tile, cxp)
 
     /* Ignore space tiles, since they won't do anything anyway. */
 
-#ifdef NONMANHATTAN
-   
     if (IsSplit(tile))
     {
 	dinfo = DBTransformDiagonal(type, &cxp->tc_scx->scx_trans);
 	type = (SplitSide(tile)) ? SplitRightType(tile) :
 		SplitLeftType(tile);
     }
-#endif
 
     if (type == TT_SPACE) return 0;
 
@@ -274,18 +268,13 @@ cifHierCopyFunc(tile, cxp)
     {
 	if (DBPaintOnPlane(type, pNum))
 	{
-#ifdef NONMANHATTAN
 	    DBNMPaintPlane(def->cd_planes[pNum], dinfo, &targetRect,
 		DBStdPaintTbl(type, pNum), (PaintUndoInfo *) NULL);
-#else
-	    DBPaintPlane(def->cd_planes[pNum], &targetRect,
-	        DBStdPaintTbl(type, pNum), (PaintUndoInfo *) NULL);
-#endif
 	}
     }
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -327,8 +316,8 @@ cifHierCellFunc(scx)
     newscx = *scx;
     GEO_EXPAND(&scx->scx_area, CIFCurStyle->cs_radius, &newscx.scx_area);
     (void) DBTreeSrTiles(&newscx, &CIFCurStyle->cs_yankLayers, 0,
-	cifHierCopyFunc, (ClientData) CIFComponentDef);
-    
+	    cifHierCopyFunc, (ClientData) CIFComponentDef);
+
     /* Set CIFErrorDef to NULL to ignore errors here... these will
      * get reported anyway when the cell is CIF'ed non-hierarchically,
      * so there's no point in making a second report here.
@@ -336,11 +325,11 @@ cifHierCellFunc(scx)
 
     CIFErrorDef = (CellDef *) NULL;
     GeoTransRect(&scx->scx_trans, &scx->scx_area, &rootArea);
-    CIFGen(CIFComponentDef, &rootArea, CIFComponentPlanes,
-	    &CIFCurStyle->cs_hierLayers, FALSE, TRUE);
+    CIFGen(CIFComponentDef, scx->scx_use->cu_def, &rootArea,
+	    CIFComponentPlanes, &CIFCurStyle->cs_hierLayers, FALSE, TRUE);
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -368,7 +357,6 @@ cifHierErrorFunc(tile, checkArea)
 
     TiToRect(tile, &area);
 
-#ifdef NONMANHATTAN
     /* Space in a diagonal tile is not an error if the corner containing
      * space bounds the checkArea.
      */
@@ -376,14 +364,13 @@ cifHierErrorFunc(tile, checkArea)
 	if (((area.r_xbot == checkArea->r_xbot) && !SplitSide(tile)) ||
 		((area.r_xtop == checkArea->r_xtop) && SplitSide(tile)))
 	    return 0;
-#endif
 
     GeoClip(&area, checkArea);
     CIFError(&area, "parent and child disagree on CIF");
     return 0;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -412,8 +399,6 @@ cifHierCheckFunc(tile, plane)
 
     TiToRect(tile, &area);
 
-#ifdef NONMANHATTAN
-
     if (IsSplit(tile))
     {
         DBSrPaintNMArea((Tile *)NULL, plane, TiGetTypeExact(tile),
@@ -429,14 +414,6 @@ cifHierCheckFunc(tile, plane)
         DBPaintPlane(plane, &area, CIFEraseTable, (PaintUndoInfo *) NULL);
     }
 
-#else
-
-    DBSrPaintArea((Tile *) NULL, plane, &area,
-		&DBSpaceBits, cifHierErrorFunc, (ClientData) &area);
-    DBPaintPlane(plane, &area, CIFEraseTable, (PaintUndoInfo *) NULL);
-
-#endif  
-
     CIFTileOps++;
     return 0;
 }
@@ -468,17 +445,15 @@ cifHierPaintFunc(tile, plane)
 
     TiToRect(tile, &area);
     if (CIFCurStyle->cs_flags & CWF_GROW_SLIVERS) cifGrowSliver(tile, &area);
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
 	DBNMPaintPlane(plane, TiGetTypeExact(tile), &area, CIFPaintTable,
 		(PaintUndoInfo *) NULL);
     else
-#endif  
-    DBPaintPlane(plane, &area, CIFPaintTable, (PaintUndoInfo *) NULL);
+	DBPaintPlane(plane, &area, CIFPaintTable, (PaintUndoInfo *) NULL);
     CIFTileOps += 1;
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -507,7 +482,7 @@ cifCheckAndErase(style)
     int i;
 
     /* Process all of the bits of CIF in CIFComponentPlanes. */
-    
+
     for (i=0; i<style->cs_nLayers; i++)
     {
 	CIFErrorLayer = i;
@@ -517,7 +492,7 @@ cifCheckAndErase(style)
 	    (ClientData) CIFTotalPlanes[i]);
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -573,7 +548,7 @@ CIFGenSubcells(def, area, output)
      * out if there are any subcell interactions within one
      * radius of the square.
      */
-    
+
     totalArea = *area;
     GeoClip(&totalArea, &def->cd_bbox);
     for (y = totalArea.r_ybot; y < totalArea.r_ytop; y += stepSize)
@@ -590,48 +565,48 @@ CIFGenSubcells(def, area, output)
 	    GEO_EXPAND(&square, radius, &square);
 	    if (!DRCFindInteractions(def, &square, radius,
 		&interaction)) continue;
-	    
+
 	    /* We've found an interaction.  Flatten it into CIFTotalUse, then
 	     * make CIF from what's flattened.  Yank extra material to
 	     * ensure that CIF is generated correctly.
 	     */
-	    
+
 	    GEO_EXPAND(&interaction, CIFCurStyle->cs_radius, &scx.scx_area);
 	    (void) DBTreeSrTiles(&scx, &CIFCurStyle->cs_yankLayers, 0,
 		cifHierCopyFunc, (ClientData) CIFTotalDef);
 	    CIFErrorDef = def;
-	    CIFGen(CIFTotalDef, &interaction, CIFTotalPlanes,
+	    CIFGen(CIFTotalDef, def, &interaction, CIFTotalPlanes,
 		    &CIFCurStyle->cs_hierLayers, TRUE, TRUE);
 
 	    /* Now go through all the subcells overlapping the area
 	     * and generate CIF for each subcell individually.  OR this
 	     * combined CIF together into CIFComponentPlanes.
 	     */
-	    
+
 	    scx.scx_area = interaction;
 	    (void) DBCellSrArea(&scx, cifHierCellFunc, (ClientData) NULL);
-	    
+
 	    /* Also generate CIF for the paint in the parent alone.  Ignore
 	     * CIF generation errors here, since they will already have been
 	     * recorded when the parent was CIF'ed by itself.
 	     */
 
 	    CIFErrorDef = (CellDef *) NULL;
-	    CIFGen(def, &interaction, CIFComponentPlanes,
+	    CIFGen(def, def, &interaction, CIFComponentPlanes,
 		    &CIFCurStyle->cs_hierLayers, FALSE, TRUE);
 
 	    /* Make sure everything in the pieces is also in the overall
 	     * CIF, then erase the piece stuff from the overall, and
 	     * throw away the pieces.
 	     */
-	    
+
 	    CIFErrorDef = def;
 	    cifCheckAndErase(CIFCurStyle);
 
 	    /* Lastly, paint everything from the pieces into the result
 	     * planes.
 	     */
-	    
+
 	    oldTileOps2 = CIFTileOps;
 	    for (i=0; i<CIFCurStyle->cs_nLayers; i++)
 	    {
@@ -644,11 +619,11 @@ CIFGenSubcells(def, area, output)
 
 	    cifHierCleanup();
 	}
-    
+
     CIFHierTileOps += CIFTileOps - oldTileOps;
     UndoEnable();
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -704,19 +679,19 @@ cifHierElementFunc(use, transform, x, y, checkArea)
 	cifHierCopyFunc, (ClientData) CIFComponentDef);
 
     CIFErrorDef = (CellDef *) NULL;
-    CIFGen(CIFComponentDef, checkArea, CIFComponentPlanes,
+    CIFGen(CIFComponentDef, use->cu_def, checkArea, CIFComponentPlanes,
 	    &CIFCurStyle->cs_hierLayers, FALSE, TRUE);
 
     return 0;
 }
-
+
 
 /*
  * ---------------------------------------------------------------------------
  * cifGrowSliver() --
  *
  * This function is passed the address of a "sliver" rectangle.  It then
- * grows the shortest dimension of the sliver so that it is at least minimum 
+ * grows the shortest dimension of the sliver so that it is at least minimum
  * width.  The minimum width is found through the variable CurCifLayer which
  * identifies the current layer being generated.  A "minwidth" command has been
  * added to the cifoutput section for which the minimum width of each layer
@@ -769,7 +744,7 @@ cifGrowSliver(tile, area)
     return 0;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -819,7 +794,7 @@ cifHierPaintArrayFunc(tile)
     }
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -877,7 +852,7 @@ cifHierArrayFunc(scx, output)
     radius = CIFCurStyle->cs_radius;
     if ((use->cu_xlo == use->cu_xhi) && (use->cu_ylo == use->cu_yhi))
 	return 2;
-    
+
     /* We only want interactions between neighboring cells, so reduce	*/
     /* the array size to at most 2x2, process, then restore the		*/
     /* original array count.						*/
@@ -894,7 +869,7 @@ cifHierArrayFunc(scx, output)
      * of the parent.  If the array is 1-dimensional, we set the
      * corresponding spacing to an impossibly large distance.
      */
-    
+
     childArea.r_xbot = 0;
     childArea.r_ybot = 0;
 
@@ -945,8 +920,8 @@ cifHierArrayFunc(scx, output)
 	(void) DBArraySr(use, &expandedArea, cifHierElementFunc,
 		(ClientData) &A);
 	CIFErrorDef = use->cu_parent;
-	CIFGen(CIFTotalDef, &A, CIFTotalPlanes, &CIFCurStyle->cs_hierLayers,
-		FALSE, TRUE);
+	CIFGen(CIFTotalDef, use->cu_def, &A, CIFTotalPlanes,
+		&CIFCurStyle->cs_hierLayers, FALSE, TRUE);
 	anyInteractions = TRUE;
     }
 
@@ -962,8 +937,8 @@ cifHierArrayFunc(scx, output)
 	(void) DBArraySr(use, &expandedArea, cifHierElementFunc,
 		(ClientData) &C);
 	CIFErrorDef = use->cu_parent;
-	CIFGen(CIFTotalDef, &C, CIFTotalPlanes, &CIFCurStyle->cs_hierLayers,
-		FALSE, TRUE);
+	CIFGen(CIFTotalDef, use->cu_def, &C, CIFTotalPlanes,
+		&CIFCurStyle->cs_hierLayers, FALSE, TRUE);
 	anyInteractions = TRUE;
     }
 
@@ -979,9 +954,9 @@ cifHierArrayFunc(scx, output)
 	(void) DBArraySr(use, &expandedArea, cifHierElementFunc,
 		(ClientData) &B);
 	CIFErrorDef = use->cu_parent;
-	CIFGen(CIFTotalDef, &B, CIFTotalPlanes, &CIFCurStyle->cs_hierLayers,
-		FALSE, TRUE);
-	
+	CIFGen(CIFTotalDef, use->cu_def, &B, CIFTotalPlanes,
+		&CIFCurStyle->cs_hierLayers, FALSE, TRUE);
+
 	/* D */
 
 	D.r_xbot = use->cu_bbox.r_xtop - xsep + radius;
@@ -992,8 +967,8 @@ cifHierArrayFunc(scx, output)
 	(void) DBArraySr(use, &expandedArea, cifHierElementFunc,
 		(ClientData) &D);
 	CIFErrorDef = use->cu_parent;
-	CIFGen(CIFTotalDef, &D, CIFTotalPlanes, &CIFCurStyle->cs_hierLayers,
-		FALSE, TRUE);
+	CIFGen(CIFTotalDef, use->cu_def, &D, CIFTotalPlanes,
+		&CIFCurStyle->cs_hierLayers, FALSE, TRUE);
     }
 
     if (anyInteractions)
@@ -1002,7 +977,7 @@ cifHierArrayFunc(scx, output)
 	/* Remove redundant CIF that's already in the children, and
 	 * make sure everything in the kids is in the parent too.
 	 */
-    
+
 	CIFErrorDef = use->cu_parent;
 	cifCheckAndErase(CIFCurStyle);
 
@@ -1011,7 +986,7 @@ cifHierArrayFunc(scx, output)
 	 * be replicated many times over to cover each of the array
 	 * interaction areas.
 	 */
-    
+
 	oldTileOps = CIFTileOps;
 	for (i=0; i<CIFCurStyle->cs_nLayers; i++)
 	{
@@ -1062,7 +1037,7 @@ cifHierArrayFunc(scx, output)
 		(void) DBSrPaintArea((Tile *) NULL, CIFTotalPlanes[i],
 			&cifArea, &CIFSolidBits, cifHierPaintArrayFunc,
 			(ClientData) NULL);
-	    
+
 		/* The right edge of the array (from D). */
 
 		cifHierXSpacing = 0;
@@ -1073,7 +1048,7 @@ cifHierArrayFunc(scx, output)
 		(void) DBSrPaintArea((Tile *) NULL, CIFTotalPlanes[i],
 			&cifArea, &CIFSolidBits, cifHierPaintArrayFunc,
 			(ClientData) NULL);
-	    
+
 		/* The core of the array (from A and B).  This code is a bit
 		 * tricky in order to work correctly even for arrays where
 		 * radius < ysep.  The "if" statement handles this case.
@@ -1106,7 +1081,7 @@ cifHierArrayFunc(scx, output)
     return 2;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1147,7 +1122,7 @@ CIFGenArrays(def, area, output)
     /* Bill all tile operations here to hierarchical processing in
      * addition to adding to the total.
      */
-    
+
     oldTileOps = CIFTileOps;
 
     if (output == NULL)
