@@ -4,16 +4,16 @@
  * Defines things shared internally by the cif module of Magic,
  * but not generally needed outside the cif module.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  *
  * rcsid "$Header: /usr/cvsroot/magic-8.0/cif/CIFint.h,v 1.3 2008/12/04 17:10:29 tim Exp $"
@@ -46,11 +46,17 @@
 typedef struct bloat_data
 {
     int bl_plane;		/* Plane on which a bloat or squares
-				 * operation is valid.
+				 * operation is valid.  If -1, then the bloat
+				 * types are CIF types.
 				 */
     int bl_distance[TT_MAXTYPES];
 } BloatData;
 
+typedef struct bridge_data
+{
+    int br_width;		/* Minimum width rule for bridge */
+} BridgeData;
+
 typedef struct squares_data
 {
     int sq_border;
@@ -80,8 +86,9 @@ typedef struct cifop
 				 * below for the legal ones.
 				 */
     int co_distance;		/* Grow or shrink distance (if needed). */
-    ClientData co_client;	/* Pointer to a BloatData, SquaresData, or
-				 * SlotsData structure, or NULL.
+    ClientData co_client;	/* Pointer to a BloatData, SquaresData,
+				 * SlotsData, or BridgeData structure,
+				 * or NULL.
 				 */
     struct cifop *co_next;	/* Next in list of operations to perform. */
 } CIFOp;
@@ -133,6 +140,7 @@ typedef struct cifop
  *			which will be painted into parent cells instead of the
  *			current cell.  This replaces the "fault" method.
  * CIFOP_CLOSE -	Added 11/25/19---close up areas smaller than indicated
+ * CIFOP_BRIDGE -	Added 6/11/20---Bridge across catecorner gaps
  */
 
 #define CIFOP_AND	1
@@ -155,6 +163,7 @@ typedef struct cifop
 #define CIFOP_MAXRECT	18
 #define CIFOP_COPYUP	19
 #define CIFOP_CLOSE	20
+#define CIFOP_BRIDGE	21
 
 /* Added by Tim 10/21/2004 */
 /* The following structure is used to pass information on how to draw
@@ -198,7 +207,7 @@ typedef struct
 				 */
     int min_width;		/* the minimum width rule in centi-microns
 				 * for the layer. This is used by Grow Sliver
-				 * to generate drc correct parent slivers	
+				 * to generate drc correct parent slivers
 				 */
 #ifdef THREE_D
     int	   cl_renderStyle;	/* Style to render CIF layer with */
@@ -258,7 +267,7 @@ typedef struct cifstyle
 				 * file more readable).  Default of 1.
 				 * Unused for GDS input/output.
 				 */
-    int cs_expander;		/* cs_scaleFactor / cs_expander = scale in 
+    int cs_expander;		/* cs_scaleFactor / cs_expander = scale in
 				 * centimicrons.  Default of 1.  Value 10
 				 * means cs_scaleFactor is measured in
 				 * nanometers (millimicrons)
