@@ -3,21 +3,21 @@
  *	This file contains global information for the CIF module,
  *	such as performance statistics.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cif/CIFmain.c,v 1.2 2008/12/04 17:10:29 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cif/CIFmain.c,v 1.3 2009/01/15 15:44:34 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -76,7 +76,7 @@ static int cifTotalHierRects = 0;
 global CellDef *CIFErrorDef;	/* Definition in which to record errors. */
 global int CIFErrorLayer;	/* Index of CIF layer associated with errors.*/
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -128,7 +128,7 @@ CIFPrintStats()
  * Results:
  *	Internal units-to-(nanometers * convert) conversion factor (float).
  *	Use convert = 1000 for centimicrons-to-microns conversion
- *  
+ *
  * Side effects:
  *	None.
  *
@@ -212,7 +212,7 @@ CIFPrintStyle(dolist, doforall, docurrent)
     }
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -269,7 +269,7 @@ CIFSetStyle(name)
     TxError("\"%s\" is not one of the CIF output styles Magic knows.\n", name);
     CIFPrintStyle(FALSE, TRUE, TRUE);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -278,21 +278,26 @@ CIFSetStyle(name)
  * 	Finds the CIF planes for a given name.
  *
  * Results:
- *	TRUE if successful, FALSE if "name" failed to match any layers.
+ *	Returns TRUE on success, or FALSE if "name" failed to match any layers.
  *
  * Side effects:
  *	If there's no match, then an error message is output.
  *	The sets 'result' to be all types containing the CIF layer named
- *	"name".  The current CIF style is used for the lookup.
+ *	"name".  The current CIF style is used for the lookup. If "depend"
+ *	is non-NULL, then it is filled with the mask of all layers on which
+ *	the named layer depends.
  * ----------------------------------------------------------------------------
  */
 
 bool
-CIFNameToMask(name, result)
+CIFNameToMask(name, result, depend)
     char *name;
     TileTypeBitMask *result;
+    TileTypeBitMask *depend;
 {
-    int i;
+    int i, j;
+    CIFOp *op;
+    CIFLayer *cl;
 
     if (!CIFCurStyle)
     {
@@ -301,15 +306,48 @@ CIFNameToMask(name, result)
     }
 
     TTMaskZero(result);
-    for (i = 0;  i < CIFCurStyle->cs_nLayers; i+=1)
-    {
+    for (i = 0; i < CIFCurStyle->cs_nLayers; i++)
 	if (strcmp(name, CIFCurStyle->cs_layers[i]->cl_name) == 0)
-	{
 	    TTMaskSetType(result, i);
-	}
-    }
 
-    if (!TTMaskEqual(result, &DBZeroTypeBits)) return (TRUE);
+    if (!TTMaskEqual(result, &DBZeroTypeBits))
+    {
+	/* This loop depends on the fact that templayers must	*/
+	/* be declared in order of dependency;  that is, you	*/
+	/* can't use a layer before it is declared.  Thus, the	*/
+	/* dependent layers will always be filled in from layer	*/
+	/* i back to zero.					*/
+
+	if (depend)
+ 	{
+	    TTMaskZero(depend);
+	    TTMaskSetMask(depend, result);
+	    for (j = CIFCurStyle->cs_nLayers - 1; j >= 0; j--)
+		if (TTMaskHasType(depend, j))
+		{
+		    cl = CIFCurStyle->cs_layers[j];
+		    for (op = cl->cl_ops; op != NULL; op = op->co_next)
+		    {
+			TTMaskSetMask(depend, &op->co_cifMask);
+
+			/* Bloat layers may depend on CIF layers    */
+			/* Currently supported only with bloat-all  */
+
+			if (op->co_opcode == CIFOP_BLOATALL)
+			{
+		            BloatData *bloats = (BloatData *)op->co_client;
+			    TileType ttype;
+
+			    if (bloats->bl_plane < 0)	 /* Use CIF types */
+		                for (ttype = 0; ttype < TT_MAXTYPES; ttype++)
+		                    if (bloats->bl_distance[ttype] > 0)
+					TTMaskSetType(depend, ttype);
+			}
+		    }
+		}
+	}
+	return TRUE;
+    }
 
     TxError("CIF name \"%s\" doesn't exist in style \"%s\".\n", name,
 	CIFCurStyle->cs_name);
@@ -322,9 +360,9 @@ CIFNameToMask(name, result)
 	    TxError(", %s", CIFCurStyle->cs_layers[i]->cl_name);
     }
     TxError(".\n");
-    return (FALSE);
+    return FALSE;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -363,7 +401,7 @@ CIFError(area, message)
     DBWFeedbackAdd(area, msg, CIFErrorDef, CIFCurStyle->cs_scaleFactor,
 	STYLE_PALEHIGHLIGHTS);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
