@@ -3,21 +3,21 @@
  *	This file provides procedures for displaying CIF layers on
  *	the screen using the highlight facilities.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cif/CIFsee.c,v 1.4 2009/01/15 15:44:34 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cif/CIFsee.c,v 1.5 2010/06/24 12:37:15 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -102,13 +102,8 @@ cifPaintDBFunc(tile, pld)
 	if (DBPaintOnPlane(type, pNum))
 	{
 	    ui.pu_pNum = pNum;
-#ifdef NONMANHATTAN
 	    DBNMPaintPlane(paintDef->cd_planes[pNum], TiGetTypeExact(tile),
 		    &area, DBStdPaintTbl(type, pNum), (PaintUndoInfo *) &ui);
-#else
-	    DBPaintPlane(paintDef->cd_planes[pNum], &area,
-		    DBStdPaintTbl(type, pNum), (PaintUndoInfo *) &ui);
-#endif
 	}
 
     return  0;		/* To keep the search alive. */
@@ -168,7 +163,8 @@ CIFPaintLayer(rootDef, area, cifLayer, magicLayer, paintDef)
     (void) DBTreeSrTiles(&scx, &DBAllButSpaceAndDRCBits, 0,
 	cifHierCopyFunc, (ClientData) CIFComponentDef);
     oldCount = DBWFeedbackCount;
-    CIFGen(CIFComponentDef, area, CIFPlanes, &depend, TRUE, TRUE);
+
+    CIFGen(CIFComponentDef, rootDef, area, CIFPlanes, &depend, TRUE, TRUE);
     DBCellClearDef(CIFComponentDef);
 
     /* Report any errors that occurred. */
@@ -229,13 +225,9 @@ cifSeeFunc(tile, sld)
     }
 
     DBWFeedbackAdd(&area, sld->text, cifSeeDef, CIFCurStyle->cs_scaleFactor,
-#ifdef NONMANHATTAN 
 	sld->style |
                 (TiGetTypeExact(tile) & (TT_DIAGONAL | TT_DIRECTION | TT_SIDE)));
         /* (preserve information about the geometry of a diagonal tile) */
-#else
-	sld->style);
-#endif
     return 0;
 }
 
@@ -286,7 +278,7 @@ CIFSeeLayer(rootDef, area, layer)
     (void) DBTreeSrTiles(&scx, &DBAllButSpaceAndDRCBits, 0,
 	cifHierCopyFunc, (ClientData) CIFComponentDef);
     oldCount = DBWFeedbackCount;
-    CIFGen(CIFComponentDef, area, CIFPlanes, &depend, TRUE, TRUE);
+    CIFGen(CIFComponentDef, rootDef, area, CIFPlanes, &depend, TRUE, TRUE);
     DBCellClearDef(CIFComponentDef);
 
     /* Report any errors that occurred. */
@@ -320,7 +312,7 @@ CIFSeeLayer(rootDef, area, layer)
     }
     UndoEnable();
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -367,7 +359,7 @@ CIFSeeHierLayer(rootDef, area, layer, arrays, subcells)
 	CIFGenSubcells(rootDef, area, CIFPlanes);
     if (arrays)
 	CIFGenArrays(rootDef, area, CIFPlanes);
-    
+
     /* Report any errors that occurred. */
 
     if (DBWFeedbackCount != oldCount)
@@ -375,7 +367,7 @@ CIFSeeHierLayer(rootDef, area, layer, arrays, subcells)
 	TxPrintf("%d problems occurred.  See feedback entries.\n",
 	    DBWFeedbackCount - oldCount);
     }
-    
+
     (void) sprintf(msg, "CIF layer \"%s\"", layer);
     cifSeeDef = rootDef;
     sld.text = msg;
@@ -449,9 +441,9 @@ CIFCoverageLayer(rootDef, area, layer)
     scx.scx_trans = GeoIdentityTransform;
     (void) DBTreeSrTiles(&scx, &DBAllButSpaceAndDRCBits, 0,
 	cifHierCopyFunc, (ClientData) CIFComponentDef);
-    CIFGen(CIFComponentDef, area, CIFPlanes, &depend, TRUE, TRUE);
+    CIFGen(CIFComponentDef, rootDef, area, CIFPlanes, &depend, TRUE, TRUE);
     DBCellClearDef(CIFComponentDef);
-    
+
     cstats.coverage = 0;
     cstats.bounds.r_xbot = cstats.bounds.r_xtop = 0;
     cstats.bounds.r_ybot = cstats.bounds.r_ytop = 0;
