@@ -4,16 +4,16 @@
  *	files pertaining to CIF, and builds the tables
  *	used by the CIF generator.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -188,7 +188,7 @@ cifTechStyleInit()
 	CIFCurStyle->cs_layers[i] = NULL;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -210,7 +210,7 @@ cifTechStyleInit()
  */
 
 void
-cifParseLayers(string, style, paintMask, cifMask,spaceOK)
+cifParseLayers(string, style, paintMask, cifMask, spaceOK)
     char *string;		/* List of layers. */
     CIFStyle *style;		/* Gives CIF style for parsing string.*/
     TileTypeBitMask *paintMask;	/* Place to store mask of paint layers.  If
@@ -284,7 +284,7 @@ okpaint:
 	/* Make sure that there's exactly one match among cif and
 	 * paint layers together.
 	 */
-	
+
 	if ((paintType == -1)
 	    || ((paintType >= 0) && !TTMaskEqual(&curCifMask, &DBZeroTypeBits)))
 	{
@@ -310,7 +310,7 @@ okpaint:
 			if (TTMaskHasType(rMask, paintType))
 			    TTMaskSetType(paintMask, rtype);
 		    }
-	    } 
+	    }
 	}
 	else if (!TTMaskEqual(&curCifMask, &DBZeroTypeBits))
 	{
@@ -332,7 +332,7 @@ okpaint:
 	}
     }
 }
-
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -498,7 +498,7 @@ CIFParseScale(true_scale, expander)
     }
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -524,13 +524,14 @@ CIFTechLine(sectionName, argc, argv)
     int argc;			/* Number of fields on line. */
     char *argv[];		/* Values of fields. */
 {
-    TileTypeBitMask mask, tempMask, bloatLayers;
+    TileTypeBitMask mask, tempMask, cifMask, bloatLayers;
     int i, j, l, distance;
     CIFLayer *newLayer;
     CIFOp *newOp = NULL;
     CIFKeep *newStyle, *p;
     char **bloatArg;
     BloatData *bloats;
+    BridgeData *bridge;
     SquaresData *squares;
     SlotsData *slots;
 
@@ -541,7 +542,7 @@ CIFTechLine(sectionName, argc, argv)
      * sure that the current (maybe default?) CIF style has
      * a name.
      */
-    
+
     if (strcmp(argv[0], "style") == 0)
     {
 	if (argc != 2)
@@ -557,7 +558,7 @@ CIFTechLine(sectionName, argc, argv)
 		return TRUE;
 	    }
 	}
-	for (newStyle = CIFStyleList; newStyle != NULL; 
+	for (newStyle = CIFStyleList; newStyle != NULL;
 		newStyle = newStyle->cs_next)
 	{
 	    /* Here we're only establishing existence;		*/
@@ -614,7 +615,7 @@ CIFTechLine(sectionName, argc, argv)
 			for (p = CIFStyleList; p->cs_next; p = p->cs_next);
 			p->cs_next = newStyle;
 		    }
-		    
+
 		    if (cptr == NULL)
 			break;
 		    else
@@ -769,7 +770,7 @@ CIFTechLine(sectionName, argc, argv)
 	while (*tptr != '\0')
 	{
 	    cptr = strchr(tptr, ',');
-	    if (cptr != NULL) 
+	    if (cptr != NULL)
 	    {
 		*cptr = '\0';
 		for (j = 1; isspace(*(cptr - j)); j++)
@@ -790,7 +791,7 @@ CIFTechLine(sectionName, argc, argv)
 		    return TRUE;
 		}
 	    }
-	
+
 	    if (cptr == NULL)
 		break;
 	    else
@@ -843,15 +844,15 @@ CIFTechLine(sectionName, argc, argv)
 	cifGotLabels = FALSE;
 
 	/* Handle a special case of a list of layer names on the layer
-	 * line.  Turn them into an OR operation. 
+	 * line.  Turn them into an OR operation.
 	 */
-	
+
 	if (argc == 3)
 	{
 	    cifCurOp = (CIFOp *) mallocMagic(sizeof(CIFOp));
 	    cifCurOp->co_opcode = CIFOP_OR;
 	    cifParseLayers(argv[2], CIFCurStyle, &cifCurOp->co_paintMask,
-		&cifCurOp->co_cifMask,FALSE);
+		&cifCurOp->co_cifMask, FALSE);
 	    cifCurOp->co_distance = 0;
 	    cifCurOp->co_next = NULL;
 	    cifCurOp->co_client = (ClientData)NULL;
@@ -976,7 +977,7 @@ CIFTechLine(sectionName, argc, argv)
         return TRUE;
     }
 
-    /* 
+    /*
      * miscellaneous cif/calma-writing boolean options
      */
 
@@ -1003,7 +1004,7 @@ CIFTechLine(sectionName, argc, argv)
     /* Anything below here is a geometric operation, so we can
      * do some set-up that is common to all the operations.
      */
-    
+
     if (cifCurLayer == NULL)
     {
 	TechError("Must define layer before specifying operation.\n");
@@ -1055,6 +1056,8 @@ CIFTechLine(sectionName, argc, argv)
 	newOp->co_opcode = CIFOP_BOUNDARY;
     else if (strcmp(argv[0], "close") == 0)
 	newOp->co_opcode = CIFOP_CLOSE;
+    else if (strcmp(argv[0], "bridge") == 0)
+	newOp->co_opcode = CIFOP_BRIDGE;
     else
     {
 	TechError("Unknown statement \"%s\".\n", argv[0]);
@@ -1070,7 +1073,7 @@ CIFTechLine(sectionName, argc, argv)
 	    cifParseLayers(argv[1], CIFCurStyle, &newOp->co_paintMask,
 		&newOp->co_cifMask,FALSE);
 	    break;
-	
+
 	case CIFOP_GROW:
 	case CIFOP_GROWMIN:
 	case CIFOP_GROW_G:
@@ -1084,7 +1087,26 @@ CIFTechLine(sectionName, argc, argv)
 		goto errorReturn;
 	    }
 	    break;
-	
+
+	case CIFOP_BRIDGE:
+	    if (argc != 3) goto wrongNumArgs;
+	    newOp->co_distance = atoi(argv[1]);
+	    if (newOp->co_distance <= 0)
+	    {
+		TechError("Bridge distance must be greater than zero.\n");
+		goto errorReturn;
+	    }
+	    bridge = (BridgeData *)mallocMagic(sizeof(BridgeData));
+	    bridge->br_width = atoi(argv[2]);
+	    if (bridge->br_width <= 0)
+	    {
+		TechError("Bridge width must be greater than zero.\n");
+		freeMagic(bridge);
+		goto errorReturn;
+	    }
+	    newOp->co_client = (ClientData)bridge;
+	    break;
+
 	case CIFOP_BLOATALL:
 	    if (argc != 3) goto wrongNumArgs;
 	    cifParseLayers(argv[1], CIFCurStyle, &newOp->co_paintMask,
@@ -1093,28 +1115,46 @@ CIFTechLine(sectionName, argc, argv)
 	    for (i = 0; i < TT_MAXTYPES; i++)
 		bloats->bl_distance[i] = 0;
 	    newOp->co_client = (ClientData)bloats;
-	    cifParseLayers(argv[2], CIFCurStyle, &mask, &tempMask, TRUE);
+	    cifParseLayers(argv[2], CIFCurStyle, &mask, &cifMask, TRUE);
+
+	    /* 5/25/2020:  Lifting restriction that bloatLayers types	*/
+	    /* cannot be CIF or temp layers.  However, CIF/temp layers	*/
+	    /* and magic database layers may not be mixed.		*/
+
+	    if (!TTMaskIsZero(&mask) && !TTMaskIsZero(&cifMask))
+		TechError("Can't mix CIF and magic layers in bloat statement.\n");
 
 	    /* 10/15/2019:  Lifting restriction that the types that	*/
 	    /* trigger the bloating must be in the same plane as the	*/
 	    /* types that are bloated into.				*/
 
 	    TTMaskZero(&bloatLayers);
-	    TTMaskSetMask(&bloatLayers, &mask);
-	    if (!TTMaskEqual(&tempMask, &DBZeroTypeBits))
-		TechError("Can't use templayers in bloat statement.\n");
+	    if (!TTMaskIsZero(&mask))
+	    {
+		TTMaskSetMask(&bloatLayers, &mask);
+		for (i = 0; i < TT_MAXTYPES;  i++)
+		    if (TTMaskHasType(&mask, i))
+			bloats->bl_distance[i] = 1;
+
+		goto bloatCheck;
+	    }
+	    else
+	    {
+		TTMaskSetMask(&bloatLayers, &cifMask);
+		for (i = 0; i < TT_MAXTYPES;  i++)
+		    if (TTMaskHasType(&cifMask, i))
+			bloats->bl_distance[i] = 1;
+
+		bloats->bl_plane = -1;	/* Indicates CIF types */
+	    }
+	    break;
 
-	    for (i = 0; i < TT_MAXTYPES;  i++)
-		if (TTMaskHasType(&mask, i))
-		    bloats->bl_distance[i] = 1;
-	    goto bloatCheck;
-		
 	case CIFOP_BLOAT:
 	case CIFOP_BLOATMIN:
 	case CIFOP_BLOATMAX:
 	    if (argc < 4) goto wrongNumArgs;
 	    cifParseLayers(argv[1], CIFCurStyle, &newOp->co_paintMask,
-		(TileTypeBitMask *) NULL,FALSE);
+		    (TileTypeBitMask *)NULL, FALSE);
 	    argc -= 2;
 	    bloatArg = argv + 2;
 	    bloatLayers = newOp->co_paintMask;
@@ -1133,12 +1173,12 @@ CIFTechLine(sectionName, argc, argv)
 		}
 		else
 		{
-		    cifParseLayers(*bloatArg, CIFCurStyle, &mask, &tempMask,TRUE);
+		    cifParseLayers(*bloatArg, CIFCurStyle, &mask, &tempMask, TRUE);
 		    TTMaskSetMask(&bloatLayers, &mask);
 		}
 		if (!TTMaskEqual(&tempMask, &DBZeroTypeBits))
 		    TechError("Can't use templayers in bloat statement.\n");
-		
+
 		distance = atoi(bloatArg[1]);
 		if ((distance < 0) && (newOp->co_opcode == CIFOP_BLOAT))
 		{
@@ -1165,7 +1205,7 @@ bloatCheck:
 	    /* Make sure that all the layers specified in the statement
 	     * fall in a single plane.
 	     */
-	
+
 	    for (i = 0; i < PL_MAXTYPES; i++)
 	    {
 		tempMask = bloatLayers;
@@ -1218,7 +1258,7 @@ bloatCheck:
 
 	case CIFOP_SQUARES_G:
 	case CIFOP_SQUARES:
-	
+
 	    squares = (SquaresData *)mallocMagic(sizeof(SquaresData));
 	    newOp->co_client = (ClientData)squares;
 
@@ -1303,7 +1343,7 @@ bloatCheck:
 	    break;
 
 	case CIFOP_SLOTS:
-	
+
 	    slots = (SlotsData *)mallocMagic(sizeof(SlotsData));
 	    newOp->co_client = (ClientData)slots;
 
@@ -1434,13 +1474,13 @@ cifCheckCalmaNum(str)
 
     while (*str) {
 	char ch = *str++;
-	if (ch < '0' || ch > '9') 
+	if (ch < '0' || ch > '9')
 	    return (FALSE);
     }
 
     return (TRUE);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1478,7 +1518,7 @@ cifComputeRadii(layer, des)
 	/* If CIF layers are used, switch to the max of current
 	 * distances and those of the layers used.
 	 */
-	
+
 	if (!TTMaskEqual(&op->co_cifMask, &DBZeroTypeBits))
 	{
 	    for (i=0; i < des->cs_nLayers; i++)
@@ -1508,11 +1548,11 @@ cifComputeRadii(layer, des)
 	    case CIFOP_GROW_G:
 		grow += op->co_distance;
 		break;
-	    
+
 	    case CIFOP_SHRINK:
 		shrink += op->co_distance;
 		break;
-	    
+
 	    /* For bloats use the largest distances (negative values are
 	     * for shrinks).
 	     */
@@ -1530,7 +1570,8 @@ cifComputeRadii(layer, des)
 		grow += curGrow;
 		shrink += curShrink;
 		break;
-	    
+
+	    case CIFOP_BRIDGE: break;
 	    case CIFOP_SQUARES: break;
 	    case CIFOP_SQUARES_G: break;
 	}
@@ -1582,8 +1623,8 @@ cifComputeHalo(style)
      *  style->cs_name, style->cs_radius);
      */
 }
-	
-
+
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1637,7 +1678,7 @@ CIFTechFinal()
     /* Allow the case where there's no CIF at all.  This is indicated
      * by a NULL CIFCurStyle.
      */
-    
+
     if (!style) return;
 
     if ((cifCurLayer != NULL) && (cifCurOp == NULL) && !cifGotLabels)
@@ -1699,6 +1740,7 @@ CIFTechFinal()
 	    if (op->co_client)
 	    {
 		BloatData *bloats;
+		BridgeData *bridge;
 		SquaresData *squares;
 		SlotsData *slots;
 		if (op->co_opcode == CIFOP_SLOTS)
@@ -1789,6 +1831,12 @@ CIFTechFinal()
 			case CIFOP_MAXRECT:
 			case CIFOP_NET:
 			    break;
+			case CIFOP_BRIDGE:
+			    bridge = (BridgeData *)op->co_client;
+			    c = FindGCF(style->cs_scaleFactor,
+						bridge->br_width);
+			    if (c < minReduce) minReduce = c;
+			    break;
 			default:
 			    bloats = (BloatData *)op->co_client;
 			    for (j = 0; j < TT_MAXTYPES; j++)
@@ -1854,12 +1902,24 @@ CIFTechFinal()
 		case CIFOP_BLOAT:
 		case CIFOP_BLOATMAX:
 		case CIFOP_BLOATMIN:
+		    bloats = (BloatData *)op->co_client;
+		    for (j = 0; j < TT_MAXTYPES; j++)
+			if (bloats->bl_distance[j] != bloats->bl_distance[TT_SPACE])
+			    TTMaskSetType(&ourYank, j);
+		    needThisLayer = TRUE;
+		    break;
+
 		case CIFOP_BLOATALL:
 		    bloats = (BloatData *)op->co_client;
 		    for (j = 0; j < TT_MAXTYPES; j++)
 		    {
-			if (bloats->bl_distance[j] != bloats->bl_distance[TT_SPACE])
-			    TTMaskSetType(&ourYank, j);
+			if (bloats->bl_distance[j] != 0)
+			{
+			    if (bloats->bl_plane < 0)
+				TTMaskSetType(&ourDepend, j);
+			    else
+				TTMaskSetType(&ourYank, j);
+			}
 		    }
 		    needThisLayer = TRUE;
 		    break;
@@ -1867,6 +1927,8 @@ CIFTechFinal()
 		case CIFOP_AND:
 		case CIFOP_ANDNOT:
 		case CIFOP_SHRINK:
+		case CIFOP_CLOSE:
+		case CIFOP_BRIDGE:
 		    needThisLayer = TRUE;
 		    break;
 	    }
@@ -1898,6 +1960,27 @@ CIFTechFinal()
 	    TTMaskSetType(&style->cs_hierLayers, i);
     }
 
+    /* Added by Tim, 6/17/20:  Do not set dependencies of BOUNDARY  */
+    /* or BBOX because these are unable to be represented in a	    */
+    /* hierarchy without producing conflicting parent/child mask    */
+    /* geometry.  Generally, BOUNDARY and BBOX operations are	    */
+    /* intended to be restricted to the child cell and should not   */
+    /* interact hierarchically.  Possibly it would be useful to	    */
+    /* provide some method to preserve the bounding box information */
+    /* when flattening?						    */
+
+    for (i = 0; i < style->cs_nLayers; i++)
+    {
+	for (op = style->cs_layers[i]->cl_ops; op != NULL; op = op->co_next)
+	{
+	    if (op->co_opcode == CIFOP_BOUNDARY || op->co_opcode == CIFOP_BBOX)
+	    {
+		TTMaskClearType(&style->cs_hierLayers, i);
+		break;
+	    }
+	}
+    }
+
     /* Added by Tim, 10/18/04					*/
 
     /* Go through the layer operators looking for those that	*/
@@ -2088,7 +2171,7 @@ CIFGetContactSize(type, edge, spacing, border)
  *   represented by integers.
  *
  *   Note that because contacts may be placed at 1/2 grid spacing to
- *   center them, the size and spacing of contacts as given in the 
+ *   center them, the size and spacing of contacts as given in the
  *   "squares" function must *always* be an even number.  To ensure this,
  *   we check for odd numbers in these positions.  If there are any, and
  *   "d" is also an odd number, then we multiply both "n" and "d" by 2.
@@ -2117,7 +2200,8 @@ CIFTechOutputScale(n, d)
     SquaresData *squares;
     SlotsData *slots;
     BloatData *bloats;
-    bool has_odd_space;
+    BridgeData *bridge;
+    bool has_odd_space = FALSE;
 
     if (ostyle == NULL) return;
 
@@ -2271,6 +2355,13 @@ CIFTechOutputScale(n, d)
 			case CIFOP_MAXRECT:
 			case CIFOP_NET:
 			    break;
+			case CIFOP_BRIDGE:
+			    bridge = (BridgeData *)op->co_client;
+			    bridge->br_width *= d;
+		            lgcf = FindGCF(abs(bridge->br_width),
+					ostyle->cs_expander);
+			    lexpand = FindGCF(lexpand, lgcf);
+			    break;
 			default:
 			    bloats = (BloatData *)op->co_client;
 			    for (j = 0; j < TT_MAXTYPES; j++)
@@ -2368,6 +2459,10 @@ CIFTechOutputScale(n, d)
 			    if (bloats->bl_distance[j] != 0)
 				bloats->bl_distance[j] /= lexpand;
 			break;
+		    case CIFOP_BRIDGE:
+			bridge = (BridgeData *)op->co_client;
+			bridge->br_width /= lexpand;
+			break;
 		    default:
 			/* op->co_opcode in CIFOP_OR is a pointer copy	*/
 			/* and in CIFOP_BBOX and CIFOP_MAXRECT is a	*/
