@@ -4,16 +4,16 @@
  * This procedure defines things that are exported by the
  * cif module to the rest of the world.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  *
  *
@@ -30,6 +30,10 @@
 				 * depends on the size of the layout.
 				 */
 
+/* Passed to CIFPaintCurrent() for print statement formatting */
+#define FILE_CIF    0
+#define FILE_CALMA  1
+
 /* Exported global variables (commands/CmdCD.c) */
 
 extern int  CIFWarningLevel;
@@ -83,6 +87,6 @@ extern void CIFPrintReadStyle();
 extern int CIFOutputScaleFactor();
 
 extern void PaintWireList();
-extern void PaintPolygon();
+extern LinkedRect *PaintPolygon();
 
 #endif /* _CIF_H */
