@@ -3,16 +3,16 @@
  *
  * Commands with names beginning with the letters A through B.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -44,7 +44,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include "utils/netlist.h"
 #include "select/select.h"
 
-
+
 /* ---------------------------------------------------------------------------
  *
  * CmdAddPath --
@@ -80,7 +80,7 @@ CmdAddPath( w, cmd )
     }
     PaAppend(&Path, cmd->tx_argv[1]);
 }
-
+
 
 /* Linked-list structure for returning information about arrayed cells */
 
@@ -110,6 +110,8 @@ typedef struct LA1
  *	array position [x y]
  *	array help
  *
+ *	array -list [count | width | height | pitch [x y] | position [x y]]
+ *
  * Results:
  *	None.
  *
@@ -143,21 +145,38 @@ CmdArray(w, cmd)
     };
 
     char **msg;
-    int option;
+    int option, locargc, argstart;
+    bool doList = FALSE;
     ArrayInfo a;
     Rect toolRect;
     LinkedArray *lahead = NULL, *la;
     int xval, yval;
 
+#ifdef MAGIC_WRAPPER
+    Tcl_Obj *tobj;
+#endif
+
     extern int selGetArrayFunc();
 
-    if (cmd->tx_argc == 1)
+    locargc = cmd->tx_argc;
+    argstart = 1;
+
+    if (locargc <= 1)
 	goto badusage;
     else
     {
-	option = Lookup(cmd->tx_argv[1], cmdArrayOption);
+	if (!strncmp(cmd->tx_argv[argstart], "-list", 5))
+	{
+	    doList = TRUE;
+	    locargc--;
+	    argstart++;
+	}
+        if (locargc <= 1)
+	    goto badusage;	/* Prohibits "array -list" alone */
+
+	option = Lookup(cmd->tx_argv[argstart], cmdArrayOption);
 	if (option < 0) {
-	    if (cmd->tx_argc == 3 || cmd->tx_argc == 5)
+	    if (locargc == 3 || locargc == 5)
 		option = ARRAY_DEFAULT;
 	    else
 		goto badusage;
@@ -167,7 +186,7 @@ CmdArray(w, cmd)
 
     /* Get all information about cell uses in the current selection */
 
-    (void) SelEnumCells(FALSE, (bool *) NULL, (SearchContext *) NULL, 
+    (void) SelEnumCells(FALSE, (bool *) NULL, (SearchContext *) NULL,
 		selGetArrayFunc, (ClientData) &lahead);
 
     /* Note:  All "unimplemented functions" below will require a routine
@@ -181,44 +200,65 @@ CmdArray(w, cmd)
     switch (option)
     {
 	case ARRAY_COUNT:
-	    if (cmd->tx_argc == 2)
+	    if (locargc == 2)
 	    {
 		for (la = lahead; la != NULL; la = la->ar_next)
 		{
-		    if (la->cellUse->cu_id != NULL)
-			TxPrintf("Cell use \"%s\":", la->cellUse->cu_id);
+#ifdef MAGIC_WRAPPER
+		    if (doList)
+		    {
+			tobj = Tcl_NewListObj(0, NULL);
+			Tcl_ListObjAppendElement(magicinterp, tobj,
+				Tcl_NewIntObj(la->arrayInfo.ar_xlo));
+			Tcl_ListObjAppendElement(magicinterp, tobj,
+				Tcl_NewIntObj(la->arrayInfo.ar_xhi));
+			Tcl_ListObjAppendElement(magicinterp, tobj,
+				Tcl_NewIntObj(la->arrayInfo.ar_ylo));
+			Tcl_ListObjAppendElement(magicinterp, tobj,
+				Tcl_NewIntObj(la->arrayInfo.ar_yhi));
+			Tcl_SetObjResult(magicinterp, tobj);
+		    }
 		    else
-			TxPrintf("Cell \"%s\":", la->cellUse->cu_def->cd_name);
-		    TxPrintf("x index %d to %d, y index %d to %d\n",
+		    {
+#endif
+			if (la->cellUse->cu_id != NULL)
+			    TxPrintf("Cell use \"%s\":", la->cellUse->cu_id);
+			else
+			    TxPrintf("Cell \"%s\":", la->cellUse->cu_def->cd_name);
+			TxPrintf("x index %d to %d, y index %d to %d\n",
 				la->arrayInfo.ar_xlo,
 				la->arrayInfo.ar_xhi,
 				la->arrayInfo.ar_ylo,
 				la->arrayInfo.ar_yhi);
+#ifdef MAGIC_WRAPPER
+		    }
+#endif
 		}
 		break;
 	    }
-	    else if ((cmd->tx_argc != 4) && (cmd->tx_argc != 6))
+	    else if ((locargc != 4) && (locargc != 6))
 		goto badusage;
 
-	    if (!StrIsInt(cmd->tx_argv[2]) || !StrIsInt(cmd->tx_argv[3])) 
+	    if (!StrIsInt(cmd->tx_argv[argstart + 1])
+			|| !StrIsInt(cmd->tx_argv[argstart + 2]))
 		goto badusage;
 
-	    if (cmd->tx_argc == 4)
+	    if (locargc == 4)
 	    {
 		a.ar_xlo = 0;
 		a.ar_ylo = 0;
-		a.ar_xhi = atoi(cmd->tx_argv[2]) - 1;
-		a.ar_yhi = atoi(cmd->tx_argv[3]) - 1;
+		a.ar_xhi = atoi(cmd->tx_argv[argstart + 1]) - 1;
+		a.ar_yhi = atoi(cmd->tx_argv[argstart + 2]) - 1;
 		if ( (a.ar_xhi < 0) || (a.ar_yhi < 0) ) goto badusage;
 	    }
-	    else if (cmd->tx_argc == 6)
+	    else if (locargc == 6)
 	    {
-		if (!StrIsInt(cmd->tx_argv[4]) || 
-			!StrIsInt(cmd->tx_argv[5])) goto badusage;
-		a.ar_xlo = atoi(cmd->tx_argv[2]);
-		a.ar_xhi = atoi(cmd->tx_argv[3]);
-		a.ar_ylo = atoi(cmd->tx_argv[4]);
-		a.ar_yhi = atoi(cmd->tx_argv[5]);
+		if (!StrIsInt(cmd->tx_argv[argstart + 3]) ||
+			!StrIsInt(cmd->tx_argv[argstart + 4])) goto badusage;
+		a.ar_xlo = atoi(cmd->tx_argv[argstart + 1]);
+		a.ar_xhi = atoi(cmd->tx_argv[argstart + 2]);
+		a.ar_ylo = atoi(cmd->tx_argv[argstart + 3]);
+		a.ar_yhi = atoi(cmd->tx_argv[argstart + 4]);
 	    }
 
 	    if (!ToolGetBox((CellDef **) NULL, &toolRect))
@@ -232,117 +272,178 @@ CmdArray(w, cmd)
 	    break;
 
 	case ARRAY_WIDTH:
-	    if (cmd->tx_argc == 2)
+	    if (locargc == 2)
 	    {
 		for (la = lahead; la != NULL; la = la->ar_next)
 		{
-		    if (la->cellUse->cu_id != NULL)
-			TxPrintf("Cell use \"%s\":", la->cellUse->cu_id);
+#ifdef MAGIC_WRAPPER
+		    if (doList)
+		    {
+			tobj = Tcl_NewListObj(0, NULL);
+			Tcl_ListObjAppendElement(magicinterp, tobj,
+				Tcl_NewIntObj(la->arrayInfo.ar_xsep));
+			Tcl_SetObjResult(magicinterp, tobj);
+		    }
 		    else
-			TxPrintf("Cell \"%s\":", la->cellUse->cu_def->cd_name);
-		    TxPrintf("x separation %d\n", la->arrayInfo.ar_xsep);
+		    {
+#endif
+			if (la->cellUse->cu_id != NULL)
+			    TxPrintf("Cell use \"%s\":", la->cellUse->cu_id);
+			else
+			    TxPrintf("Cell \"%s\":", la->cellUse->cu_def->cd_name);
+			TxPrintf("x separation %d\n", la->arrayInfo.ar_xsep);
+#ifdef MAGIC_WRAPPER
+		    }
+#endif
 		}
 		break;
 	    }
-	    if ((cmd->tx_argc != 3) || (!StrIsInt(cmd->tx_argv[2])))
+	    if ((locargc != 3) || (!StrIsInt(cmd->tx_argv[argstart + 1])))
 		goto badusage;
 
-	    xval = atoi(cmd->tx_argv[2]);
-	    yval = atoi(cmd->tx_argv[3]);
+	    xval = atoi(cmd->tx_argv[argstart + 1]);
+	    yval = atoi(cmd->tx_argv[argstart + 2]);
 
 	    TxPrintf("Unimplemented function.\n");
 	    break;
 
 	case ARRAY_HEIGHT:
-	    if (cmd->tx_argc == 2)
+	    if (locargc == 2)
 	    {
 		for (la = lahead; la != NULL; la = la->ar_next)
 		{
-		    if (la->cellUse->cu_id != NULL)
-			TxPrintf("Cell use \"%s\":", la->cellUse->cu_id);
+#ifdef MAGIC_WRAPPER
+		    if (doList)
+		    {
+			tobj = Tcl_NewListObj(0, NULL);
+			Tcl_ListObjAppendElement(magicinterp, tobj,
+				Tcl_NewIntObj(la->arrayInfo.ar_ysep));
+			Tcl_SetObjResult(magicinterp, tobj);
+		    }
 		    else
-			TxPrintf("Cell \"%s\":", la->cellUse->cu_def->cd_name);
-		    TxPrintf("y separation %d\n", la->arrayInfo.ar_ysep);
+		    {
+#endif
+			if (la->cellUse->cu_id != NULL)
+			    TxPrintf("Cell use \"%s\":", la->cellUse->cu_id);
+			else
+			    TxPrintf("Cell \"%s\":", la->cellUse->cu_def->cd_name);
+			TxPrintf("y separation %d\n", la->arrayInfo.ar_ysep);
+#ifdef MAGIC_WRAPPER
+		    }
+#endif
 		}
 		break;
 	    }
-	    if ((cmd->tx_argc != 3) || (!StrIsInt(cmd->tx_argv[2])))
+	    if ((locargc != 3) || (!StrIsInt(cmd->tx_argv[argstart + 1])))
 		goto badusage;
 
-	    xval = atoi(cmd->tx_argv[2]);
-	    yval = atoi(cmd->tx_argv[3]);
+	    xval = atoi(cmd->tx_argv[argstart + 1]);
+	    yval = atoi(cmd->tx_argv[argstart + 2]);
 
 	    TxPrintf("Unimplemented function.\n");
 	    break;
 
 	case ARRAY_PITCH:
-	    if (cmd->tx_argc == 2)
+	    if (locargc == 2)
 	    {
 		for (la = lahead; la != NULL; la = la->ar_next)
 		{
-		    if (la->cellUse->cu_id != NULL)
-			TxPrintf("Cell use \"%s\":", la->cellUse->cu_id);
+#ifdef MAGIC_WRAPPER
+		    if (doList)
+		    {
+			tobj = Tcl_NewListObj(0, NULL);
+			Tcl_ListObjAppendElement(magicinterp, tobj,
+				Tcl_NewIntObj(la->arrayInfo.ar_xsep));
+			Tcl_ListObjAppendElement(magicinterp, tobj,
+				Tcl_NewIntObj(la->arrayInfo.ar_ysep));
+			Tcl_SetObjResult(magicinterp, tobj);
+		    }
 		    else
-			TxPrintf("Cell \"%s\":", la->cellUse->cu_def->cd_name);
-		    TxPrintf("x separation %d ", la->arrayInfo.ar_xsep);
-		    TxPrintf("y separation %d\n", la->arrayInfo.ar_ysep);
+		    {
+#endif
+			if (la->cellUse->cu_id != NULL)
+			    TxPrintf("Cell use \"%s\":", la->cellUse->cu_id);
+			else
+			    TxPrintf("Cell \"%s\":", la->cellUse->cu_def->cd_name);
+			TxPrintf("x separation %d ", la->arrayInfo.ar_xsep);
+			TxPrintf("y separation %d\n", la->arrayInfo.ar_ysep);
+#ifdef MAGIC_WRAPPER
+		    }
+#endif
 		}
 		break;
 	    }
-	    if ((cmd->tx_argc != 4) || (!StrIsInt(cmd->tx_argv[2])) ||
-				(!StrIsInt(cmd->tx_argv[3])))
+	    if ((locargc != 4) || (!StrIsInt(cmd->tx_argv[argstart + 1])) ||
+				(!StrIsInt(cmd->tx_argv[argstart + 2])))
 		goto badusage;
 
-	    xval = atoi(cmd->tx_argv[2]);
-	    yval = atoi(cmd->tx_argv[3]);
+	    xval = atoi(cmd->tx_argv[argstart + 1]);
+	    yval = atoi(cmd->tx_argv[argstart + 2]);
 
 	    TxPrintf("Unimplemented function.\n");
 	    break;
 
 	case ARRAY_POSITION:
-	    if (cmd->tx_argc == 2)
+	    if (locargc == 2)
 	    {
 		for (la = lahead; la != NULL; la = la->ar_next)
 		{
-		    if (la->cellUse->cu_id != NULL)
-			TxPrintf("Cell use \"%s\":", la->cellUse->cu_id);
+#ifdef MAGIC_WRAPPER
+		    if (doList)
+		    {
+			tobj = Tcl_NewListObj(0, NULL);
+			Tcl_ListObjAppendElement(magicinterp, tobj,
+				Tcl_NewIntObj(la->cellUse->cu_bbox.r_xbot));
+			Tcl_ListObjAppendElement(magicinterp, tobj,
+				Tcl_NewIntObj(la->cellUse->cu_bbox.r_ybot));
+			Tcl_SetObjResult(magicinterp, tobj);
+		    }
 		    else
-			TxPrintf("Cell \"%s\":", la->cellUse->cu_def->cd_name);
-		    TxPrintf("x=%d ", la->cellUse->cu_bbox.r_xbot);
-		    TxPrintf("y=%d\n", la->cellUse->cu_bbox.r_ybot);
+		    {
+#endif
+			if (la->cellUse->cu_id != NULL)
+			    TxPrintf("Cell use \"%s\":", la->cellUse->cu_id);
+			else
+			    TxPrintf("Cell \"%s\":", la->cellUse->cu_def->cd_name);
+			TxPrintf("x=%d ", la->cellUse->cu_bbox.r_xbot);
+			TxPrintf("y=%d\n", la->cellUse->cu_bbox.r_ybot);
+#ifdef MAGIC_WRAPPER
+		    }
+#endif
 		}
 		break;
 	    }
 
-	    if ((cmd->tx_argc != 4) || (!StrIsInt(cmd->tx_argv[2])) ||
-				(!StrIsInt(cmd->tx_argv[3])))
+	    if ((locargc != 4) || (!StrIsInt(cmd->tx_argv[argstart + 1])) ||
+				(!StrIsInt(cmd->tx_argv[argstart + 2])))
 		goto badusage;
 
-	    xval = atoi(cmd->tx_argv[2]);
-	    yval = atoi(cmd->tx_argv[3]);
+	    xval = atoi(cmd->tx_argv[argstart + 1]);
+	    yval = atoi(cmd->tx_argv[argstart + 2]);
 
 	    TxPrintf("Unimplemented function.\n");
 	    break;
 
 	case ARRAY_DEFAULT:
-	    if (!StrIsInt(cmd->tx_argv[1]) || !StrIsInt(cmd->tx_argv[2])) 
+	    if (!StrIsInt(cmd->tx_argv[argstart])
+			|| !StrIsInt(cmd->tx_argv[argstart + 1]))
 		    goto badusage;
-	    if (cmd->tx_argc == 3)
+	    if (locargc == 3)
 	    {
 		a.ar_xlo = 0;
 		a.ar_ylo = 0;
-		a.ar_xhi = atoi(cmd->tx_argv[1]) - 1;
-		a.ar_yhi = atoi(cmd->tx_argv[2]) - 1;
+		a.ar_xhi = atoi(cmd->tx_argv[argstart]) - 1;
+		a.ar_yhi = atoi(cmd->tx_argv[argstart + 1]) - 1;
 		if ( (a.ar_xhi < 0) || (a.ar_yhi < 0) ) goto badusage;
 	    }
 	    else
 	    {
-		if (!StrIsInt(cmd->tx_argv[3]) || 
-			!StrIsInt(cmd->tx_argv[4])) goto badusage;
-		a.ar_xlo = atoi(cmd->tx_argv[1]);
-		a.ar_xhi = atoi(cmd->tx_argv[2]);
-		a.ar_ylo = atoi(cmd->tx_argv[3]);
-		a.ar_yhi = atoi(cmd->tx_argv[4]);
+		if (!StrIsInt(cmd->tx_argv[argstart + 2]) ||
+			!StrIsInt(cmd->tx_argv[argstart + 3])) goto badusage;
+		a.ar_xlo = atoi(cmd->tx_argv[argstart]);
+		a.ar_xhi = atoi(cmd->tx_argv[argstart + 1]);
+		a.ar_ylo = atoi(cmd->tx_argv[argstart + 2]);
+		a.ar_yhi = atoi(cmd->tx_argv[argstart + 3]);
 	    }
 
 	    if (!ToolGetBox((CellDef **) NULL, &toolRect))
@@ -386,7 +487,7 @@ selGetArrayFunc(selUse, use, trans, arg)
    LinkedArray **arg;
 {
     /* Check "use" for array information and pass this to arrayInfo */
-  
+
     LinkedArray *la;
     int xlo, xhi, ylo, yhi, xsep, ysep, t;
 
@@ -409,17 +510,17 @@ selGetArrayFunc(selUse, use, trans, arg)
     la->arrayInfo.ar_xhi = xhi;
     la->arrayInfo.ar_ylo = ylo;
     la->arrayInfo.ar_yhi = yhi;
-    
+
     /* Reverse the transformation in DBMakeArray */
 
     ysep = (trans->t_d * use->cu_xsep - trans->t_a * use->cu_ysep);
     ysep /= (trans->t_d * trans->t_b - trans->t_a * trans->t_e);
- 
+
     if (trans->t_a == 0)
 	xsep = (use->cu_ysep - trans->t_e * ysep) / trans->t_d;
     else
 	xsep = (use->cu_xsep - trans->t_b * ysep) / trans->t_a;
-    
+
     la->arrayInfo.ar_xsep = xsep;
     la->arrayInfo.ar_ysep = ysep;
 
@@ -430,7 +531,7 @@ selGetArrayFunc(selUse, use, trans, arg)
     return 0;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -511,7 +612,8 @@ CmdBox(w, cmd)
     Rect rootBox, editbox, *boxptr;
     Point ll;
     int option, direction, distancex, distancey;
-    int width, height, area;
+    int width, height;
+    dlong area;
     int argc;
     int tcorner;
     float iscale, oscale;
@@ -588,7 +690,7 @@ CmdBox(w, cmd)
 #endif
 	    return;
     }
-    
+
     if (needBox)
     {
 	if (refEdit)
@@ -980,7 +1082,7 @@ CmdBox(w, cmd)
 
 	    width = boxptr->r_xtop - boxptr->r_xbot;
 	    height = boxptr->r_ytop - boxptr->r_ybot;
-	    area = width * height;
+	    area = (dlong)width * (dlong)height;
 
 	    TxPrintf("%s cell box:\n", (refEdit) ? "Edit" : "Root");
 
@@ -1020,7 +1122,7 @@ CmdBox(w, cmd)
 			boxptr->r_xbot, boxptr->r_ybot,
 			boxptr->r_xtop, boxptr->r_ytop);
 	    if (area > 0)
-		TxPrintf("  %-10d", area);
+		TxPrintf("  %-10lld", area);
 	    TxPrintf("\n");
 	    break;
 
