@@ -3,16 +3,16 @@
  *
  * Commands with names beginning with the letters L through Q.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -50,7 +50,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 
 void CmdPaintEraseButton();
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -125,7 +125,7 @@ CmdLabelProc(text, font, size, rotate, offx, offy, pos, sticky, type)
     DBWLabelChanged(EditCellUse->cu_def, lab, DBW_ALLWINDOWS);
     lab->lab_rect = tmpArea;
 }
-
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -281,7 +281,7 @@ CmdLabel(w, cmd)
 		pos = GeoTransPos(&RootToEditTransform, pos);
 	}
     }
-    
+
     /*
      * Find and check validity of type parameter.  Accept prefix "-" on
      * layer as an indication of a "sticky" label (label is fixed to
@@ -309,7 +309,7 @@ CmdLabel(w, cmd)
     CmdLabelProc(p, font, size, rotate, offx, offy, pos, sticky, type);
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -480,8 +480,8 @@ keepGoing(use, clientdata)
  * Implement the "move" command.
  *
  * Usage:
- *	move [direction [amount]]
- *	move to x y
+ *	move [origin] [direction [amount]]
+ *	move [origin] to x y
  *
  * Results:
  *	None.
@@ -502,12 +502,13 @@ CmdMove(w, cmd)
     Point rootPoint, editPoint;
     CellDef *rootDef;
     int argpos;
+    bool doOrigin = FALSE;
 
     if (cmd->tx_argc > 4)
     {
 	badUsage:
-	TxError("Usage: %s [direction [amount]]\n", cmd->tx_argv[0]);
-	TxError("   or: %s to x y\n", cmd->tx_argv[0]);
+	TxError("Usage: %s [origin] [direction [amount]]\n", cmd->tx_argv[0]);
+	TxError("   or: %s [origin] to x y\n", cmd->tx_argv[0]);
 	return;
     }
 
@@ -518,18 +519,26 @@ CmdMove(w, cmd)
 
 	if (!ToolGetEditBox((Rect *)NULL)) return;
 
-	if (strcmp(cmd->tx_argv[1], "to") == 0)
+	argpos = 1;
+
+	if (strcmp(cmd->tx_argv[1], "origin") == 0)
+	{
+	    doOrigin = TRUE;
+	    argpos++;
+	}
+
+	if (strcmp(cmd->tx_argv[argpos], "to") == 0)
 	{
 	    if (cmd->tx_argc != 4)
 		goto badUsage;
-	    editPoint.p_x = cmdParseCoord(w, cmd->tx_argv[2], FALSE, TRUE);
-	    editPoint.p_y = cmdParseCoord(w, cmd->tx_argv[3], FALSE, FALSE);
+	    editPoint.p_x = cmdParseCoord(w, cmd->tx_argv[argpos + 1], FALSE, TRUE);
+	    editPoint.p_y = cmdParseCoord(w, cmd->tx_argv[argpos + 2], FALSE, FALSE);
 	    GeoTransPoint(&EditToRootTransform, &editPoint, &rootPoint);
 	    goto moveToPoint;
 	}
 
-	indx = GeoNameToPos(cmd->tx_argv[1], FALSE, FALSE);
-	argpos = (indx < 0) ? 1 : 2;
+	indx = GeoNameToPos(cmd->tx_argv[argpos], FALSE, FALSE);
+	if (indx >= 0) argpos++;
 
 	if (cmd->tx_argc >= 3)
 	{
@@ -605,8 +614,9 @@ CmdMove(w, cmd)
 	 * but is retained for backward compatibility.
 	 */
 
-	if (ToolGetBox(&rootDef, &rootBox) && ((rootDef == SelectRootDef)
-                                           || (SelectRootDef == NULL)))
+	if (ToolGetBox(&rootDef, &rootBox) &&
+		((rootDef == SelectRootDef) || (SelectRootDef == NULL))
+		&& (doOrigin == FALSE))
 	{
 	    GeoTransRect(&t, &rootBox, &newBox);
 	    DBWSetBox(rootDef, &newBox);
@@ -617,7 +627,7 @@ CmdMove(w, cmd)
 	/* Use the displacement between the box lower-left corner and
 	 * the point as the transform.
 	 */
-	
+
 	MagWindow *window;
 
 	window = ToolGetPoint(&rootPoint, (Rect *) NULL);
@@ -640,13 +650,43 @@ moveToPoint:
 	}
 	GeoTransTranslate(rootPoint.p_x - rootBox.r_xbot,
 	    rootPoint.p_y - rootBox.r_ybot, &GeoIdentityTransform, &t);
-	GeoTransRect(&t, &rootBox, &newBox);
-	DBWSetBox(rootDef, &newBox);
+	if (doOrigin == FALSE)
+	{
+	    GeoTransRect(&t, &rootBox, &newBox);
+	    DBWSetBox(rootDef, &newBox);
+	}
     }
-    
-    SelectTransform(&t);
+
+    if (doOrigin)
+    {
+	DBMoveCell(rootDef, t.t_c, t.t_f);
+
+	/* Adjust box to maintain relative position */
+
+	if (ToolGetBox(&rootDef, &rootBox) &&
+		((rootDef == SelectRootDef) || (SelectRootDef == NULL)))
+	{
+	    t.t_c = -t.t_c;
+	    t.t_f = -t.t_f;
+	    GeoTransRect(&t, &rootBox, &newBox);
+	    DBWSetBox(rootDef, &newBox);
+	    t.t_c = -t.t_c;
+	    t.t_f = -t.t_f;
+	}
+
+        /* Adjust all window viewing positions and redraw */
+
+        WindTranslate(t.t_c, t.t_f);
+
+        /* This is harsh.  Might work to change all distance measures	*/
+        /* in the undo record, but this is simple and direct.           */
+
+        UndoFlush();
+    }
+    else
+	SelectTransform(&t);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -718,7 +758,7 @@ CmdPaint(w, cmd)
     if (DRCBackGround)
 	DRCCheckThis (EditCellUse->cu_def, TT_CHECKPAINT, &editRect);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -756,7 +796,7 @@ CmdPaintEraseButton(w, butPoint, isPaint)
 
     WindPointToSurface(w, butPoint, (Point *) NULL, &rootRect);
 
-    DBSeeTypesAll(((CellUse *)w->w_surfaceID), &rootRect, 
+    DBSeeTypesAll(((CellUse *)w->w_surfaceID), &rootRect,
 	    crec->dbw_bitmask, &mask);
     TTMaskAndMask(&mask, &DBActiveLayerBits);
     TTMaskAndMask(&mask, &crec->dbw_visibleLayers);
@@ -927,7 +967,7 @@ CmdPath(w, cmd)
 	(void) StrDup(pathptr, srcptr);
     }
     return;
-	
+
 usage:
     TxError("Usage: %s [search|cell|sys] [[+]path]\n", cmd->tx_argv[0]);
 }
@@ -1076,7 +1116,7 @@ cmdPortLabelFunc2(scx, label, tpath, cdata)
 /*									*/
 /* If "port" is true, then search only for labels that are ports.	*/
 /* If "unique" is true, then return a label only if exactly one label	*/
-/* is found in the edit box.						*/ 
+/* is found in the edit box.						*/
 /*----------------------------------------------------------------------*/
 
 Label *
@@ -1091,7 +1131,9 @@ portFindLabel(editDef, port, unique, nonEdit)
     Rect editBox;
 
     /*
-     * Check for unique label in box area
+     * Check for unique label in box area.  Note that GEO_OVERLAP
+     * requires non-zero overlap area, so also check GEO_SURROUND to
+     * catch zero-area label rectangles.
      */
 
     ToolGetEditBox(&editBox);
@@ -1100,7 +1142,8 @@ portFindLabel(editDef, port, unique, nonEdit)
     lab = NULL;
     for (sl = editDef->cd_labels; sl != NULL; sl = sl->lab_next)
     {
-	if (GEO_OVERLAP(&editBox, &sl->lab_rect))
+	if (GEO_OVERLAP(&editBox, &sl->lab_rect) ||
+			GEO_SURROUND(&editBox, &sl->lab_rect))
 	{
 	    if (found > 0)
 	    {
@@ -1150,9 +1193,31 @@ portFindLabel(editDef, port, unique, nonEdit)
 	if (lab != NULL)
 	    if (nonEdit) *nonEdit = TRUE;
     }
-    
+
     return lab;
 }
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * complabel() --
+ *
+ * qsort() callback routine used by CmdPort when renumbering ports.
+ * Simply do a string comparison on the two labels.  There is no special
+ * meaning to the lexigraphic ordering;  it is meant only to enforce a
+ * consistent and repeatable port order.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+complabel(const void *one, const void *two)
+{
+    Label *l1 = *((Label **)one);
+    Label *l2 = *((Label **)two);
+    return strcmp(l1->lab_text, l2->lab_text);
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1164,9 +1229,11 @@ portFindLabel(editDef, port, unique, nonEdit)
  * usage, below).
  *
  * Usage:
- *	port make|makeall [num] [connect_direction(s)]
+ *	port make [num] [connect_direction(s)]
  * or
- *	port [name|num] class|use|index [value]
+ *	port makeall|renumber [connect_direction(s)]
+ * or
+ *	port [name|num] class|use|shape|index [value]
  *
  * num is the index of the port, usually beginning with 1.  This indicates
  *	the order in which ports should be written to a subcircuit record
@@ -1195,16 +1262,18 @@ portFindLabel(editDef, port, unique, nonEdit)
 
 #define PORT_CLASS	0
 #define PORT_USE	1
-#define PORT_INDEX	2
-#define PORT_EQUIV	3
-#define PORT_EXISTS	4
-#define PORT_CONNECT	5
-#define PORT_LAST	6
-#define PORT_MAKE	7
-#define PORT_MAKEALL	8
-#define PORT_NAME	9
-#define PORT_REMOVE	10
-#define PORT_HELP	11	
+#define PORT_SHAPE	2
+#define PORT_INDEX	3
+#define PORT_EQUIV	4
+#define PORT_EXISTS	5
+#define PORT_CONNECT	6
+#define PORT_LAST	7
+#define PORT_MAKE	8
+#define PORT_MAKEALL	9
+#define PORT_NAME	10
+#define PORT_REMOVE	11
+#define PORT_RENUMBER	12
+#define PORT_HELP	13
 
 void
 CmdPort(w, cmd)
@@ -1225,6 +1294,7 @@ CmdPort(w, cmd)
     {
 	"class	[type]		get [set] port class type",
 	"use	[type]		get [set] port use type",
+	"shape	[type]		get [set] port shape type",
 	"index	[number]	get [set] port number",
 	"equivalent [number]	make port equivalent to another port",
 	"exists			report if a label is a port or not",
@@ -1234,6 +1304,7 @@ CmdPort(w, cmd)
 	"makeall [index] [dir]	turn all labels into ports",
 	"name			report the port name",
 	"remove			turn a port back into a label",
+	"renumber		renumber all ports",
 	"help			print this help information",
 	NULL
     };
@@ -1286,6 +1357,25 @@ CmdPort(w, cmd)
 	PORT_USE_CLOCK
     };
 
+    static char *cmdPortShapeTypes[] =
+    {
+	"default",
+	"abutment",
+	"ring",
+	"feedthrough",
+	"feedthru",
+	NULL
+    };
+
+    static int cmdShapeToBitmask[] =
+    {
+	PORT_SHAPE_DEFAULT,
+	PORT_SHAPE_ABUT,
+	PORT_SHAPE_RING,
+	PORT_SHAPE_THRU,
+	PORT_SHAPE_THRU
+    };
+
     argstart = 1;
     argc = cmd->tx_argc;
     if (argc > 6 || argc == 1)
@@ -1329,7 +1419,10 @@ CmdPort(w, cmd)
 		}
 		if (lab == NULL)
 		{
-		    TxError("No label found with that name\n.");
+		    if (StrIsInt(cmd->tx_argv[1]))
+			TxError("No label found with index %s.\n", cmd->tx_argv[1]);
+		    else
+			TxError("No port found with name %s.\n", cmd->tx_argv[1]);
 		    return;
 		}
 		argstart = 2;
@@ -1370,11 +1463,15 @@ CmdPort(w, cmd)
 	    }
 
 	}
-	if ((option != PORT_LAST) && (option != PORT_MAKEALL) && (lab == NULL))
+	if ((option != PORT_LAST) && (option != PORT_MAKEALL)
+		&& (option != PORT_RENUMBER) && (lab == NULL))
 	{
 	    /* Let "port remove" fail without complaining. */
 	    if (option != PORT_REMOVE)
+	    {
 		TxError("Exactly one label may be present under the cursor box.\n");
+		TxError("Use \"port <name> ...\" to specify a uniqe port.\n");
+	    }
 
 	    return;
 	}
@@ -1382,7 +1479,8 @@ CmdPort(w, cmd)
 	/* Check for options that require label to be a port */
 
 	if ((option != PORT_MAKE) && (option != PORT_MAKEALL)
-		&& (option != PORT_EXISTS) && (option != PORT_LAST))
+		&& (option != PORT_EXISTS) && (option != PORT_RENUMBER)
+		&& (option != PORT_LAST))
 	{
 	    /* label "lab" must already be a port */
 	    if (!(lab->lab_flags & PORT_DIR_MASK))
@@ -1396,8 +1494,9 @@ CmdPort(w, cmd)
 	/* Check for options that cannot operate on a non-edit cell label */
 	if (nonEdit)
 	{
-	    if ((option == PORT_MAKE) || (option == PORT_MAKEALL) ||
-		(option == PORT_REMOVE) || (argc == 3))
+	    if ((option == PORT_MAKE) || (option == PORT_MAKEALL)
+		    || (option == PORT_REMOVE) || (option == PORT_RENUMBER)
+		    || (argc == 3))
 	    {
 		TxError("Cannot modify a port in an non-edit cell.\n");
 		return;
@@ -1491,8 +1590,16 @@ CmdPort(w, cmd)
 		    }
 		    else
 		    {
-		        lab->lab_flags &= (~PORT_CLASS_MASK);
-			lab->lab_flags |= (PORT_CLASS_MASK & cmdClassToBitmask[type]);
+			for (sl = lab; sl; sl = sl->lab_next)
+			{
+			    if (((sl->lab_flags & PORT_DIR_MASK) != 0) &&
+					!strcmp(sl->lab_text, lab->lab_text))
+			    {
+				sl->lab_flags &= (~PORT_CLASS_MASK);
+				sl->lab_flags |= (PORT_CLASS_MASK &
+					    cmdClassToBitmask[type]);
+			    }
+			}
 			editDef->cd_flags |= (CDMODIFIED | CDGETNEWSTAMP);
 		    }
 		}
@@ -1528,8 +1635,60 @@ CmdPort(w, cmd)
 		    }
 		    else
 		    {
-		        lab->lab_flags &= (~PORT_USE_MASK);
-			lab->lab_flags |= (PORT_USE_MASK & cmdUseToBitmask[type]);
+			for (sl = lab; sl; sl = sl->lab_next)
+			{
+			    if (((sl->lab_flags & PORT_DIR_MASK) != 0) &&
+					!strcmp(sl->lab_text, lab->lab_text))
+			    {
+				sl->lab_flags &= (~PORT_USE_MASK);
+				sl->lab_flags |= (PORT_USE_MASK & cmdUseToBitmask[type]);
+			    }
+			}
+			editDef->cd_flags |= (CDMODIFIED | CDGETNEWSTAMP);
+		    }
+		}
+		else
+		    goto portWrongNumArgs;
+		break;
+
+	    case PORT_SHAPE:
+		if (argc == 2)
+		{
+		    type = lab->lab_flags & PORT_SHAPE_MASK;
+		    for (idx = 0; cmdPortShapeTypes[idx] != NULL; idx++)
+			if (cmdShapeToBitmask[idx] == type)
+			{
+#ifdef MAGIC_WRAPPER
+			    Tcl_AppendResult(magicinterp, cmdPortShapeTypes[idx],
+					NULL);
+#else
+			    TxPrintf("Shape = %s\n", cmdPortShapeTypes[idx]);
+#endif
+			    break;
+			}
+		}
+		else if (argc == 3)
+		{
+		    type = Lookup(cmd->tx_argv[argstart + 1], cmdPortShapeTypes);
+		    if (type < 0)
+		    {
+		        TxError("Usage:  port shape <type>, where <type> is one of:\n");
+			for (msg = &(cmdPortShapeTypes[0]); *msg != NULL; msg++)
+			{
+                	    TxError("    %s\n", *msg);
+			}
+		    }
+		    else
+		    {
+			for (sl = lab; sl; sl = sl->lab_next)
+			{
+			    if (((sl->lab_flags & PORT_DIR_MASK) != 0) &&
+					!strcmp(sl->lab_text, lab->lab_text))
+			    {
+				sl->lab_flags &= (~PORT_SHAPE_MASK);
+				sl->lab_flags |= (PORT_SHAPE_MASK & cmdShapeToBitmask[type]);
+			    }
+			}
 			editDef->cd_flags |= (CDMODIFIED | CDGETNEWSTAMP);
 		    }
 		}
@@ -1553,8 +1712,15 @@ CmdPort(w, cmd)
 		        TxError("Usage:  port index <integer>\n");
 		    else
 		    {
-		        lab->lab_flags &= (~PORT_NUM_MASK);
-			lab->lab_flags |= atoi(cmd->tx_argv[argstart + 1]);
+			for (sl = lab; sl; sl = sl->lab_next)
+			{
+			    if (((sl->lab_flags & PORT_DIR_MASK) != 0) &&
+					!strcmp(sl->lab_text, lab->lab_text))
+			    {
+				sl->lab_flags &= (~PORT_NUM_MASK);
+				sl->lab_flags |= atoi(cmd->tx_argv[argstart + 1]);
+			    }
+			}
 			editDef->cd_flags |= (CDMODIFIED | CDGETNEWSTAMP);
 		    }
 		}
@@ -1595,6 +1761,55 @@ CmdPort(w, cmd)
 		editDef->cd_flags |= (CDMODIFIED | CDGETNEWSTAMP);
 		break;
 
+	    case PORT_RENUMBER:
+		/* Renumber ports in canonical order (by alphabetical
+		 * order of the label text).
+		 */
+		{
+		    int numlabels, n, p;
+		    Label **slablist, *tlab, *lastlab;
+		    extern int complabel();
+
+		    /* Create a sortable list of labels */
+		    numlabels = 0;
+		    for (lab = editDef->cd_labels; lab; lab = lab->lab_next)
+			numlabels++;
+
+		    slablist = (Label **)mallocMagic(numlabels * sizeof(Label *));
+		    numlabels = 0;
+		    for (lab = editDef->cd_labels; lab; lab = lab->lab_next)
+		    {
+			*(slablist + numlabels) = lab;
+			numlabels++;
+		    }
+
+		    /* Sort the list */
+		    qsort(slablist, numlabels, sizeof(Label *), complabel);
+
+		    /* Number the ports by sorted order */
+		    p = 0;
+		    lastlab = NULL;
+		    for (n = 0; n < numlabels; n++)
+		    {
+			tlab = *(slablist + n);
+			if (tlab->lab_flags & PORT_DIR_MASK)
+			{
+			    if (lastlab)
+				if (!strcmp(lastlab->lab_text, tlab->lab_text))
+				    p--;
+
+			    tlab->lab_flags &= ~PORT_NUM_MASK;
+			    tlab->lab_flags |= p;
+			    lastlab = tlab;
+			    p++;
+			}
+		    }
+
+		    /* Clean up */
+		    freeMagic((char *)slablist);
+		}
+		break;
+
 	    case PORT_MAKEALL:
 		lab = editDef->cd_labels;
 		// Fall through. . .
@@ -1616,7 +1831,7 @@ portWrongNumArgs:
 	}
 	return;
     }
-    
+
 parseindex:
 
     if ((option != PORT_MAKEALL) && (lab->lab_flags & PORT_DIR_MASK))
@@ -1677,7 +1892,7 @@ parseindex:
 			break;
 		    }
 		    else if ((sl->lab_flags & PORT_NUM_MASK) > idx)
-			idx = (sl->lab_flags & PORT_NUM_MASK); 
+			idx = (sl->lab_flags & PORT_NUM_MASK);
 		}
 	    }
 	    idx++;
@@ -1797,7 +2012,7 @@ CmdDoProperty(def, cmd, argstart)
     char *value;
     bool propfound;
     int locargc = cmd->tx_argc - argstart + 1;
-    
+
     if (locargc == 1)
     {
 	/* print all properties and their values */
@@ -1832,7 +2047,7 @@ CmdDoProperty(def, cmd, argstart)
 	    DBPropPut(def, cmd->tx_argv[argstart], value);
 	}
 	def->cd_flags |= (CDMODIFIED | CDGETNEWSTAMP);
-    }    
+    }
     else
     {
 	TxError("Usage: property [name] [value]\n");
@@ -1911,7 +2126,7 @@ printPropertiesFunc(name, value)
     }
     Tcl_AppendElement(magicinterp, keyvalue);
     freeMagic(keyvalue);
-    
+
 #else
     TxPrintf("%s = %s\n", name, value);
 #endif
@@ -1952,7 +2167,7 @@ CmdNetlist(w, cmd)
     char **msg, *lastargv;
     Point cursor;
     static char *cmdNetlistOption[] =
-    {	
+    {
 	"help           print this help information",
 	"select		select the net nearest the cursor",
 	"join           join current net and net containing terminal nearest the cursor",
