@@ -4,16 +4,16 @@
  * The functions in this file are local to the commands module
  * and not intended to be used by its clients.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -33,6 +33,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include "database/database.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
+#include "utils/malloc.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
 #include "utils/main.h"
@@ -261,7 +262,7 @@ CmdInit()
     CmdYMAllButSpace = DBAllButSpaceBits;
     TTMaskClearType(&CmdYMAllButSpace, L_CELL);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -325,7 +326,7 @@ cmdFlushCell(def)
 	    &parentUse->cu_bbox);
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -442,7 +443,7 @@ CmdParseLayers(s, mask)
 			    	|| (window->w_client != DBWclientID))
 			    return (FALSE);
 			crec = (DBWclientRec *) window->w_clientData;
-			DBSeeTypesAll(((CellUse *)window->w_surfaceID), 
+			DBSeeTypesAll(((CellUse *)window->w_surfaceID),
 				&rootRect, crec->dbw_bitmask, &newmask);
 			TTMaskAndMask(&newmask, &crec->dbw_visibleLayers);
 			tempmask = DBAllButSpaceAndDRCBits;
@@ -511,7 +512,7 @@ printTypes:
 
     return (TRUE);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -550,7 +551,7 @@ cmdMaskToType(mask)
 	return (TT_SPACE);
     return (type);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -582,13 +583,15 @@ cmdSaveCell(cellDef, newName, noninteractive, tryRename)
 			 * saved.  May be NULL, in which case the name from
 			 * the CellDef is taken.
 			 */
-    bool noninteractive;/* If true, try hard but don't ask the user 
+    bool noninteractive;/* If true, try hard but don't ask the user
 			 * questions.
 			 */
     bool tryRename;	/* We should rename the cell to the name of the
 			 * place where it was saved.
 			 */
 {
+    char *fileName = newName;
+
     /* Eliminate the phony labels added for use by rsim */
 #ifndef NO_SIM_MODULE
     SimEraseLabels();
@@ -599,46 +602,46 @@ cmdSaveCell(cellDef, newName, noninteractive, tryRename)
      * cell changes to the name of the file in which it was
      * saved.
      */
-    
+
     if (strcmp(cellDef->cd_name, UNNAMED) == 0)
     {
 	if (newName == NULL)
 	    TxPrintf("Must specify name for cell %s.\n", UNNAMED);
-	newName = cmdCheckNewName(cellDef, newName, TRUE, noninteractive);
-	if (newName == NULL) return;
+	fileName = cmdCheckNewName(cellDef, newName, TRUE, noninteractive);
+	if (fileName == NULL) return;
     }
     else if (newName != NULL)
     {
-	newName = cmdCheckNewName(cellDef, newName, TRUE, noninteractive);
-	if (newName == NULL) return;
+	fileName = cmdCheckNewName(cellDef, newName, TRUE, noninteractive);
+	if (fileName == NULL) return;
     }
     else
     {
 	if (cellDef->cd_file == NULL)
 	{
-	    newName = cmdCheckNewName(cellDef, cellDef->cd_name,
+	    fileName = cmdCheckNewName(cellDef, cellDef->cd_name,
 		TRUE, noninteractive);
-	    if (newName == NULL) return;
+	    if (fileName == NULL) return;
 	}
     }
 
     DBUpdateStamps();
-    if (!DBCellWrite(cellDef, newName))
+    if (!DBCellWrite(cellDef, fileName))
     {
 	TxError("Could not write file.  Cell not written.\n");
-	return;
+	goto cleanup;
     }
 
-    if (!tryRename || (newName == NULL) || (strcmp(cellDef->cd_name, newName) == 0))
-	return;
+    if (!tryRename || (fileName == NULL) || (strcmp(cellDef->cd_name, fileName) == 0))
+	goto cleanup;
 
     /* Rename the cell */
-    if (!DBCellRenameDef(cellDef, newName))
+    if (!DBCellRenameDef(cellDef, fileName))
     {
 	/* This should never happen */
 	TxError("Magic error: there is already a cell named \"%s\"\n",
-		    newName);
-	return;
+		    fileName);
+	goto cleanup;
     }
 
     if (EditCellUse && (cellDef == EditCellUse->cu_def))
@@ -657,11 +660,16 @@ cmdSaveCell(cellDef, newName, noninteractive, tryRename)
 	 * We want to find all windows for which this is
 	 * the root cell and update their captions.
 	 */
-	(void) WindSearch(DBWclientID, (ClientData) NULL, (Rect *) NULL, 
+	(void) WindSearch(DBWclientID, (ClientData) NULL, (Rect *) NULL,
 		cmdSaveWindSet, (ClientData) cellDef);
     }
+
+cleanup:
+    if ((fileName != newName) && (fileName != cellDef->cd_name))
+	freeMagic(fileName);
+    return;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -694,38 +702,44 @@ cmdCheckNewName(def, newName, tryRename, noninteractive)
     bool tryRename;
     bool noninteractive;
 {
-    static char newname[256];
     static char *yesno[] = { "no", "yes", 0 };
     char *filename;
     char *prompt;
+    char *returnname;
     int code;
     FILE *f;
 
+    returnname = newName;
+
 again:
-    if (newName == NULL)
+    if (returnname == NULL)
     {
 	if (noninteractive) {
 	    TxError("Can't write file named '%s'\n", def->cd_name);
 	    return NULL;
 	};
 	TxPrintf("File for cell %s: [hit return to abort save] ", def->cd_name);
-	if (TxGetLine(newname, sizeof newname) == NULL || newname[0] == '\0')
+	returnname = (char *)mallocMagic(1024 * sizeof(char));
+	if (TxGetLine(returnname, sizeof returnname) == NULL || returnname[0] == '\0')
 	{
 	    TxPrintf("Cell not saved.\n");
+	    freeMagic(returnname);
 	    return ((char *) NULL);
 	}
-	if (CmdIllegalChars(newname, "[],", "Cell name"))
+	if (CmdIllegalChars(returnname, "[],", "Cell name"))
+	{
+	    freeMagic(returnname);
 	    goto again;
-	newName = newname;
+	}
     }
 
     /* Remove any ".mag" file extension from the name */
-    else if (!strcmp(newName + strlen(newName) - 4, ".mag"))
-	*(newName + strlen(newName) - 4) = '\0';
+    if (!strcmp(returnname + strlen(returnname) - 4, ".mag"))
+	*(returnname + strlen(returnname) - 4) = '\0';
 
-    if (strcmp(newName, def->cd_name) != 0)
+    if (strcmp(returnname, def->cd_name) != 0)
     {
-	if (f = PaOpen(newName, "r", DBSuffix, ".", (char *) NULL, &filename))
+	if (f = PaOpen(returnname, "r", DBSuffix, ".", (char *) NULL, &filename))
 	{
 	    (void) fclose(f);
 	    if (noninteractive) {
@@ -740,25 +754,57 @@ again:
 		if (code == 0)
 		{
 		    /* No -- don't overwrite */
-		    newName = NULL;
+		    if (returnname != newName) freeMagic(returnname);
+		    returnname = NULL;
 		    goto again;
 		}
 	    }
 	}
 
-	if (tryRename && DBCellLookDef(newName) != NULL)
+	if (tryRename && DBCellLookDef(returnname) != NULL)
 	{
 	    TxError("Can't rename cell '%s' to '%s' because that cell already exists.\n",
-	    def->cd_name, newName);
+			def->cd_name, returnname);
+	    if (returnname != newName) freeMagic(returnname);
 	    if (noninteractive) return NULL;
-	    newName = NULL;
+	    returnname = NULL;
 	    goto again;
 	}
     }
-
-    return (newName);
+    return (returnname);
 }
-
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * nameEllipsis ---
+ *
+ *  Truncate a string an append an ellipsis ("...") to the end if the string
+ *  will overflow a fixed array length.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+static char *
+nameEllipsis(name, maxlen, prefix)
+    char *name;
+    int maxlen;
+    char **prefix;
+{
+    int l = strlen(name);
+
+    if (l < maxlen)
+    {
+	*prefix = "";
+	return name;
+    }
+    else
+    {
+	*prefix = "...";
+	return &name[l - maxlen + 3];
+    }
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -789,17 +835,19 @@ cmdSaveWindSet(window, def)
 {
     char caption[200];
     CellDef *rootDef;
+    char *name, *name_pfx;
 
     rootDef = ((CellUse *) window->w_surfaceID)->cu_def;
     if (rootDef != def)
 	return 0;
 
-    (void) sprintf(caption, "%s [NOT BEING EDITED]", def->cd_name);
+    name = nameEllipsis(def->cd_name, 175, &name_pfx);
+    (void) snprintf(caption, sizeof(caption), "%s%s [NOT BEING EDITED]", name_pfx, name);
     (void) StrDup(&window->w_iconname, def->cd_name);
     WindCaption(window, caption);
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -845,10 +893,10 @@ CmdSetWindCaption(newEditUse, rootDef)
 
     newEditDef = (newEditUse) ? newEditUse->cu_def : NULL;
     newRootDef = rootDef;
-    (void) WindSearch(DBWclientID, (ClientData) NULL, (Rect *) NULL, 
+    (void) WindSearch(DBWclientID, (ClientData) NULL, (Rect *) NULL,
 	    cmdWindSet, (ClientData) 0);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -881,13 +929,22 @@ cmdWindSet(window)
 {
     char caption[200];
     CellDef *wDef;
+    char *name[2], *name_pfx[2];
 
     wDef = ((CellUse *) window->w_surfaceID)->cu_def;
-    if (wDef != newRootDef)
-	(void) sprintf(caption, "%s [NOT BEING EDITED]", wDef->cd_name);
-    else {
-	(void) sprintf(caption, "%s EDITING %s", wDef->cd_name, 
-		newEditDef->cd_name);
+
+
+
+    if (wDef != newRootDef) {
+	name[0] = nameEllipsis(wDef->cd_name, 175, &name_pfx[0]);
+	(void) snprintf(caption, sizeof(caption), "%s%s [NOT BEING EDITED]",
+		    name_pfx[0], name[0]);
+    } else {
+	name[0] = nameEllipsis(wDef->cd_name, 90, &name_pfx[0]);
+	name[1] = nameEllipsis(newEditDef->cd_name, 90, &name_pfx[1]);
+	(void) snprintf(caption, sizeof(caption), "%s%s EDITING %s%s",
+		name_pfx[0], name[0], name_pfx[1], name[1]);
+
 #ifdef SCHEME_INTERPRETER
         /* Add a binding to scheme variable "edit-cell" */
         LispSetEdit (newEditDef->cd_name);
@@ -899,7 +956,7 @@ cmdWindSet(window)
     return 0;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -936,7 +993,7 @@ CmdGetRootPoint(point, rect)
 
     return (window);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -976,7 +1033,7 @@ CmdGetEditPoint(point, rect)
 
     return (window);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1012,7 +1069,7 @@ CmdWarnWrite()
     prompt = TxPrintString("%d Magic cell%s been modified.\n  Do you"
 		" want to exit magic and lose %s? ", count,
 		count == 1 ? " has" : "s have",
-		count == 1 ? "it" : "them");		
+		count == 1 ? "it" : "them");
     code = TxDialog(prompt, yesno, 0);
     return (code) ? TRUE : FALSE;
 }
@@ -1026,12 +1083,12 @@ cmdWarnWriteFunc(cellDef, pcount)
 	(*pcount)++;
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  * cmdExpandOneLevel --
  *
- *	Expand (unexpand) a cell, and unexpand all of its children.  This is 
+ *	Expand (unexpand) a cell, and unexpand all of its children.  This is
  *	called by commands such as getcell, expand current cell, and load.
  *	Don't bother to unexpand children if we are unexpanding this cell.
  *
@@ -1067,7 +1124,7 @@ cmdExpand1func(cu, bitmask)
     DBExpand(cu, (int) bitmask, FALSE);
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1119,7 +1176,7 @@ cmdGetSelFunc(selUse, realUse, transform, pResult)
 	*cmdSelTrans = *transform;
     return 1;			/* Skip any other selected cells. */
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
