@@ -3,16 +3,16 @@
  *
  * CellUse/CellDef creation, deletion, naming.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -107,7 +107,17 @@ DBCellRename(cellname, newname)
 
     if ((celldef->cd_flags & CDINTERNAL) == CDINTERNAL)
     {
-	TxError("Attempt to rename internal cell \"%s\"\n", cellname);
+	TxError("Error:  Attempt to rename internal cell \"%s\"\n", cellname);
+	return FALSE;
+    }
+
+    /* Disallow renaming if the cell has the READONLY flag set,	*/
+    /* because the cellname must match the name in the GDS	*/
+    /* file referenced.						*/
+
+    if ((celldef->cd_flags & CDVENDORGDS) == CDVENDORGDS)
+    {
+	TxError("Error:  Attempt to rename read-only cell \"%s\"\n", cellname);
 	return FALSE;
     }
 
@@ -148,7 +158,7 @@ DBEnumerateTypes(rMask)
     HashSearch hs;
     HashEntry *entry;
     CellDef *cellDef;
-    
+
     TTMaskZero(rMask);
     HashStartSearch(&hs);
     while ((entry = HashNext(&dbCellDefTable, &hs)) != NULL)
@@ -240,7 +250,7 @@ DBCellDelete(cellname, force)
     DBUndoReset(celldef);
 
     /* If we're destroying "UNNAMED", then we want to rename it first	*/
-    /* so that WindUnload() will create a new one.			*/ 
+    /* so that WindUnload() will create a new one.			*/
 
     if (!strcmp(cellname, UNNAMED))
 	DBCellRename(cellname, "__UNNAMED__");
@@ -271,7 +281,7 @@ DBCellDelete(cellname, force)
 }
 
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -307,7 +317,7 @@ DBCellInit()
  *
  * ----------------------------------------------------------------------------
  */
- 
+
 char *
 dbGetUseName(celluse)
    CellUse *celluse;
@@ -345,7 +355,7 @@ dbGetUseName(celluse)
 	    newsize += strlen(ybuf);
 	}
     }
-   
+
     newID = (char *)mallocMagic(newsize);
     strcpy(newID, useID);
     if (isx || isy)
@@ -359,7 +369,7 @@ dbGetUseName(celluse)
     return (newID);
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -549,7 +559,7 @@ dbCellPrintInfo(StartDef, who, dolist)
     }  /* endswitch */
 }
 
-
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -620,7 +630,7 @@ DBCellPrint(CellName, who, dolist)
     HashEntry *entry;
     CellDef *celldef;
     CellUse *celluse;
-    
+
     if (!dolist)
     {
 	switch (who)
@@ -953,7 +963,7 @@ dbUsePrintInfo(StartUse, who, dolist)
  *	if it is not a hierarchical name;  otherwise, if it is a
  *	hierarchical name, the instance before the last '/' is mapped
  *	to its cellDef, and that cellDef is searched for the indicated
- *	instance.  
+ *	instance.
  *
  * ----------------------------------------------------------------------------
  */
@@ -972,14 +982,14 @@ DBUsePrint(CellName, who, dolist)
     char *lasthier;
 
     int dbCellUsePrintFunc();
-    
+
     if ((CellName != NULL) && ((lasthier = strrchr(CellName, '/')) != NULL))
     {
 	char *prevhier;
 	*lasthier = '\0';
 	prevhier = strrchr(CellName, '/');
 	if (prevhier == NULL)
-	    prevhier = CellName;	
+	    prevhier = CellName;
 	else
 	    prevhier++;
 
@@ -1144,8 +1154,8 @@ DBLockUse(UseName, bval)
     CellDef *celldef;
     CellUse *celluse;
 
-    int dbUseLockFunc();
-    
+    int dbLockUseFunc();
+
    /*
     *
     * Check to see if a cell name was specified. If not,
@@ -1189,6 +1199,169 @@ DBLockUse(UseName, bval)
     }
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBOrientUse --
+ *
+ * 	This routine sets or reports a cell instance's orientation
+ *	UseName is the name of a specific CellUse.  If NULL, then the
+ *	operation applies to all selected cell uses.  "orient" is a
+ *	string in the form used by "getcell" (e.g., "180", "270v",
+ *	etc.), unless "dodef" is true, in which case the output is
+ *	given in the form used by DEF ("N", "FN", etc.).
+ *	reported.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	cu_transform changed for indicated cell use.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+DBOrientUse(UseName, dodef)
+    char *UseName;
+    bool dodef;
+{
+    int found;
+    HashSearch hs;
+    HashEntry *entry;
+    CellDef *celldef;
+    CellUse *celluse;
+
+    int dbOrientUseFunc();
+
+   /*
+    *
+    * Check to see if a cell name was specified. If not, then search
+    * for selected cells.
+    *
+    */
+
+    if (UseName == NULL)
+    {
+	if (EditCellUse == NULL)
+	    TxError("Cannot set orientation of a non-edit cell!\n");
+	else
+	    SelEnumCells(TRUE, (int *)NULL, (SearchContext *)NULL,
+			dbOrientUseFunc, (ClientData)&dodef);
+    }
+    else
+    {
+	SearchContext scx;
+
+	bzero(&scx, sizeof(SearchContext));
+	found = 0;
+
+	HashStartSearch(&hs);
+	while( (entry = HashNext(&dbCellDefTable, &hs)) != NULL)
+	{
+	    celldef = (CellDef *) HashGetValue(entry);
+	    if (celldef != (CellDef *) NULL)
+	    {
+		celluse = celldef->cd_parents;   /* only need one */
+		if (celluse != (CellUse *)NULL) {
+		    DBTreeFindUse(UseName, celluse, &scx);
+		    if (scx.scx_use != NULL) break;
+		}
+	    }
+	}
+
+	if (scx.scx_use == NULL)
+	    TxError("Cell %s is not currently loaded.\n", UseName);
+	else
+	    dbOrientUseFunc(NULL, scx.scx_use, NULL, (ClientData)&dodef);
+    }
+}
+
+/*
+ *  dbOrientUseFunc()
+ */
+
+/* For corresponding enumerations, see GeoTransOrient() */
+enum def_orient {ORIENT_NORTH, ORIENT_SOUTH, ORIENT_EAST, ORIENT_WEST,
+        ORIENT_FLIPPED_NORTH, ORIENT_FLIPPED_SOUTH, ORIENT_FLIPPED_EAST,
+        ORIENT_FLIPPED_WEST};
+
+int
+dbOrientUseFunc(selUse, use, transform, data)
+    CellUse *selUse;	/* Use from selection cell */
+    CellUse *use;	/* Use from layout corresponding to selection */
+    Transform *transform;
+    ClientData data;
+{
+    bool *dodef = (bool *)data;
+
+    if (EditCellUse && !DBIsChild(use, EditCellUse))
+    {
+	TxError("Cell %s (%s) isn't a child of the edit cell.\n",
+		use->cu_id, use->cu_def->cd_name);
+	return 0;
+    }
+
+    if (selUse != NULL)
+    {
+	switch (GeoTransOrient(&selUse->cu_transform)) {
+#ifdef MAGIC_WRAPPER
+	    case ORIENT_NORTH:
+		Tcl_AppendElement(magicinterp, (*dodef) ? "N" : "0");
+		break;
+	    case ORIENT_EAST:
+		Tcl_AppendElement(magicinterp, (*dodef) ? "E" : "90");
+		break;
+	    case ORIENT_SOUTH:
+		Tcl_AppendElement(magicinterp, (*dodef) ? "S" : "180");
+		break;
+	    case ORIENT_WEST:
+		Tcl_AppendElement(magicinterp, (*dodef) ? "W" : "270");
+		break;
+	    case ORIENT_FLIPPED_NORTH:
+		Tcl_AppendElement(magicinterp, (*dodef) ? "FN" : "0h");
+		break;
+	    case ORIENT_FLIPPED_EAST:
+		Tcl_AppendElement(magicinterp, (*dodef) ? "FE" : "90h");
+		break;
+	    case ORIENT_FLIPPED_SOUTH:
+		Tcl_AppendElement(magicinterp, (*dodef) ? "FS" : "180h");
+		break;
+	    case ORIENT_FLIPPED_WEST:
+		Tcl_AppendElement(magicinterp, (*dodef) ? "FW" : "270h");
+		break;
+#else
+	    case ORIENT_NORTH:
+		TxPrintf((*dodef) ? "N" : "0");
+		break;
+	    case ORIENT_EAST:
+		TxPrintf((*dodef) ? "E" : "90");
+		break;
+	    case ORIENT_SOUTH:
+		TxPrintf((*dodef) ? "S" : "180");
+		break;
+	    case ORIENT_WEST:
+		TxPrintf((*dodef) ? "W" : "270");
+		break;
+	    case ORIENT_FLIPPED_NORTH:
+		TxPrintf((*dodef) ? "FN" : "0h");
+		break;
+	    case ORIENT_FLIPPED_EAST:
+		TxPrintf((*dodef) ? "FE" : "90h");
+		break;
+	    case ORIENT_FLIPPED_SOUTH:
+		TxPrintf((*dodef) ? "FS" : "180h");
+		break;
+	    case ORIENT_FLIPPED_WEST:
+		TxPrintf((*dodef) ? "FW" : "270h");
+		break;
+#endif
+	}
+	
+    }
+    return 0;
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1215,7 +1388,7 @@ DBCellLookDef(cellName)
     return ((CellDef *) HashGetValue(entry));
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1245,14 +1418,12 @@ DBCellLookDef(cellName)
  */
 
 CellDef *
-DBCellNewDef(cellName, cellFileName)
+DBCellNewDef(cellName)
     char *cellName;		/* Name by which the cell is known */
-    char *cellFileName;		/* Name of disk file in which the cell 
-				 * should be kept when written out.
-				 */
 {
     CellDef *cellDef;
     HashEntry *entry;
+    char *dotptr;
 
     if (cellName == (char *) NULL)
 	cellName = UNNAMED;
@@ -1264,13 +1435,15 @@ DBCellNewDef(cellName, cellFileName)
     cellDef = DBCellDefAlloc();
     HashSetValue(entry, (ClientData) cellDef);
     cellDef->cd_name = StrDup((char **) NULL, cellName);
-    if (cellFileName == (char *) NULL)
-	cellDef->cd_file = cellFileName;
-    else
-	cellDef->cd_file = StrDup((char **) NULL, cellFileName);
+
+    /* Strip any .mag extension off of the cell name */
+    dotptr = strrchr(cellDef->cd_name, '.');
+    if (dotptr && !strcmp(dotptr, ".mag")) *dotptr = '\0';
+
+    cellDef->cd_file = NULL;
     return (cellDef);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1318,7 +1491,8 @@ DBCellDefAlloc()
     TTMaskZero(&cellDef->cd_types);
     HashInit(&cellDef->cd_idHash, 16, HT_STRINGKEYS);
 
-    cellDef->cd_planes[PL_CELL] = DBNewPlane((ClientData) NULL);
+    cellDef->cd_cellPlane = BPNew();
+    cellDef->cd_planes[PL_ROUTER] = DBNewPlane((ClientData) NULL);
     for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
 	cellDef->cd_planes[pNum] = DBNewPlane((ClientData) TT_SPACE);
 
@@ -1328,7 +1502,7 @@ DBCellDefAlloc()
 
     return (cellDef);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1376,7 +1550,7 @@ DBCellNewUse(cellDef, useName)
 
     /* Initial client field */
     /* (commands can use this field for whatever
-     * they like, but should restore its value to CLIENTDEFAULT before exiting.) 
+     * they like, but should restore its value to CLIENTDEFAULT before exiting.)
      */
     cellUse->cu_client = (ClientData) CLIENTDEFAULT;
 
@@ -1384,7 +1558,7 @@ DBCellNewUse(cellDef, useName)
     DBComputeUseBbox(cellUse);
     return (cellUse);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1429,7 +1603,7 @@ DBCellRenameDef(cellDef, newName)
 
     return (TRUE);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1472,7 +1646,7 @@ DBCellDeleteDef(cellDef)
     DBCellDefFree(cellDef);
     return TRUE;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1512,9 +1686,10 @@ DBCellDefFree(cellDef)
      */
 
     SigDisableInterrupts();
-    DBFreeCellPlane(cellDef->cd_planes[PL_CELL]);
-    TiFreePlane(cellDef->cd_planes[PL_CELL]);
+    DBClearCellPlane(cellDef);	/* Remove instances only */
+    BPFree(cellDef->cd_cellPlane);	/* Remove the cell plane itself */
 
+    TiFreePlane(cellDef->cd_planes[PL_ROUTER]);
     for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
     {
 	DBFreePaintPlane(cellDef->cd_planes[pNum]);
@@ -1529,7 +1704,7 @@ DBCellDefFree(cellDef)
 
     freeMagic((char *) cellDef);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1586,7 +1761,7 @@ DBCellDeleteUse(cellUse)
     freeMagic((char *) cellUse);
     return (TRUE);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1620,7 +1795,7 @@ DBCellSrDefs(pattern, func, cdata)
     int pattern;	/* Used for selecting cell definitions.  If any
 			 * of the bits in the pattern are in a def->cd_flags,
 			 * or if pattern is 0, the user-supplied function
-			 * is invoked.  
+			 * is invoked.
 			 */
     int (*func)();	/* Function to be applied to each matching CellDef */
     ClientData cdata;	/* Client data also passed to function */
@@ -1641,7 +1816,7 @@ DBCellSrDefs(pattern, func, cdata)
     }
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1698,7 +1873,7 @@ DBLinkCell(use, parentDef)
     lastName = strrchr(use->cu_def->cd_name, '/');
     if (lastName == NULL) lastName = use->cu_def->cd_name;
     else lastName++;
-    
+
     /* This search must not be interrupted */
     SigDisableInterrupts();
     (void) DBCellEnum(parentDef, dbLinkFunc, (ClientData) lastName);
@@ -1761,7 +1936,7 @@ dbLinkFunc(cellUse, defname)
 
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1806,7 +1981,7 @@ DBReLinkCell(cellUse, newName)
     if (UndoIsEnabled()) DBUndoCellUse(cellUse, UNDO_CELL_SETID);
     return (TRUE);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1831,7 +2006,11 @@ DBFindUse(id, parentDef)
 {
     HashEntry *he;
     char *delimit;
-   
+
+    /* Sanity checks */
+    if (id == NULL) return NULL;
+    if (parentDef == NULL) return NULL;
+
     /* Array delimiters should be ignored */
     if ((delimit = strrchr(id, '[')) != NULL) *delimit = '\0';
 
@@ -1842,7 +2021,7 @@ DBFindUse(id, parentDef)
 
     return (CellUse *) HashGetValue(he);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1886,7 +2065,7 @@ DBGenerateUniqueIds(def, warn)
     HashKill(&dbUniqueDefTable);
     HashKill(&dbUniqueNameTable);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1928,7 +2107,7 @@ dbFindNamesFunc(use, parentDef)
     }
     return (0);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1991,7 +2170,7 @@ setHash:
     DBSetUseIdHash(use, parentDef);
     return (0);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -2019,7 +2198,7 @@ DBSetUseIdHash(use, parentDef)
     he = HashFind(&parentDef->cd_idHash, use->cu_id);
     HashSetValue(he, (ClientData) use);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -2048,7 +2227,7 @@ DBUnLinkCell(use, parentDef)
 	HashSetValue(he, (ClientData) NULL);
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -2077,7 +2256,7 @@ DBNewYank(yname, pyuse, pydef)
     *pydef = DBCellLookDef(yname);
     if (*pydef == (CellDef *) NULL)
     {
-	*pydef = DBCellNewDef (yname,(char *) NULL);
+	*pydef = DBCellNewDef(yname);
 	ASSERT(*pydef != (CellDef *) NULL, "DBNewYank");
 	DBCellSetAvail(*pydef);
 	(*pydef)->cd_flags |= CDINTERNAL;
