@@ -3,16 +3,16 @@
  *
  * Reading and writing of cells
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -92,7 +92,7 @@ bool dbReadUse();
 /*
  * ----------------------------------------------------------------------------
  *
- *   file_is_not_writeable -- 
+ *   file_is_not_writeable --
  *
  *  Check to see if file is not writeable. (wen-king@cs.caltech.edu)
  *  Modified to deal with changed semantics of access() (rajit@cs.caltech.edu)
@@ -205,7 +205,7 @@ file_is_not_writeable(name)
  *	the direction of the corner made by the right triangle.
  *	If the split tile contains more than one type, separate entries
  *	are output for each.
- *	
+ *
  *	6. Zero or more groups of lines describing cell uses.  Each group
  *	is of the form
  *		use <filename> <id> [<path>]
@@ -345,7 +345,7 @@ dbCellReadDef(f, cellDef, name, ignoreTech, dereference)
 	}
 	if (dbFgets(line, sizeof line, f) == NULL)
 	    goto badfile;
-	
+
 	if (line[0] == 'm')
 	{
 	    if (!strncmp(line, "magscale", 8))
@@ -356,13 +356,13 @@ dbCellReadDef(f, cellDef, name, ignoreTech, dereference)
 		    n = d = 1;
 		}
 	    }
-		
+
 	    /* For backward compatibility, accept (and throw away) lines
 	     * whose first word is "maxlabscale".
 	     */
 	    else if (!strncmp(line, "maxlabscale", 11))
 		TxError("Deprecated keyword \"maxlabscale\" in input file.\n");
-	    else 
+	    else
 		TxError("Expected magscale but got: %s", line);
 	    if (dbFgets(line, sizeof line, f) == NULL)
 		goto badfile;
@@ -409,9 +409,9 @@ dbCellReadDef(f, cellDef, name, ignoreTech, dereference)
     }
     if (n > 1)
     {
-	TxPrintf("Scaled magic input cell %s geometry by factor of %d", 
+	TxPrintf("Scaled magic input cell %s geometry by factor of %d",
 		cellDef->cd_name, n);
-	if (d > 1) 
+	if (d > 1)
 	{
 	    TxPrintf("/ %d\n", d);
 	    TxError("Warning:  Geometry may be lost because internal grid"
@@ -481,7 +481,7 @@ dbCellReadDef(f, cellDef, name, ignoreTech, dereference)
 	/*
 	 * Record presence of material in cell.
 	 */
-	    
+
 	if (DBPlane(type) > 0)
 	{
 	    if (type < DBNumUserLayers)
@@ -520,7 +520,7 @@ nextrect:
 			((((dir & 0x2) >> 1) ^ (dir & 0x1)) ?
 			TT_DIRECTION : 0);
 	    }
-            else  
+            else
 	    {
 		dinfo = 0;
 		if (!GetRect(f, 4, rp, n, d)) goto badfile;
@@ -659,7 +659,7 @@ badfile:
  *	If DBbackupFile is non-NULL, then memory is freed, and the backup
  *	file is removed from the filesystem temp directory.  Otherwise,
  *	nothing happens.
- *	
+ *
  * ----------------------------------------------------------------------------
  */
 
@@ -753,16 +753,16 @@ DBFileRecovery(filename)
 				continue;
 
 			recent = sbuf.st_ctime;
-			DBbackupFile = StrDup(&DBbackupFile, tempname);
+			StrDup(&DBbackupFile, tempname);
 		    }
-		}  
-	    } 
+		}
+	    }
 	}
 	closedir(cwd);
     }
     else
     {
-	DBbackupFile = StrDup(&DBbackupFile, filename);
+	StrDup(&DBbackupFile, filename);
 	recent = 1;
     }
 
@@ -859,7 +859,7 @@ DBReadBackup(name)
 
  	    cellDef = DBCellLookDef(rootname);
 	    if (cellDef == (CellDef *)NULL)
-		cellDef = DBCellNewDef(rootname, (char *)NULL);
+		cellDef = DBCellNewDef(rootname);
 
 	    cellDef->cd_flags &= ~CDNOTFOUND;
 	    cellDef->cd_flags |= CDAVAILABLE;
@@ -874,7 +874,7 @@ DBReadBackup(name)
 		return FALSE;
 	    }
 	    /* Update timestamp flags from dbCellReadDef() */
-	    DBFlagMismatches();
+	    DBFlagMismatches(cellDef);
 	}
 	else
 	{
@@ -1054,23 +1054,22 @@ dbReadOpen(cellDef, name, setFileName, errptr)
 
 	    if (f != NULL)
 	    {
-		if (pptr != NULL) *pptr = '.';
-
 		/* NOTE:  May not want to present this as an error, as	*/
 		/* it is a common technique to read files from, say, a	*/
 		/* LEF file but not save them locally, and then expect	*/
 		/* that the layout views will be picked up from		*/
 		/* somewhere else in the search paths.			*/
 
+		if (pptr != NULL) *pptr = '.';
 		TxError("Warning:  Parent cell lists instance of \"%s\" at bad file "
 			"path %s.\n", cellDef->cd_name, cellDef->cd_file);
-		TxError("The cell exists in the search paths at %s.\n", filename);
-		TxError("The discovered version will be used.\n");
 
 		/* Write the new path to cd_file or else magic will	*/
 		/* generate another error later.			*/
+		StrDup(&cellDef->cd_file, filename);
 
-		cellDef->cd_file = StrDup(&cellDef->cd_file, filename);
+		TxError("The cell exists in the search paths at %s.\n", filename);
+		TxError("The discovered version will be used.\n");
 	    }
 	}
 
@@ -1099,7 +1098,7 @@ dbReadOpen(cellDef, name, setFileName, errptr)
 	    realname = (char *) mallocMagic((unsigned) (strlen(cellDef->cd_name)
 			+ strlen(DBSuffix) + 1));
 	    (void) sprintf(realname, "%s%s", cellDef->cd_name, DBSuffix);
-	    cellDef->cd_file = StrDup(&cellDef->cd_file, realname);
+	    StrDup(&cellDef->cd_file, realname);
 	}
 	if (errptr) TxError("%s\n", strerror(*errptr));
 
@@ -1133,7 +1132,14 @@ dbReadOpen(cellDef, name, setFileName, errptr)
     cellDef->cd_flags &= ~CDNOTFOUND;
 #endif
     if (setFileName)
+    {
+	/* Remove any ".mag" file extension */
+	char *pptr = strrchr(filename, '.');
+	if (pptr != NULL)
+	    if (!strcmp(pptr, DBSuffix)) *pptr = '\0';
+
 	(void) StrDup(&cellDef->cd_file, filename);
+    }
     cellDef->cd_flags |= CDAVAILABLE;
     return (f);
 }
@@ -1172,7 +1178,7 @@ DBTestOpen(name, fullPath)
     return FALSE;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1346,7 +1352,7 @@ badTransform:
     subCellDef = DBCellLookDef(cellname);
     if (subCellDef == (CellDef *) NULL)
     {
-	subCellDef = DBCellNewDef(cellname, (char *)NULL);
+	subCellDef = DBCellNewDef(cellname);
 	subCellDef->cd_timestamp = childStamp;
 
 	/* Make sure rectangle is non-degenerate */
@@ -1455,7 +1461,8 @@ badTransform:
 			pathOK = TRUE;
 		}
 
-		if ((pathOK == FALSE) && strcmp(subCellDef->cd_file, pathptr))
+		if ((pathOK == FALSE) && strcmp(subCellDef->cd_file, pathptr)
+			    && (dereference == FALSE))
 		{
 		    TxError("Duplicate cell in %s:  Instance of cell %s is from "
 				"path %s but cell was previously read from %s.\n",
@@ -1483,7 +1490,7 @@ badTransform:
 	    StrDup(&subCellDef->cd_file, path);
 	}
     }
-    
+
     subCellUse = DBCellNewUse(subCellDef, (useid[0]) ?
 		((locked) ? useid + 1 : useid) : (char *) NULL);
 
@@ -1532,7 +1539,7 @@ nextLine:
     return (dbFgets(line, len, f) != NULL);
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1644,7 +1651,7 @@ dbReadProperties(cellDef, line, len, f, scalen, scaled)
 			    locbbox.r_xbot, locbbox.r_ybot,
 			    locbbox.r_xtop, locbbox.r_ytop);
 		    (void) DBPropPut(cellDef, propertyname, storedvalue);
-	    
+
 		}
 	    }
 	    else
@@ -1866,7 +1873,7 @@ dbReadLabels(cellDef, line, len, f, scalen, scaled)
     int scalen;		/* Scale up by this factor */
     int scaled;		/* Scale down by this factor */
 {
-    char layername[50], text[1024], port_use[50], port_class[50];
+    char layername[50], text[1024], port_use[50], port_class[50], port_shape[50];
     TileType type;
     int ntok, orient, size, rotate, font, flags;
     Point offset;
@@ -1975,9 +1982,9 @@ dbReadLabels(cellDef, line, len, f, scalen, scaled)
 
 	    if (((lab = cellDef->cd_lastLabel) == NULL) ||
 			(lab->lab_flags & PORT_DIR_MASK) ||
-			(((ntok = sscanf(line, "port %d %4s %49s %49s",
-				&idx, ppos, port_use, port_class)) != 2) &&
-			(ntok != 4)))
+			(((ntok = sscanf(line, "port %d %4s %49s %49s %49s",
+				&idx, ppos, port_use, port_class, port_shape)) != 2) &&
+			(ntok != 4) && (ntok != 5)))
 	    {
 		TxError("Skipping bad \"port\" line: %s", line);
 		goto nextlabel;
@@ -2002,7 +2009,7 @@ dbReadLabels(cellDef, line, len, f, scalen, scaled)
 			break;
 		}
 	    }
-	    if (ntok == 4)
+	    if (ntok >= 4)
 	    {
 		switch(port_use[0])
 		{
@@ -2053,7 +2060,26 @@ dbReadLabels(cellDef, line, len, f, scalen, scaled)
 			TxError("Ignoring unknown \"port\" use: %s", port_use);
 			break;
 		}
-
+		if (ntok == 5) {
+		    switch(port_shape[0])
+		    {
+			case 'a':
+			    lab->lab_flags |= PORT_SHAPE_ABUT;
+			    break;
+			case 'r':
+			    lab->lab_flags |= PORT_SHAPE_RING;
+			    break;
+			case 'f':
+			    lab->lab_flags |= PORT_SHAPE_THRU;
+			    break;
+			case 'd':
+			    lab->lab_flags |= PORT_SHAPE_DEFAULT;
+			    break;
+			default:
+			    TxError("Ignoring unknown \"port\" shape: %s", port_shape);
+			    break;
+		    }
+		}
 	    }
 	    goto nextlabel;
 	}
@@ -2140,7 +2166,7 @@ nextlabel:
     return (TRUE);
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -2252,7 +2278,7 @@ DBCellFindScale(cellDef)
     }
 
     /* Finally, cell uses */
-	
+
     if (DBCellEnum(cellDef, dbFindCellGCFFunc, (ClientData) &ggcf))
 	return 1;
 
@@ -2276,7 +2302,7 @@ dbFindGCFFunc(tile, ggcf)
 	*ggcf = FindGCF(r.r_ytop, *ggcf);
     if (r.r_ybot % (*ggcf) != 0)
 	*ggcf = FindGCF(r.r_ybot, *ggcf);
-   
+
     return (*ggcf == 1) ? 1 : 0;
 }
 
@@ -2319,7 +2345,7 @@ dbFindCellGCFFunc(cellUse, ggcf)
 
     return (*ggcf == 1) ? 1 : 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -2498,7 +2524,7 @@ DBCellWriteFile(cellDef, f)
 		sprintf(lstring, "port %d %s", lab->lab_flags & PORT_NUM_MASK,
 			ppos);
 
-		if (lab->lab_flags & (PORT_USE_MASK | PORT_CLASS_MASK))
+		if (lab->lab_flags & (PORT_USE_MASK | PORT_CLASS_MASK | PORT_SHAPE_MASK))
 		{
 		    switch (lab->lab_flags & PORT_USE_MASK)
 		    {
@@ -2543,15 +2569,28 @@ DBCellWriteFile(cellDef, f)
 			    strcat(lstring, " default");
 			    break;
 		    }
+
+		    switch (lab->lab_flags & PORT_SHAPE_MASK)
+		    {
+			case PORT_SHAPE_ABUT:
+			    strcat(lstring, " abutment");
+			    break;
+			case PORT_SHAPE_RING:
+			    strcat(lstring, " ring");
+			    break;
+			case PORT_SHAPE_THRU:
+			    strcat(lstring, " feedthrough");
+			    break;
+		    }
 		}
 		strcat(lstring, "\n");
-		FPRINTF(f, lstring); 
+		FPRINTF(f, lstring);
 	    }
 	}
     }
 
     /* Now any persistent elements */
-    estring = DBWPrintElements(cellDef, DBW_ELEMENT_PERSISTENT);
+    estring = DBWPrintElements(cellDef, DBW_ELEMENT_PERSISTENT, reducer);
     if (estring != NULL)
     {
 	FPRINTF(f, "<< elements >>\n");
@@ -2651,7 +2690,7 @@ dbWritePropFunc(key, value, cdata)
     return 0;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -2661,7 +2700,7 @@ dbWritePropFunc(key, value, cdata)
  * Mark the cell as having been written out.  Before calling this
  * procedure, the caller should make sure that timestamps have been
  * updated where appropriate.
- * 
+ *
  * This code is fairly tricky to ensure that we never destroy the
  * original contents of a cell in the event of an I/O error.  We
  * try the following approaches in order.
@@ -2702,7 +2741,7 @@ DBCellWrite(cellDef, fileName)
 #define NAME_SIZE	1000
     char *template = ".XXXXXXX";
     char *realname, *tmpname, *expandname;
-    char *cp1, *cp2;
+    char *cp1, *cp2, *dotptr;
     char expandbuf[NAME_SIZE];
     FILE *realf, *tmpf;
     int tmpres;
@@ -2714,29 +2753,34 @@ DBCellWrite(cellDef, fileName)
     /*
      * Figure out the name of the file we will eventually write.
      */
-    if (fileName)
+    if (!fileName)
+    {
+	if (cellDef->cd_file)
+	    fileName = cellDef->cd_file;
+	else if (cellDef->cd_name)
+	    fileName = cellDef->cd_name;
+	else
+	    return FALSE;
+    }
+
+    /* Bug fix: 7/17/99, Michael D. Godfrey:  Forces		    */
+    /* cd_name and cd_file to ALWAYS be the same, otherwise ugly    */
+    /* surprises can occur after saving a file as a different	    */
+    /* filename.						    */
+
+    if (fileName != cellDef->cd_file)
+	StrDup(&cellDef->cd_file, fileName);
+
+    /* The cd_file should not have the .mag suffix, but make sure   */
+    /* it doesn't before adding one.				    */
+
+    if (strcmp(fileName + strlen(fileName) - 4, DBSuffix))
     {
 	realname = (char *) mallocMagic(strlen(fileName) + strlen(DBSuffix) + 1);
 	(void) sprintf(realname, "%s%s", fileName, DBSuffix);
-
-	/* Bug fix: 7/17/99, Michael D. Godfrey:  Forces		*/
-	/* cd_name and cd_file to ALWAYS be the same, otherwise ugly	*/
-	/* surprises can occur after saving a file as a different	*/
-	/* filename.							*/
-
-	cellDef->cd_file = StrDup(&cellDef->cd_file, realname);
     }
-    else if (cellDef->cd_file)
-    {
-	realname = StrDup((char **) NULL, cellDef->cd_file);
-    }
-    else if (cellDef->cd_name)
-    {
-	realname = (char *) mallocMagic((unsigned) (strlen(cellDef->cd_name)
-		+ strlen(DBSuffix) + 1));
-	(void) sprintf(realname, "%s%s", cellDef->cd_name, DBSuffix);
-    }
-    else return (FALSE);
+    else
+	realname = StrDup((char **)NULL, fileName);
 
     /*
      * Expand the filename, removing the leading ~, if any.
@@ -2834,7 +2878,7 @@ DBCellWrite(cellDef, fileName)
 	    close(cellDef->cd_fd);
 	    cellDef->cd_fd = -1;
 	}
-#endif  
+#endif
 
 	/*
 	 * The temp file is in good shape -- rename it to the real name,
@@ -2860,7 +2904,7 @@ DBCellWrite(cellDef, fileName)
 	    /* Re-aquire the lock on the new file by opening it. */
 	    DBCellRead(cellDef, NULL, TRUE, dereference, NULL);
 	}
-#endif  
+#endif
 
     }
     else if (exists)
@@ -2924,17 +2968,25 @@ DBCellWrite(cellDef, fileName)
 	}
     }
 
+    /* Copy expandname back to cellDef->cd_file, if the name was changed.   */
+    /* The file extension does not get copied into cd_file.		    */
+
+    dotptr = strrchr(expandname, '.');
+    if (dotptr) *dotptr = '\0';
+    if (strcmp(expandname, cellDef->cd_file))
+	StrDup(&cellDef->cd_file, expandname);
+    if (dotptr) *dotptr = '.';
+
     /* Everything worked so far. */
 
-    (void) StrDup(&cellDef->cd_file, expandname);
     result = TRUE;
     {
 	struct stat thestat;
-	realf = fopen(expandname,"r");
+	realf = fopen(expandname, "r");
 	if (realf == NULL)
 	{
 	    cellDef->cd_flags |= CDMODIFIED;
-	    TxError("Warning: Cannot open file for writing!\n");
+	    TxError("Warning: Cannot open file \"%s\" for writing!\n", expandname);
 	}
 	else
 	{
@@ -2942,7 +2994,7 @@ DBCellWrite(cellDef, fileName)
 	    if (thestat.st_size != DBFileOffset)
 	    {
 		cellDef->cd_flags |= CDMODIFIED;
-		TxError("Warning: I/O error in writing file\n");
+		TxError("Warning: I/O error in writing file \"%s\"\n", expandname);
 	    }
 	    fclose(realf);
 	}
@@ -2955,7 +3007,7 @@ cleanup:
     freeMagic(tmpname);
     return result;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -2988,8 +3040,8 @@ dbWritePaintFunc(tile, cdarg)
 
     int dir;
 
-    if (IsSplit(tile))      
-    { 
+    if (IsSplit(tile))
+    {
 	lMask = DBResidueMask(SplitLeftType(tile));
 	rMask = DBResidueMask(SplitRightType(tile));
 
@@ -3043,7 +3095,7 @@ dbWritePaintFunc(tile, cdarg)
     FPRINTR(arg->wa_file,pstring);
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -3186,7 +3238,7 @@ dbWriteCellFunc(cellUse, cdarg)
     FPRINTR(arg->wa_file,cstring)
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -3303,7 +3355,7 @@ DBWriteBackup(filename)
 		return FALSE;
 	    }
 	    close(fd);
-	    DBbackupFile = StrDup(&DBbackupFile, template);
+	    StrDup(&DBbackupFile, template);
 	    freeMagic(template);
 	    TxPrintf("Created database crash recovery file %s\n", DBbackupFile);
 	}
@@ -3313,10 +3365,10 @@ DBWriteBackup(filename)
     {
 	if (strlen(filename) == 0)
 	{
-	    DBbackupFile = StrDup(&DBbackupFile, (char *)NULL);
+	    StrDup(&DBbackupFile, (char *)NULL);
 	    return TRUE;
 	}
-	DBbackupFile = StrDup(&DBbackupFile, filename);
+	StrDup(&DBbackupFile, filename);
 	TxPrintf("Created database crash recovery file %s\n", DBbackupFile);
     }
 
@@ -3363,7 +3415,7 @@ dbWriteBackupFunc(def, f)
     if (name == NULL) name = def->cd_name;
 
     fprintf(f, "file %s\n", name);
-    
+
     /* Save/restore flags such that the crash recovery file write does	*/
     /* *not* clear the CDMODIFIED, et al., bits				*/
 
