@@ -3,21 +3,21 @@
  *
  * Label manipulation primitives.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBlabel.c,v 1.7 2008/12/11 18:36:43 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBlabel.c,v 1.9 2010/08/25 17:33:55 tim Exp $";
 #endif  /* not lint */
 
 #include <sys/types.h>
@@ -73,11 +73,11 @@ DBIsSubcircuit(cellDef)
     for (lab = cellDef->cd_labels; lab != NULL; lab = lab->lab_next)
 	if (lab->lab_flags & PORT_DIR_MASK)
 	    return TRUE;
- 
+
     return FALSE;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -156,7 +156,7 @@ DBPutFontLabel(cellDef, rect, font, size, rot, offset, align, text, type, flags)
      * from the boundary, so it won't stick out past the edge of
      * the cell boundary.
      */
-    
+
 #define BORDER 5
     if (align < 0)
     {
@@ -224,15 +224,15 @@ DBPutFontLabel(cellDef, rect, font, size, rot, offset, align, text, type, flags)
     cellDef->cd_flags |= CDMODIFIED|CDGETNEWSTAMP;
     return lab;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
  * DBEraseLabel --
  *
  * Delete labels attached to tiles of the indicated types that
- * are in the given area (as determined by the macro GEO_LABEL_IN_AREA).  
- * If this procedure is called as part of a command that also modifies paint, 
+ * are in the given area (as determined by the macro GEO_LABEL_IN_AREA).
+ * If this procedure is called as part of a command that also modifies paint,
  * then the paint modifications should be done BEFORE calling here.
  *
  * Results:
@@ -313,7 +313,55 @@ DBEraseLabel(cellDef, area, mask, areaReturn)
 	cellDef->cd_flags |= CDMODIFIED|CDGETNEWSTAMP;
     return (erasedAny);
 }
-
+
+#define	RECTEQUAL(r1, r2)	  ((r1)->r_xbot == (r2)->r_xbot \
+				&& (r1)->r_ybot == (r2)->r_ybot \
+				&& (r1)->r_xtop == (r2)->r_xtop \
+				&& (r1)->r_ytop == (r2)->r_ytop)
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBCheckLabelsByContent --
+ *
+ * Return any label found on the label list for the given
+ * CellDef that matches the given specification.
+ *
+ * Results:
+ *	Returns a label if a match is found, otherwise returns NULL.
+ *
+ * Side effects:
+ *	None.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+Label *
+DBCheckLabelsByContent(def, rect, type, text)
+    CellDef *def;		/* Where to look for label to delete. */
+    Rect *rect;			/* Coordinates of label.  If NULL, then
+				 * labels are searched regardless of coords.
+				 */
+    TileType type;		/* Layer label is attached to.  If < 0, then
+				 * labels are searched regardless of type.
+				 */
+    char *text;			/* Text associated with label.  If NULL, then
+				 * labels are searched regardless of text.
+				 */
+{
+    Label *lab;
+
+    for (lab = def->cd_labels; lab; lab = lab->lab_next)
+    {
+	if ((rect != NULL) && !(RECTEQUAL(&lab->lab_rect, rect))) continue;
+	if ((type >= 0) && (type != lab->lab_type)) continue;
+	if ((text != NULL) && (strcmp(text, lab->lab_text) != 0)) continue;
+
+	return lab;
+    }
+    return NULL;
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -348,11 +396,6 @@ DBEraseLabelsByContent(def, rect, type, text)
 {
     Label *lab, *labPrev;
 
-#define	RECTEQUAL(r1, r2)	  ((r1)->r_xbot == (r2)->r_xbot \
-				&& (r1)->r_ybot == (r2)->r_ybot \
-				&& (r1)->r_xtop == (r2)->r_xtop \
-				&& (r1)->r_ytop == (r2)->r_ytop)
-
     for (labPrev = NULL, lab = def->cd_labels;
 	    lab != NULL;
 	    labPrev = lab, lab = lab->lab_next)
@@ -379,7 +422,7 @@ DBEraseLabelsByContent(def, rect, type, text)
 	else goto nextCheck;
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -440,7 +483,7 @@ DBEraseLabelsByFunction(def, func)
 	else goto nextCheck;
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -482,7 +525,7 @@ DBReOrientLabel(cellDef, area, newPos)
 	}
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -517,12 +560,13 @@ DBAdjustLabels(def, area)
     /* First, find each label that crosses the area we're
      * interested in.
      */
-    
+
     for (lab = def->cd_labels; lab != NULL; lab = lab->lab_next)
     {
 	if (!GEO_TOUCH(&lab->lab_rect, area)) continue;
 	newType = DBPickLabelLayer(def, lab, 0);
 	if (newType == lab->lab_type) continue;
+	if (lab->lab_flags & LABEL_STICKY) continue;
 	if (DBVerbose && ((def->cd_flags & CDINTERNAL) == 0)) {
 	    TxPrintf("Moving label \"%s\" from %s to %s in cell %s.\n",
 		    lab->lab_text, DBTypeLongName(lab->lab_type),
@@ -539,7 +583,7 @@ DBAdjustLabels(def, area)
 
 
 /*
- * Extended version of DBAdjustLabels.  If noreconnect==0, 
+ * Extended version of DBAdjustLabels.  If noreconnect==0,
  * this is supposed to be the same as DBAdjustlabels() above.
  */
 void
@@ -557,26 +601,28 @@ DBAdjustLabelsNew(def, area, noreconnect)
     /* First, find each label that crosses the area we're
      * interested in.
      */
-    
+
     labPrev = NULL;
     lab = def->cd_labels;
     while (lab != NULL)
     {
+	    int locnoreconnect = noreconnect;
 	    if (!GEO_TOUCH(&lab->lab_rect, area)) {
 		    goto nextLab;
 	    }
-	    newType = DBPickLabelLayer(def, lab, noreconnect);
+	    if (lab->lab_type == TT_SPACE) locnoreconnect = FALSE;
+	    newType = DBPickLabelLayer(def, lab, locnoreconnect);
 	    if (newType == lab->lab_type) {
 		    goto nextLab;
-	    } 
-	    if(newType < 0) {
+	    }
+	    if(newType < 0 && !(lab->lab_flags & LABEL_STICKY)) {
 		    TxPrintf("Deleting ambiguous-layer label \"%s\" from %s in cell %s.\n",
 			     lab->lab_text, DBTypeLongName(lab->lab_type),
 			     def->cd_name);
-	    
+
 		    if (labPrev == NULL)
 			    def->cd_labels = lab->lab_next;
-		    else 
+		    else
 			    labPrev->lab_next = lab->lab_next;
 		    if (def->cd_lastLabel == lab)
 			    def->cd_lastLabel = labPrev;
@@ -586,7 +632,7 @@ DBAdjustLabelsNew(def, area, noreconnect)
 		    lab = lab->lab_next;
 		    modified = TRUE;
 		    continue;
-	    } else {
+	    } else if (!(lab->lab_flags & LABEL_STICKY)) {
 		    if (DBVerbose && ((def->cd_flags & CDINTERNAL) == 0)) {
 			    TxPrintf("Moving label \"%s\" from %s to %s in cell %s.\n",
 				     lab->lab_text, DBTypeLongName(lab->lab_type),
@@ -597,7 +643,7 @@ DBAdjustLabelsNew(def, area, noreconnect)
 		    DBUndoPutLabel(def, lab);
 		    modified = TRUE;
 	    }
-    nextLab: 
+    nextLab:
 	    labPrev = lab;
 	    lab = lab->lab_next;
     }
@@ -605,7 +651,7 @@ DBAdjustLabelsNew(def, area, noreconnect)
     if (modified) DBCellSetModified(def, TRUE);
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -642,7 +688,7 @@ DBPickLabelLayer(def, lab, noreconnect)
 {
     TileTypeBitMask types[3], types2[3];
     Rect check1, check2;
-    int i, plane;
+    int i, j, plane;
     TileType choice1, choice2, choice3, choice4, choice5, choice6;
     extern int dbPickFunc1(), dbPickFunc2();
 
@@ -680,7 +726,7 @@ DBPickLabelLayer(def, lab, noreconnect)
 	 * left and one on the right.  For each side, compute
 	 * the type arrays separately.  Then merge them together.
 	 */
-	
+
 	check1 = lab->lab_rect;
 	check2 = lab->lab_rect;
 	check1.r_xbot -= 1;
@@ -713,7 +759,7 @@ DBPickLabelLayer(def, lab, noreconnect)
 	 * top and one on the bottom.  Use the code from above
 	 * to handle.
 	 */
-	
+
 	check1 = lab->lab_rect;
 	check2 = lab->lab_rect;
 	check1.r_ybot -= 1;
@@ -725,7 +771,7 @@ DBPickLabelLayer(def, lab, noreconnect)
 	/* This is a rectangular label.  Same thing as for line labels,
 	 * except there's only one area to search.
 	 */
-	
+
 	types[0] = DBAllButSpaceAndDRCBits;
 	types[1] = DBAllButSpaceAndDRCBits;
 	TTMaskZero(&types[2]);
@@ -752,63 +798,73 @@ DBPickLabelLayer(def, lab, noreconnect)
      * 5. A layer that covers the label.
      * 6. A layer that is a component of material that covers the label.
      * 7. Space.
+     *
+     * All searches are done from the lowest to highest plane, so that
+     * the label connects to material on the highest plane that matches
+     * the criteria above.  This avoids weirdnesses caused by declaring
+     * types out of order in the techfile.
      */
-    
+
     if (TTMaskHasType(&types[0], lab->lab_type)) return lab->lab_type;
     plane = DBPlane(lab->lab_type);
     choice1 = choice2 = choice3 = choice4 = choice5 = choice6 = TT_SPACE;
-    for (i = TT_SELECTBASE; i < DBNumUserLayers; i += 1)
+
+    for (j = PL_SELECTBASE; j < DBNumPlanes; j++)
     {
-	if (!TTMaskHasType(&types[2], i)) continue;
-	if (DBConnectsTo(i, lab->lab_type))
+	for (i = TT_SELECTBASE; i < DBNumUserLayers; i += 1)
 	{
-	    if (DBPlane(i) == plane)
+	    if (!TTMaskHasType(&DBPlaneTypes[j], i)) continue;
+
+	    if (DBConnectsTo(i, lab->lab_type))
 	    {
+		if (DBPlane(i) == plane)
+		{
+		    if (TTMaskHasType(&types[0], i))
+		    {
+			choice1 = i;
+			continue;
+		    }
+		    else if (TTMaskHasType(&types[1], i))
+		    {
+			choice2 = i;
+			continue;
+		    }
+		}
 		if (TTMaskHasType(&types[0], i))
 		{
-		    choice1 = i;
+		    choice3 = i;
 		    continue;
 		}
 		else if (TTMaskHasType(&types[1], i))
 		{
-		    choice2 = i;
+		    choice4 = i;
 		    continue;
 		}
 	    }
 	    if (TTMaskHasType(&types[0], i))
 	    {
-		choice3 = i;
+		/* A type that connects to more than itself is preferred */
+		if (choice5 == TT_SPACE)
+		    choice5 = i;
+		else
+		{
+		    TileTypeBitMask ctest;
+		    TTMaskZero(&ctest);
+		    TTMaskSetMask(&ctest, &DBConnectTbl[i]);
+		    TTMaskClearType(&ctest, i);
+		    if (!TTMaskIsZero(&ctest))
+			choice5 = i;
+		    else if (TTMaskHasType(&types[1], i))
+			choice6 = i;
+		}
 		continue;
 	    }
 	    else if (TTMaskHasType(&types[1], i))
 	    {
-		choice4 = i;
+		choice6 = i;
 		continue;
 	    }
 	}
-	if (TTMaskHasType(&types[0], i))
-	{
-	    /* A type that connects to more than itself is preferred */
-	    if (choice5 == TT_SPACE)
-		choice5 = i;
-	    else
-	    {
-		TileTypeBitMask ctest;
-		TTMaskZero(&ctest);
-		TTMaskSetMask(&ctest, &DBConnectTbl[i]);
-		TTMaskClearType(&ctest, i);
-		if (!TTMaskIsZero(&ctest))
-		    choice5 = i;
-		else if (TTMaskHasType(&types[1], i))
-		    choice6 = i;
-	    }
-	    continue;
-	}
-	else if (TTMaskHasType(&types[1], i))
-	{
-	    choice6 = i;
-	    continue;
-	}
     }
 
     if (choice1 != TT_SPACE) return choice1;
@@ -818,7 +874,7 @@ DBPickLabelLayer(def, lab, noreconnect)
     else if (noreconnect) {
 #ifdef notdef
 	TxPrintf("DBPickLabelLayer \"%s\" (on %s at %d,%d) choice4=%s choice5=%s choice6=%s.\n",
-		     lab->lab_text, 
+		     lab->lab_text,
 		     DBTypeLongName(lab->lab_type),
 		     lab->lab_rect.r_xbot,
 		     lab->lab_rect.r_ytop,
@@ -849,12 +905,10 @@ dbPickFunc1(tile, mask)
 {
     TileType type;
 
-#ifdef NONMANHATTAN 
-    if (IsSplit(tile)) 
+    if (IsSplit(tile))
 	type = (SplitSide(tile)) ? SplitRightType(tile) : SplitLeftType(tile);
     else
-#endif
-    type = TiGetType(tile);
+	type = TiGetType(tile);
 
     if (type == TT_SPACE) return 0;
     TTMaskSetType(mask, type);
@@ -878,12 +932,10 @@ dbPickFunc2(tile, mask)
     TileType type;
     TileTypeBitMask tmp, *rMask;
 
-#ifdef NONMANHATTAN 
-    if (IsSplit(tile)) 
+    if (IsSplit(tile))
 	type = (SplitSide(tile)) ? SplitRightType(tile) : SplitLeftType(tile);
     else
-#endif
-    type = TiGetType(tile);
+	type = TiGetType(tile);
 
     if (type == TT_SPACE)
     {
@@ -953,7 +1005,7 @@ DBFontInitCurves()
  *
  *  Note:  This routine does not take into account the curvature, and therefore
  *  does not optimize for the minimum number of line segments needed to
- *  represent the curve within a fixed margin of error. 
+ *  represent the curve within a fixed margin of error.
  *
  * ----------------------------------------------------------------------------
  */
@@ -994,7 +1046,7 @@ CalcBezierPoints(fp, bp)
 	    newPath->fp_point.p_x = tmpx;
 	    newPath->fp_point.p_y = tmpy;
 	    curPath->fp_next = newPath;
-	    curPath = newPath; 
+	    curPath = newPath;
 	}
     }
 
@@ -1063,7 +1115,7 @@ dbGetToken(ff)
  *	units of (database / 8) for a finer resolution, and measured
  *	relative to the label rectangle's origin so we don't run out of
  *	bits in computing the corner positions.
- *	
+ *
  *	This routine needs to be run whenever a font label changes
  *	properties (including when the font label is created).
  *
@@ -1498,7 +1550,7 @@ DBLoadFont(fontfile, scale)
 		/* If the first and last points are the same, remove the last one */
 		if (!GEO_SAMEPOINT(curPath->fp_point, pathStart->fp_point))
 		    newChar->fc_numpoints++;
-		
+
 		newChar->fc_points = (Point *)mallocMagic(i * sizeof(Point));
 		newChar->fc_next = NULL;
 		for (i = 0, curPath = pathStart; i < newChar->fc_numpoints;
