@@ -6,16 +6,16 @@
  * of the general-purpose exported TileTypeBitMasks, and
  * the "connect" section.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -80,7 +80,7 @@ DBTechInit()
     /* Initialization of bezier coefficients for font vectors */
     DBFontInitCurves();
 }
-
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -115,7 +115,7 @@ DBTechSetTech(sectionName, argc, argv)
 	    {
 		if (StrIsInt(argv[1]))
 		{
-		    TechFormatVersion = atoi(argv[1]);	
+		    TechFormatVersion = atoi(argv[1]);
 		    return TRUE;
 		}
 		else
@@ -139,7 +139,7 @@ DBTechSetTech(sectionName, argc, argv)
  * DBTechInitVersion --
  *
  *	Clean up memory allocated by the "version" section
- * 
+ *
  * ----------------------------------------------------------------------------
  */
 
@@ -209,7 +209,7 @@ usage:
     TechError("Badly formed version line\nUsage: {version text}|{description text}\n");
     return FALSE;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -238,7 +238,7 @@ DBTechInitConnect()
 	DBAllConnPlanes[i] = 0;
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -287,7 +287,7 @@ DBTechAddConnect(sectionName, argc, argv)
 
     return TRUE;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -324,8 +324,9 @@ void
 DBTechFinalConnect()
 {
     /* TileTypeBitMask saveConnect[TT_MAXTYPES]; */
+    TileTypeBitMask *cMask, *rMask;
     TileType base, s;
-    LayerInfo *lp;
+    LayerInfo *lp, *ls;
     int n;
 
     for (s = 0; s < DBNumTypes; s++)
@@ -363,10 +364,50 @@ DBTechFinalConnect()
 	    if (TTMaskHasType(&DBConnectTbl[base], s))
 		TTMaskSetType(&DBConnectTbl[s], base);
 
-    /* Construct DBNotConnectTbl[] to be the complement of DBConnectTbl[] */
+    /* Construct DBNotConnectTbl[] */
+    /* For purposes of connectivity searching, this is not exactly the	*/
+    /* complement of DBConnectTbl[].  For non-contact layers, it is.	*/
+    /* For contact images, we compute differently to account for	*/
+    /* contacts being stacked by having different parts in different	*/
+    /* cells, which requires a different mask.				*/
+
     for (base = 0; base < TT_MAXTYPES; base++)
 	TTMaskCom2(&DBNotConnectTbl[base], &DBConnectTbl[base]);
 
+    for (n = 0; n < dbNumContacts; n++)
+    {
+	lp = dbContactInfo[n];
+	TTMaskSetOnlyType(&DBNotConnectTbl[lp->l_type], lp->l_type);
+	rMask = DBResidueMask(lp->l_type);
+
+	/* Different contact types may share residues.		*/
+	/* Use TTMaskIntersect(), not TTMaskEqual()---types	*/
+	/* which otherwise stack may be in separate cells.	*/
+
+	for (s = 0; s < dbNumContacts; s++)
+	{
+	    ls = dbContactInfo[s];
+	    cMask = DBResidueMask(ls->l_type);
+	    if (TTMaskIntersect(rMask, cMask))
+		TTMaskSetType(&DBNotConnectTbl[lp->l_type], ls->l_type);
+	}
+
+	/* The mask of contact types must include all stacked contacts */
+
+	for (base = DBNumUserLayers; base < DBNumTypes; base++)
+	{
+	    cMask = DBResidueMask(base);
+	    if (TTMaskHasType(cMask, lp->l_type))
+		TTMaskSetType(&DBNotConnectTbl[lp->l_type], base);
+	}
+
+	/* Note that everything above counted for all the	*/
+	/* connecting types.  Invert this to get non-connecting	*/
+	/* types.						*/
+
+	TTMaskCom(&DBNotConnectTbl[lp->l_type]);
+    }
+
     /*
      * DBConnPlanes[] is nonzero only for contact images, for which
      * it shows the planes connected to an image.  It is the
@@ -384,10 +425,36 @@ DBTechFinalConnect()
      * for those planes to which each type 'base' connects, exclusive
      * of its home plane and those planes to which it connects as a
      * contact.
+     *
+     * Note that DBAllConnPlanes() is used to check for connectivity
+     * (e.g., during extraction) where connections are made where no
+     * contact exists.  This is VERY compute-intensive, so adding
+     * planes unnecessarily must be avoided at all costs.  Consider
+     * that a better alternative might be to designate layers that
+     * connect to other planes as contacts themselves (such as
+     * nsd, which connects to nwell) and restrict the "connect" section
+     * to types in a plane (such as poly and transistor).
      */
+
     for (base = TT_TECHDEPBASE; base < DBNumTypes; base++)
     {
-	DBAllConnPlanes[base] = DBTechTypesToPlanes(&DBConnectTbl[base]);
+	TileTypeBitMask baseConnList;
+
+	/* For one, remove all contact types from the list of	*/
+	/* connecting types.  Otherwise, simple connect		*/
+	/* expressions like "*m1 *m1" have the unintended	*/
+	/* consequence of connecting the metal2 plane (because	*/
+	/* via contains m1) to the active plane (because active	*/
+	/* contacts also contain m1).				*/
+
+	baseConnList = DBConnectTbl[base];
+	for (s = 0; s < dbNumContacts; s++)
+	{
+	    ls = dbContactInfo[s];
+	    TTMaskClearType(&baseConnList, ls->l_type);
+	}
+
+	DBAllConnPlanes[base] = DBTechTypesToPlanes(&baseConnList);
 	DBAllConnPlanes[base] &= ~(PlaneNumToMaskBit(DBPlane(base)));
 	DBAllConnPlanes[base] &= ~DBConnPlanes[base];
     }
