@@ -6,16 +6,16 @@
  * DB*ShortName(), * the procedures in this file MUST be called
  * after DBTechFinalType() has been called.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -39,7 +39,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include "textio/textio.h"
 #include "utils/malloc.h"
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -203,7 +203,7 @@ DBTechNameTypes(typename, bitmask)
     return (type < DBNumUserLayers) ? type : -2;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -267,7 +267,7 @@ DBTechNamePlane(planename)
 {
     return ((spointertype) dbTechNameLookup(planename, &dbPlaneNameLists));
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -304,7 +304,7 @@ DBTechNoisyNamePlane(planename)
 
     return (pNum);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -365,7 +365,7 @@ DBPlaneShortName(pNum)
 	return (DBPlaneLongNameTbl[pNum]);
     return ("???");
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -393,8 +393,8 @@ DBTechTypesToPlanes(mask)
     TileType t;
     PlaneMask planeMask, noCellMask, retMask;
 
-    /* Space tiles are present in all planes but the cell plane */
-    noCellMask = ~(PlaneNumToMaskBit(PL_CELL));
+    /* Space tiles are present in all planes but the router plane */
+    noCellMask = ~(PlaneNumToMaskBit(PL_ROUTER));
     if (TTMaskHasType(mask, TT_SPACE)) {
 	retMask = PlaneNumToMaskBit(DBNumPlanes) - 1;
 	retMask &= noCellMask;
@@ -409,7 +409,7 @@ DBTechTypesToPlanes(mask)
     retMask = planeMask & noCellMask;
     return retMask;
 }
-
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -453,7 +453,7 @@ DBTechPrintTypes(mask, dolist)
 	{
 	    if (((TileType)(spointertype) p->sn_value) == i)
 	    {
-		if (dolist) 
+		if (dolist)
 		{
 		    if (firstname) keepname = p->sn_name;
 		    else if (strlen(p->sn_name) > strlen(keepname))
@@ -505,7 +505,7 @@ DBTechPrintTypes(mask, dolist)
 	    {
 		if (((TileType)(spointertype) p->sn_value) == dtp->dt_type)
 		{
-		    if (dolist) 
+		    if (dolist)
 		    {
 		        if (firstname) keepname = p->sn_name;
 		        else if (strlen(p->sn_name) > strlen(keepname))
@@ -547,7 +547,7 @@ DBTechPrintTypes(mask, dolist)
     }
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -592,9 +592,10 @@ DBTechPrintTypes(mask, dolist)
  */
 
 PlaneMask
-DBTechNoisyNameMask(layers, mask)
+DBTechNameMask0(layers, mask, noisy)
     char *layers;			/* String to be parsed. */
     TileTypeBitMask *mask;		/* Where to store the layer mask. */
+    bool noisy;				/* Whether or not to output errors */
 {
     char *p, *p2, c;
     TileTypeBitMask m2;        /* Each time around the loop, we will
@@ -652,7 +653,7 @@ DBTechNoisyNameMask(layers, mask)
 	    }
 	    save = *p2;
 	    *p2 = 0;
-	    planemask |= DBTechNoisyNameMask(p, &m2);
+	    planemask |= DBTechNameMask0(p, &m2, noisy);
 	    *p2 = save;
 	    if (save == ')') p = p2 + 1;
 	    else p = p2;
@@ -700,7 +701,10 @@ DBTechNoisyNameMask(layers, mask)
 			p++;
 		    }
 
-		    t = DBTechNoisyNameType(p);
+		    if (noisy)
+			t = DBTechNoisyNameType(p);
+		    else
+			t = DBTechNameType(p);
 		    if (t >= 0)
 			m2 = DBLayerTypeMaskTbl[t];
 
@@ -743,7 +747,10 @@ DBTechNoisyNameMask(layers, mask)
 	    while ((*p2 != 0) && (*p2 != ',')) p2 += 1;
 	    save = *p2;
 	    *p2 = 0;
-    	    plane = DBTechNoisyNamePlane(p+1);
+	    if (noisy)
+    		plane = DBTechNoisyNamePlane(p+1);
+	    else
+    		plane = DBTechNamePlane(p+1);
 	    *p2 = save;
 	    p = p2;
 	    if (plane > 0)
@@ -754,7 +761,7 @@ DBTechNoisyNameMask(layers, mask)
 	}
 	else
 	{
-	    TileType t; 
+	    TileType t;
 
 	    for (t = TT_TECHDEPBASE; t < DBNumUserLayers; t++)
 		if (TTMaskHasType(&m2, t))
@@ -775,3 +782,20 @@ DBTechNoisyNameMask(layers, mask)
     return planemask;
 }
 
+/* Wrappers for DBTechNameMask0() */
+
+PlaneMask
+DBTechNoisyNameMask(layers, mask)
+    char *layers;			/* String to be parsed. */
+    TileTypeBitMask *mask;		/* Where to store the layer mask. */
+{
+    return DBTechNameMask0(layers, mask, TRUE);
+}
+
+PlaneMask
+DBTechNameMask(layers, mask)
+    char *layers;			/* String to be parsed. */
+    TileTypeBitMask *mask;		/* Where to store the layer mask. */
+{
+    return DBTechNameMask0(layers, mask, FALSE);
+}
