@@ -4,16 +4,16 @@
  * Creation of tile and plane types and their names.
  * Lookup procedures are in DBtechname.c
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -66,11 +66,11 @@ TileTypeBitMask DBUserLayerBits;
 TileTypeBitMask DBActiveLayerBits;	/* Layers that are locked */
 TileTypeBitMask DBTechActiveLayerBits;	/* Layers marked locked in the techfile */
 
-
+
 /* Table of default, builtin planes */
 DefaultPlane dbTechDefaultPlanes[] =
 {
-    PL_CELL,		"subcell",
+    PL_ROUTER,		"router",
     PL_DRC_ERROR,	"designRuleError",
     PL_DRC_CHECK,	"designRuleCheck",
     PL_M_HINT,		"mhint",
@@ -97,7 +97,7 @@ DefaultType dbTechDefaultTypes[] =
 /* Forward declarations */
 char *dbTechNameAdd();
 NameList *dbTechNameAddOne();
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -152,7 +152,7 @@ DBTechInitPlane()
 
     DBNumPlanes = PL_TECHDEPBASE;
 }
-
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -394,7 +394,7 @@ DBTechAddAlias(sectionName, argc, argv)
     return TRUE;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -479,7 +479,7 @@ DBTechAddType(sectionName, argc, argv)
     return TRUE;
 }
 
-
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -584,8 +584,8 @@ DBTechFinalType()
     TTMaskZero(&DBTechActiveLayerBits);
     TTMaskSetMask(&DBTechActiveLayerBits, &DBActiveLayerBits);
 
-    /* Space is visible on all planes but the subcell plane */
-    TTMaskZero(&DBPlaneTypes[PL_CELL]);
+    /* Space is visible on all planes except the router */
+    TTMaskZero(&DBPlaneTypes[PL_ROUTER]);
     for (pNum = PL_PAINTBASE;  pNum < PL_MAXTYPES;  pNum++)
 	TTMaskSetOnlyType(&DBPlaneTypes[pNum], TT_SPACE);
 }
@@ -625,7 +625,7 @@ dbTechNameLookupExact(str, table)
     return ((ClientData) -2);
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -708,7 +708,7 @@ dbTechNameLookup(str, table)
 
     /*NOTREACHED*/
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -774,7 +774,7 @@ dbTechNameAdd(name, cdata, ptable, alias)
 	primary->sn_primary = TRUE;
     return (first);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
