@@ -3,21 +3,21 @@
  *
  * Management of composition rules and the paint/erase tables.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBtpaint.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBtpaint.c,v 1.2 2010/06/08 19:16:42 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -38,6 +38,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 PaintResultType DBPaintResultTbl[NP][NT][NT];
 PaintResultType DBEraseResultTbl[NP][NT][NT];
 PaintResultType DBWriteResultTbl[NT][NT];
+PaintResultType DBSpecialResultTbl[NT];
 
 PlaneMask DBTypePaintPlanesTbl[NT];
 PlaneMask DBTypeErasePlanesTbl[NT];
@@ -63,7 +64,7 @@ extern void dbTechBitTypeInit();
 
 bool dbTechAddPaintErase();
 bool dbTechSaveCompose();
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -107,7 +108,7 @@ DBTechInitCompose()
 
     stype = dtype = &(DBEraseResultTbl[0][0][0]);
     for (ps = 0; ps < TT_MAXTYPES; ps++)
-	*dtype++ = (PaintResultType)ps; 
+	*dtype++ = (PaintResultType)ps;
     for (ps = 1; ps < PL_MAXTYPES * TT_MAXTYPES; ps++)
     {
 	memcpy((void *)dtype, (void *)stype, (size_t)TT_MAXTYPES
@@ -236,8 +237,15 @@ DBTechInitCompose()
     dbSetPaintEntry(TT_CHECKSUBCELL, TT_CHECKPAINT, PCHK, TT_CHECKPAINT);
 #undef	PCHK
 #undef	PERR
+
+    /* Added 5/27/10:  Special table used for painting non-Manhattan	*/
+    /* tiles.  Uses TT_CHECKSUBCELL because that type does not exist on	*/
+    /* any paintable plane, and the checkpaint plane does not use non-	*/
+    /* manhattan tiles.							*/
+
+    for (s = 0; s < DBNumTypes; s++) DBSpecialResultTbl[s] = TT_CHECKSUBCELL;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -250,7 +258,7 @@ DBTechInitCompose()
  *
  *	bitToType[i] and bitToType[j] combine to yield bitToType[i | j]
  *
- * Also (unless composeFlag is set) erasing bitToType[j] from bitToType[i] 
+ * Also (unless composeFlag is set) erasing bitToType[j] from bitToType[i]
  * gives bitToType[i & (~j)],
  * i.e., it clears all of the j-type material out of the i-type material.
  * The bitToType[k] for which k's binary representation has only a single
@@ -294,7 +302,7 @@ dbTechBitTypeInit(bitToType, n, pNum, composeFlag)
     }
 }
 
-/* Returns nonzero if exactly one bit set */ 
+/* Returns nonzero if exactly one bit set */
 
 int
 dbIsPrimary(n)
@@ -312,8 +320,8 @@ dbIsPrimary(n)
 
     return (bitCount==1);
 }
-      
-
+
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -432,7 +440,7 @@ DBTechAddCompose(sectionName, argc, argv)
 
     return TRUE;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -512,7 +520,7 @@ dbTechSaveCompose(ruleType, t, argc, argv)
 
     return TRUE;
 }
-
+
 
 #if 0	/* deprecated function (5/11/04) */
 /*
@@ -563,7 +571,7 @@ dbTechCheckImages(t, r, s)
 }
 #endif	/* 0 */
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -674,7 +682,7 @@ dbTechAddPaintErase(type, sectionName, argc, argv)
     return TRUE;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -725,7 +733,7 @@ dbTechCheckPaint(where)
 	}
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
