@@ -5,16 +5,16 @@
  * command-level.  They do things like give information about
  * errors and print statistics.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -69,9 +69,8 @@ TileType DRCErrorType;		/* Type of error tile to paint. */
 
 /* Used by drcPrintError: */
 
-HashTable DRCErrorTable;	/* Hash table used to eliminate duplicate
-				 * error strings.
-				 */
+int *DRCErrorList;		/* List of DRC error type counts */
+HashTable DRCErrorTable;	/* Table of DRC errors and geometry */
 
 /* Global variables used by all DRC modules to record statistics.
  * For each statistic we keep two values, the count since stats
@@ -126,7 +125,7 @@ static int drcTotalVRulesHisto[DRC_MAXRULESHISTO];
 static int drcTotalHRulesHisto[DRC_MAXRULESHISTO];
 #endif	/* DRCRULESHISTO */
 
-
+
 /*
  * ----------------------------------------------------------------------------
  * drcPaintError --
@@ -183,11 +182,12 @@ drcSubstitute (cptr)
     DRCCookie * cptr;  		/* Design rule violated */
 {
     static char *why_out = NULL;
-    char *whyptr = cptr->drcc_why, *sptr, *wptr;
+    char *whyptr, *sptr, *wptr;
     int subscnt = 0, whylen;
     float oscale, value;
     extern float CIFGetOutputScale();
 
+    whyptr = DRCCurStyle->DRCWhyList[cptr->drcc_tag];
     while ((sptr = strchr(whyptr, '%')) != NULL)
     {
 	subscnt++;
@@ -195,11 +195,11 @@ drcSubstitute (cptr)
     }
     if (subscnt == 0) return whyptr;	/* No substitutions */
 
-    whyptr = cptr->drcc_why;
+    whyptr = DRCCurStyle->DRCWhyList[cptr->drcc_tag];
     whylen = strlen(whyptr) + 20 * subscnt;
     if (why_out != NULL) freeMagic(why_out);
     why_out = (char *)mallocMagic(whylen * sizeof(char));
-    strcpy(why_out, whyptr);	
+    strcpy(why_out, whyptr);
 
     oscale = CIFGetOutputScale(1000);	/* 1000 for conversion to um */
     wptr = why_out;
@@ -243,7 +243,7 @@ drcSubstitute (cptr)
     return why_out;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  * drcPrintError --
@@ -256,7 +256,7 @@ drcSubstitute (cptr)
  *
  * Side effects:
  *	DRCErrorCount is incremented.  The text associated with
- *	the error is entered into DRCErrorTable, and, if this is
+ *	the error is entered into DRCErrorList, and, if this is
  *	the first time that entry has been seen, then the error
  *	text is printed.  If the area parameter is non-NULL, then
  *	only errors intersecting that area are considered.
@@ -279,12 +279,11 @@ drcPrintError (celldef, rect, cptr, scx)
     area = &scx->scx_area;
     if ((area != NULL) && (!GEO_OVERLAP(area, rect))) return;
     DRCErrorCount += 1;
-    h = HashFind(&DRCErrorTable, cptr->drcc_why);
-    i = (spointertype) HashGetValue(h);
+
+    i = DRCErrorList[cptr->drcc_tag];
     if (i == 0)
 	TxPrintf("%s\n", drcSubstitute(cptr));
-    i++;
-    HashSetValue(h, (spointertype)i);
+    DRCErrorList[cptr->drcc_tag] = i + 1;
 }
 
 /* Same routine as above, but output goes to a Tcl list and is appended	*/
@@ -308,8 +307,7 @@ drcListError (celldef, rect, cptr, scx)
     area = &scx->scx_area;
     if ((area != NULL) && (!GEO_OVERLAP(area, rect))) return;
     DRCErrorCount += 1;
-    h = HashFind(&DRCErrorTable, cptr->drcc_why);
-    i = (spointertype) HashGetValue(h);
+    i = DRCErrorList[cptr->drcc_tag];
     if (i == 0)
     {
 	Tcl_Obj *lobj;
@@ -318,8 +316,7 @@ drcListError (celldef, rect, cptr, scx)
 			Tcl_NewStringObj(drcSubstitute(cptr), -1));
 	Tcl_SetObjResult(magicinterp, lobj);
     }
-    i += 1;
-    HashSetValue(h, (spointertype)i);
+    DRCErrorList[cptr->drcc_tag] = i + 1;
 }
 
 /* Same routine as above, but output for every single error is recorded	*/
@@ -347,7 +344,7 @@ drcListallError (celldef, rect, cptr, scx)
     lobj = (Tcl_Obj *) HashGetValue(h);
     if (lobj == NULL)
        lobj = Tcl_NewListObj(0, NULL);
-    
+
     pobj = Tcl_NewListObj(0, NULL);
 
     Tcl_ListObjAppendElement(magicinterp, pobj, Tcl_NewIntObj(r.r_xbot));
@@ -364,7 +361,7 @@ drcListallError (celldef, rect, cptr, scx)
 #define drcListError drcPrintError
 
 #endif
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -441,7 +438,7 @@ DRCPrintStats()
     }
 #endif	/* DRCRULESHISTO */
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -475,11 +472,15 @@ DRCWhy(dolist, use, area)
 {
     SearchContext scx;
     Rect box;
+    int i;
     extern int drcWhyFunc();		/* Forward reference. */
 
-    /* Create a hash table to used for eliminating duplicate messages. */
+    /* Create a hash table to eliminate duplicate messages. */
+
+    DRCErrorList = (int *)mallocMagic((DRCCurStyle->DRCWhySize + 1) * sizeof(int));
+    for (i = 0; i <= DRCCurStyle->DRCWhySize; i++)
+	DRCErrorList[i] = 0;
 
-    HashInit(&DRCErrorTable, 16, HT_STRINGKEYS);
     DRCErrorCount = 0;
     box = DRCdef->cd_bbox;
 
@@ -494,16 +495,13 @@ DRCWhy(dolist, use, area)
     drcWhyFunc(&scx, (pointertype)dolist);
     UndoEnable();
 
-    /* Delete the hash table now that we're finished (otherwise there
-     * will be a core leak.
-     */
-	
-    HashKill(&DRCErrorTable);
+    /* Delete the error list */
+    freeMagic(DRCErrorList);
 
     /* Redisplay the DRC yank definition in case anyone is looking
      * at it.
      */
-    
+
     DBReComputeBbox(DRCdef);
     (void) GeoInclude(&DRCdef->cd_bbox, &box);
     DBWAreaChanged(DRCdef, &box, DBW_ALLWINDOWS, &DBAllButSpaceBits);
@@ -532,7 +530,7 @@ DRCWhyAll(use, area, fout)
     HashEntry	*he;
     Tcl_Obj *lobj, *robj;
 
-    /* Create a hash table to used for eliminating duplicate messages. */
+    /* Create a hash table for storing all of the results */
 
     HashInit(&DRCErrorTable, 16, HT_STRINGKEYS);
     DRCErrorCount = 0;
@@ -552,30 +550,27 @@ DRCWhyAll(use, area, fout)
     /* Generate results */
 
     robj = Tcl_NewListObj(0, NULL);
-    
+
     HashStartSearch(&hs);
     while ((he = HashNext(&DRCErrorTable, &hs)) != (HashEntry *)NULL)
     {
 	lobj = (Tcl_Obj *)HashGetValue(he);
 	if (lobj != NULL)
 	{
-	    Tcl_ListObjAppendElement(magicinterp, robj, 
+	    Tcl_ListObjAppendElement(magicinterp, robj,
 			Tcl_NewStringObj((char *)he->h_key.h_name, -1));
 	    Tcl_ListObjAppendElement(magicinterp, robj, lobj);
 	}
     }
     Tcl_SetObjResult(magicinterp, robj);
 
-    /* Delete the hash table now that we're finished (otherwise there
-     * will be a core leak.
-     */
-	
+    /* Delete the error table now that we're finished */
     HashKill(&DRCErrorTable);
 
     /* Redisplay the DRC yank definition in case anyone is looking
      * at it.
      */
-    
+
     DBReComputeBbox(DRCdef);
     (void) GeoInclude(&DRCdef->cd_bbox, &box);
     DBWAreaChanged(DRCdef, &box, DBW_ALLWINDOWS, &DBAllButSpaceBits);
@@ -614,14 +609,14 @@ drcWhyFunc(scx, cdarg)
     bool dolist = (bool)((pointertype)cdarg);
 
     /* Check paint and interactions in this subcell. */
-    
+
     (void) DRCInteractionCheck(def, &scx->scx_area, &scx->scx_area,
 		(dolist) ? drcListError : drcPrintError,
 		(ClientData) scx);
     (void) DRCArrayCheck(def, &scx->scx_area,
 		(dolist) ? drcListError : drcPrintError,
 		(ClientData) scx);
-    
+
     /* New behavior:  Don't search children, instead propagate errors up. */
     /* (void) DBCellSrArea(scx, drcWhyFunc, (ClientData)cdarg); */
 
@@ -638,12 +633,12 @@ drcWhyAllFunc(scx, cdarg)
     CellDef *def = scx->scx_use->cu_def;
 
     /* Check paint and interactions in this subcell. */
-    
+
     (void) DRCInteractionCheck(def, &scx->scx_area, &scx->scx_area,
 		drcListallError, (ClientData)scx);
     (void) DRCArrayCheck(def, &scx->scx_area,
 		drcListallError, (ClientData)scx);
-    
+
     /* New behavior:  Don't search children, instead propagate errors up. */
     /* (void) DBCellSrArea(scx, drcWhyAllFunc, (ClientData)cdarg); */
 
@@ -723,12 +718,12 @@ drcCheckFunc(scx, cdarg)
     /* As a special performance hack, if the complete cell area is
      * handled here, don't bother to look at any more array elements.
      */
-    
+
     if (GEO_SURROUND(&cellArea, &def->cd_bbox))
 	return 2;
     else return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -747,31 +742,114 @@ drcCheckFunc(scx, cdarg)
  * ----------------------------------------------------------------------------
  */
 
-int
-DRCCount(use, area)
+DRCCountList *
+DRCCount(use, area, recurse)
     CellUse *use;		/* Top-level use of hierarchy. */
     Rect *area;			/* Area in which violations are counted. */
+    bool recurse;		/* If TRUE, count errors in all subcells */
 {
+    DRCCountList  *dcl, *newdcl;
+    HashTable	  dupTable;
+    HashEntry	  *he;
+    HashSearch	  hs;
     int		  count;
     SearchContext scx;
     CellDef	  *def;
-    extern int drcCountFunc2();
+    extern int drcCountFunc();
+
+    /* Shouldn't happen? */
+    if (!(use->cu_def->cd_flags & CDAVAILABLE)) return NULL;
+
+    /* Use a hash table to make sure that we don't output information
+     * for any cell more than once.
+     */
+
+    HashInit(&dupTable, 16, HT_WORDKEYS);
+
+    /* Clearing CDAVAILABLE from cd_flags keeps the count from recursing */
+    if (recurse == FALSE)
+	use->cu_def->cd_flags &= ~CDAVAILABLE;
 
     scx.scx_use = use;
     scx.scx_x = use->cu_xlo;
     scx.scx_y = use->cu_ylo;
     scx.scx_area = *area;
     scx.scx_trans = GeoIdentityTransform;
+    (void) drcCountFunc(&scx, &dupTable);
 
-    def = use->cu_def;
+    /* Create the list from the hash table */
+
+    dcl = NULL;
+    if (dupTable.ht_table != (HashEntry **) NULL)
+    {
+	HashStartSearch(&hs);
+	while ((he = HashNext(&dupTable, &hs)) != (HashEntry *)NULL)
+	{
+	    count = (spointertype)HashGetValue(he);
+	    if (count > 1)
+	    {
+		newdcl = (DRCCountList *)mallocMagic(sizeof(DRCCountList));
+		newdcl->dcl_count = count - 1;
+		newdcl->dcl_def = (CellDef *)he->h_key.h_ptr;
+		newdcl->dcl_next = dcl;
+		dcl = newdcl;
+	    }
+	}
+    }
+    HashKill(&dupTable);
+
+    /* Restore the CDAVAILABLE flag */
+    if (recurse == FALSE)
+	use->cu_def->cd_flags |= CDAVAILABLE;
+
+    return dcl;
+}
+
+int
+drcCountFunc(scx, dupTable)
+    SearchContext *scx;
+    HashTable *dupTable;	    /* Passed as client data, used to
+				     * avoid searching any cell twice.
+				     */
+{
+    int count;
+    HashEntry *h;
+    CellDef *def;
+    extern int drcCountFunc2();
+
+    /* If we've already seen this cell definition before, then skip it
+     * now.
+     */
+
+    def = scx->scx_use->cu_def;
+    h = HashFind(dupTable, (char *)def);
+    if (HashGetValue(h) != 0) goto done;
+    HashSetValue(h, 1);
 
     /* Count errors in this cell definition by scanning the error plane. */
 
     count = 0;
     (void) DBSrPaintArea((Tile *) NULL, def->cd_planes[PL_DRC_ERROR],
 	&def->cd_bbox, &DBAllButSpaceBits, drcCountFunc2, (ClientData)(&count));
+    HashSetValue(h, (spointertype)count + 1);
 
-    return count;
+    /* Ignore children that have not been loaded---we will only report	*/
+    /* errors that can be seen.  This avoids immediately loading and	*/
+    /* drc processing large layouts simply because we asked for an	*/
+    /* error count.  When the cell is loaded, drc will be checked	*/
+    /* anyway, and the count can be updated in response to that check.	*/
+
+    if ((scx->scx_use->cu_def->cd_flags & CDAVAILABLE) == 0) return 0;
+
+    /* Scan children recursively. */
+
+    DBCellSrArea(scx, drcCountFunc, (ClientData)dupTable);
+
+    /* As a special performance hack, if the complete cell area is
+     * handled here, don't bother to look at any more array elements.
+     */
+    done: if (GEO_SURROUND(&scx->scx_area, &def->cd_bbox)) return 2;
+    else return 0;
 }
 
 int
@@ -810,7 +888,7 @@ DRCCatchUp()
     DRCContinuous();
     DRCBackGround = background;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -850,7 +928,7 @@ DRCFind(use, area, rect, indx)
 {
     SearchContext scx;
     Sindx finddata;
-    Rect trect;	
+    Rect trect;
     int result;
     int drcFindFunc();
     HashTable defTable;
@@ -911,7 +989,7 @@ drcFindFunc(scx, finddata)
 	finddata->trans = scx->scx_trans;
 	return 1;
     }
-    
+
     /* New behavior:  Don't search children, instead propagate errors up. */
     /* return DBCellSrArea(scx, drcFindFunc, (ClientData)finddata); */
     return 0;
