@@ -5,16 +5,16 @@
  * violations that occur as a result of interactions between
  * subcells and either paint or other subcells.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -43,9 +43,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 static Rect drcSubIntArea;	/* Accumulates area of interactions. */
 static CellDef *drcSubDef;	/* Cell definition we're checking. */
 static int drcSubRadius;	/* Interaction radius. */
-static CellTileBody *drcCurSub;	/* Holds current tile when checking to see
-				 * if more than one use in an area.
-				 */
+static CellUse *drcCurSub;	/* Holds current use while searching for interactions */
 static Rect drcSubLookArea;	/* Area where we're looking for interactions */
 static void (*drcSubFunc)();	/* Error function. */
 static ClientData drcSubClientData;
@@ -59,63 +57,51 @@ static DRCCookie drcSubcellCookie = {
     0, 0, 0, 0,
     { 0 }, { 0 },
     0, 0, 0,
-    "This layer can't abut or partially overlap between subcells",
+    DRC_SUBCELL_OVERLAP_TAG,
     (DRCCookie *) NULL
 };
 
 extern int DRCErrorType;
 
-
 /*
  * ----------------------------------------------------------------------------
  *
  * drcFindOtherCells --
  *
- * 	This is a search function invoked when looking around a given
- *	cell for interactions.  If a tile is found other than drcCurSub,
- *	then it constitutes an interaction, and its area is included
- *	into the area parameter.
+ *	This is a search function invoked when looking around a given
+ *	cell for interactions.  If a cell use is found other than drcCurSub,
+ *	then it constitutes an interaction, and its area is included into
+ *	the area parameter.
  *
  * Results:
  *	Always returns 0 to keep the search alive.
  *
  * Side effects:
  *	The area parameter may be modified by including the area
- *	of the current tile.
+ *	of the current use.
  *
  * ----------------------------------------------------------------------------
  */
 
 int
-drcFindOtherCells(tile, area)
-    Tile *tile;			/* Tile in subcell plane. */
-    Rect *area;			/* Area in which to include interactions. */
-{
-    Rect r;
+drcFindOtherCells(use, area)
     CellUse *use;
-    CellTileBody *ctbptr = (CellTileBody *) tile->ti_body;
+    Rect *area;
+{
+    if (use != drcCurSub)
+    	GeoInclude(&use->cu_bbox, area);
 
-    /* XXX */
-    /* if (ctbptr == NULL) return 0; */
-
-    if (ctbptr == drcCurSub) return 0;
-
-    for (ctbptr = (CellTileBody *) TiGetBody(tile); ctbptr != NULL;
-		ctbptr = ctbptr->ctb_next)
-    {
-	use = ctbptr->ctb_use;
-	GeoInclude(&use->cu_bbox, &r);
-    }
     return 0;
 }
-
+
+
 /*
  * ----------------------------------------------------------------------------
  *
- * drcSubcellTileFunc --
+ * drcSubcellFunc --
  *
- * 	Called by TiSrArea when looking for interactions in
- *	a given area.  It sees if this subcell tile participates
+ * 	Called by DBSrCellPlaneArea when looking for interactions in
+ *	a given area.  It sees if this subcell participates
  *	in any interactions in the area we're rechecking.
  *
  * Results:
@@ -129,36 +115,23 @@ drcFindOtherCells(tile, area)
  */
 
 int
-drcSubcellTileFunc(tile, propagate)
-    Tile *tile;			/* Subcell tile. */
+drcSubcellFunc(subUse, propagate)
+    CellUse *subUse;		/* Subcell instance. */
     bool *propagate;		/* Errors to propagate up */
 {
     Rect area, haloArea, intArea, subIntArea, locIntArea;
     int i;
-    CellTileBody *ctbptr = (CellTileBody *) tile->ti_body;
-    CellUse *subUse;
-
-    if (ctbptr == NULL) return 0;
 
     /* To determine interactions, find the bounding box of
      * all paint and other subcells within one halo of this
-     * subcell tile (and also within the original area where
+     * subcell (and also within the original area where
      * we're recomputing errors).
      */
 
-    /* XXX This is not right---the area of the cell plane tile is not	*/
-    /* the area of the subcell.						*/
-    /* TiToRect(tile, &area); */
+    area = subUse->cu_bbox;
 
-    for (ctbptr = (CellTileBody *) TiGetBody(tile); ctbptr != NULL;
-		ctbptr = ctbptr->ctb_next)
-    {
-	subUse = ctbptr->ctb_use;
-	area = subUse->cu_bbox;
-
-	GEO_EXPAND(&area, drcSubRadius, &haloArea);
-	GeoClip(&haloArea, &drcSubLookArea);
-    }
+    GEO_EXPAND(&area, drcSubRadius, &haloArea);
+    GeoClip(&haloArea, &drcSubLookArea);
 
     intArea = GeoNullRect;
     for (i = PL_TECHDEPBASE; i < DBNumPlanes; i++)
@@ -172,23 +145,18 @@ drcSubcellTileFunc(tile, propagate)
     /* interaction area of the parent.	Ultimately this is recursive as	*/
     /* all cells are checked and errors propagate to the top level.	*/
 
-    /* XXX */
-    for (ctbptr = (CellTileBody *) TiGetBody(tile); ctbptr != NULL;
-		ctbptr = ctbptr->ctb_next)
-    {
-	subUse = ctbptr->ctb_use;
-	subIntArea = GeoNullRect;
-	DBSrPaintArea((Tile *) NULL, subUse->cu_def->cd_planes[PL_DRC_ERROR],
+    subIntArea = GeoNullRect;
+    DBSrPaintArea((Tile *) NULL, subUse->cu_def->cd_planes[PL_DRC_ERROR],
 		&TiPlaneRect, &DBAllButSpaceBits, drcIncludeArea,
 		(ClientData) &subIntArea);
-	GeoTransRect(&(subUse->cu_transform), &subIntArea, &locIntArea);
-	GeoInclude(&locIntArea, &intArea);
-	if (!GEO_RECTNULL(&subIntArea)) *propagate = TRUE;
-    }
 
-    drcCurSub = ctbptr;
-    (void) TiSrArea((Tile *) NULL, drcSubDef->cd_planes[PL_CELL],
-	&haloArea, drcFindOtherCells, (ClientData) &intArea);
+    GeoTransRect(&(subUse->cu_transform), &subIntArea, &locIntArea);
+    GeoInclude(&locIntArea, &intArea);
+    if (!GEO_RECTNULL(&subIntArea)) *propagate = TRUE;
+
+    drcCurSub = subUse;
+    (void) DBSrCellPlaneArea(drcSubDef->cd_cellPlane, &haloArea,
+		drcFindOtherCells, (ClientData)(&intArea));
     if (GEO_RECTNULL(&intArea)) return 0;
 
     GEO_EXPAND(&intArea, drcSubRadius, &intArea);
@@ -196,7 +164,7 @@ drcSubcellTileFunc(tile, propagate)
     (void) GeoInclude(&intArea, &drcSubIntArea);
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -220,7 +188,7 @@ drcAlwaysOne()
 {
     return 1;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -270,7 +238,7 @@ drcSubCheckPaint(scx, curUse)
     }
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -331,15 +299,15 @@ DRCFindInteractions(def, area, radius, interaction)
     drcSubIntArea = GeoNullRect;
     GEO_EXPAND(area, radius, &drcSubLookArea);
     propagate = FALSE;
-    (void) TiSrArea((Tile *) NULL, def->cd_planes[PL_CELL],
-	&drcSubLookArea, drcSubcellTileFunc, (ClientData)(&propagate));
-    
+    (void) DBSrCellPlaneArea(def->cd_cellPlane, &drcSubLookArea,
+		drcSubcellFunc, (ClientData)(&propagate));
+
     /* If there seems to be an interaction area, make a second pass
      * to make sure there's more than one cell with paint in the
      * area.  This will save us a lot of work where two cells
      * have overlapping bounding boxes without overlapping paint.
      */
-    
+
     if (GEO_RECTNULL(&drcSubIntArea)) return FALSE;
     use = NULL;
 
@@ -366,13 +334,13 @@ DRCFindInteractions(def, area, radius, interaction)
     }
 
     /* OK, no more excuses, there's really an interaction area here. */
-    
+
     *interaction = drcSubIntArea;
     GeoClip(interaction, area);
     if (GEO_RECTNULL(interaction)) return FALSE;
     return TRUE;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -409,7 +377,7 @@ drcExactOverlapCheck(tile, arg)
     (*(arg->dCD_errors))++;
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -448,7 +416,7 @@ drcExactOverlapTile(tile, cxp)
     Tile *tp;
     Rect r1, r2, r3, rex;
     int i;
-    
+
     arg = (struct drcClientData *) cxp->tc_filter->tf_arg;
     TiToRect(tile, &r1);
     GeoTransRect(&(cxp->tc_scx->scx_trans), &r1, &r2);
@@ -537,7 +505,7 @@ drcExactOverlapTile(tile, cxp)
     }
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -596,7 +564,7 @@ DRCInteractionCheck(def, area, erasebox, func, cdarg)
     /* Divide the area to be checked up into squares.  Process each
      * square separately.
      */
-    
+
     x = (area->r_xbot/DRCStepSize) * DRCStepSize;
     if (x > area->r_xbot) x -= DRCStepSize;
     y = (area->r_ybot/DRCStepSize) * DRCStepSize;
@@ -609,7 +577,7 @@ DRCInteractionCheck(def, area, erasebox, func, cdarg)
 	    square.r_xtop = square.r_xbot + DRCStepSize;
 	    square.r_ytop = square.r_ybot + DRCStepSize;
 
-	    /* Limit square to area.  Otherwise, a huge processing	*/
+	    /* Limit square to erasebox.  Otherwise, a huge processing	*/
 	    /* penalty is incurred for finding a single error (e.g.,	*/
 	    /* using "drc find" or "drc why" in a large design with a	*/
 	    /* large step size.						*/
@@ -623,7 +591,7 @@ DRCInteractionCheck(def, area, erasebox, func, cdarg)
 	    if (!DRCFindInteractions(def, &cliparea, DRCTechHalo, &intArea))
 	    {
 		/* Added May 4, 2008---if there are no subcells, run the
-		 * basic check over the area of the square.
+		 * basic check over the area of the erasebox.
 		 */
 		subArea = *erasebox;
 		GeoClip(&subArea, &cliparea);
@@ -631,7 +599,7 @@ DRCInteractionCheck(def, area, erasebox, func, cdarg)
 
 		errorSaveType = DRCErrorType;
 		DRCErrorType = TT_ERROR_P;	// Basic check is always ERROR_P
-                DRCBasicCheck(def, &intArea, &subArea, func, cdarg); 
+                DRCBasicCheck(def, &intArea, &subArea, func, cdarg);
 		DRCErrorType = errorSaveType;
 		continue;
 	    }
@@ -663,16 +631,16 @@ DRCInteractionCheck(def, area, erasebox, func, cdarg)
 		{
 		    subArea.r_ybot = intArea.r_ytop;
 		    GEO_EXPAND(&subArea, DRCTechHalo, &eraseHalo);
-                    DRCBasicCheck(def, &eraseHalo, &subArea, func, cdarg); 
-		} 
+                    DRCBasicCheck(def, &eraseHalo, &subArea, func, cdarg);
+		}
 		/* check below */
 		if (intArea.r_ybot > eraseClip.r_ybot)
 		{
 		    subArea.r_ybot = eraseClip.r_ybot;
 		    subArea.r_ytop = intArea.r_ybot;
 		    GEO_EXPAND(&subArea, DRCTechHalo, &eraseHalo);
-                    DRCBasicCheck(def, &eraseHalo, &subArea, func, cdarg); 
-		} 
+                    DRCBasicCheck(def, &eraseHalo, &subArea, func, cdarg);
+		}
 		subArea.r_ytop = intArea.r_ytop;
 		subArea.r_ybot = intArea.r_ybot;
 
@@ -681,7 +649,7 @@ DRCInteractionCheck(def, area, erasebox, func, cdarg)
 		{
 		    subArea.r_xbot = intArea.r_xtop;
 		    GEO_EXPAND(&subArea, DRCTechHalo, &eraseHalo);
-                    DRCBasicCheck(def, &eraseHalo, &subArea, func, cdarg); 
+                    DRCBasicCheck(def, &eraseHalo, &subArea, func, cdarg);
 		}
 		/* check left */
 		if (intArea.r_xbot > eraseClip.r_xbot)
@@ -689,11 +657,17 @@ DRCInteractionCheck(def, area, erasebox, func, cdarg)
 		    subArea.r_xtop = intArea.r_xbot;
 		    subArea.r_xbot = eraseClip.r_xbot;
 		    GEO_EXPAND(&subArea, DRCTechHalo, &eraseHalo);
-                    DRCBasicCheck(def, &eraseHalo, &subArea, func, cdarg); 
+                    DRCBasicCheck(def, &eraseHalo, &subArea, func, cdarg);
 		}
 		DRCErrorType = errorSaveType;
 	    }
-    
+
+ 	    /* Clip interaction area against subArea-expanded-by-halo */
+
+	    subArea = *erasebox;
+	    GEO_EXPAND(&subArea, DRCTechHalo, &cliparea);
+	    GeoClip(&intArea, &cliparea);
+
 	    /* Flatten the interaction area. */
 
 	    DRCstatInteractions += 1;
@@ -734,7 +708,7 @@ DRCInteractionCheck(def, area, erasebox, func, cdarg)
 	    (void) DBTreeSrUniqueTiles(&scx, &DRCCurStyle->DRCExactOverlapTypes,
 			0, drcExactOverlapTile, (ClientData) &arg);
 	}
-    
+
     /* Update count of interaction tiles processed. */
 
     DRCstatIntTiles += DRCstatTiles - oldTiles;
