@@ -3,16 +3,16 @@
  *
  * Definitions for the DRC module.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  *
  * Needs to include: magic.h database.h
@@ -39,10 +39,15 @@ typedef struct drccookie
     int		      drcc_edgeplane;	/* Plane of edge */
     int		      drcc_plane;	/* Index of plane on which to check
 					 * legal types. */
-    char              *drcc_why;	/* Explanation of error found */
+    int 	      drcc_tag;		/* Tag to explanation of error found */
     struct drccookie  *drcc_next;
 } DRCCookie;
 
+/* These DRC tags in DRCcookie are predefined. */
+#define DRC_ARRAY_OVERLAP_TAG	1
+#define DRC_OVERLAP_TAG		2
+#define DRC_SUBCELL_OVERLAP_TAG	3
+
 /* *This is size "int" because it holds an area for DRC_AREA rules,	  */
 /* and therefore may have twice the bit length of a normal rule distance. */
 
@@ -98,7 +103,7 @@ struct drcClientData
     int		  dCD_plane;
     Rect	* dCD_rect;
     Tile 	* dCD_initial;		/* Initial tile for search (left side
-					 * for forward rules, right for reverse 
+					 * for forward rules, right for reverse
 					 * rules).
 					 */
     Rect	* dCD_clip;		/* Clip error tiles against this. */
@@ -125,6 +130,15 @@ typedef struct drcpendingcookie
     struct drcpendingcookie *dpc_next;
 } DRCPendingCookie;
 
+/* Structure used to pass back lists of cell definitions and error tile counts */
+
+typedef struct drccountlist
+{
+    CellDef             *dcl_def;
+    int                 dcl_count;
+    struct drccountlist *dcl_next;
+} DRCCountList;
+
 /* Structure used to keep information about the current DRC style */
 
 typedef struct drckeep
@@ -133,22 +147,6 @@ typedef struct drckeep
     char		*ds_name;
 } DRCKeep;
 
-/*
- * DRC "why" strings are potentially referred to hundreds of times by
- * DRC cookies in the rule table.  Rather than creating hundreds of
- * copies of each string, we create just one copy and let all the cookies
- * point to that one copy.
- *
- * Since we can't free these shared "why" strings when we delete a cookie,
- * we keep a list of these strings and free them all when convenient.
- */
-    
-typedef struct drcwhylist
-{
-    char                * dwl_string;
-    struct drcwhylist   * dwl_next;
-} drcWhyList;      
- 
 /*
  * Structure defining a DRC style
  */
@@ -164,7 +162,8 @@ typedef struct drcstyle
     int			DRCTechHalo;	/* largest action distance of design rules */
     int			DRCStepSize;	/* chunk size for decomposing large areas */
     char		DRCFlags;	/* Option flags */
-    drcWhyList		*DRCWhyList;
+    char		**DRCWhyList;	/* Indexed list of "why" text strings */
+    int			DRCWhySize;	/* Length of DRCWhyList */
     PaintResultType	DRCPaintTable[NP][NT][NT];
 } DRCStyle;
 
@@ -221,7 +220,7 @@ extern DRCStyle *DRCCurStyle;	/* Current DRC style in effect */
 extern CellDef  *DRCdef;	/* Current cell being checked for DRC */
 extern CellUse  *DRCuse, *DRCDummyUse;
 
-/* 
+/*
  * Internal procedures
  */
 
@@ -232,6 +231,7 @@ extern int drcExactOverlapTile();
 extern void drcInitRulesTbl();
 extern void drcAssign();
 extern void drcCifAssign();
+extern int drcWhyCreate();
 
 /*
  * Exported procedures
@@ -239,6 +239,7 @@ extern void drcCifAssign();
 
 extern int DRCGetDefaultLayerWidth();
 extern int DRCGetDefaultLayerSpacing();
+extern int DRCGetDefaultWideLayerSpacing();
 extern int DRCGetDefaultLayerSurround();
 
 extern void DRCTechInit();
@@ -258,7 +259,7 @@ extern void DRCPrintRulesTable();
 extern void DRCWhy();
 extern void DRCPrintStats();
 extern void DRCCheck();
-extern int DRCCount();
+extern DRCCountList *DRCCount();
 extern int DRCFind();
 extern void DRCCatchUp();
 extern bool DRCFindInteractions();
