@@ -37,6 +37,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include "textio/txcommands.h"
 #endif
 #include "extflat/extflat.h"
+#include "extflat/EFint.h"
 #include "extract/extract.h"	/* for extDevTable */
 #include "utils/runstats.h"
 #include "utils/malloc.h"
@@ -97,12 +98,12 @@ struct {
 } fetInfo[MAXDEVTYPES];
 
 typedef struct {
-	long    visitMask:MAXDEVTYPES; 
+	long    visitMask:MAXDEVTYPES;
 } nodeClient;
 
 typedef struct {
 	HierName *lastPrefix;
-	long    visitMask:MAXDEVTYPES; 
+	long    visitMask:MAXDEVTYPES;
 } nodeClientHier;
 
 #define NO_RESCLASS	-1
@@ -204,7 +205,7 @@ Exttosim_Init(interp)
     if (interp == NULL) return TCL_ERROR;
     if (Tcl_PkgRequire(interp, "Tclmagic", MAGIC_VERSION, 0) == NULL)
 	return TCL_ERROR;
-    if (Tcl_InitStubs(interp, "8.1", 0) == NULL) return TCL_ERROR;
+    if (Tcl_InitStubs(interp, "8.5", 0) == NULL) return TCL_ERROR;
 
     TxPrintf("Auto-loading EXTTOSIM module\n");
     TxFlushOut();
@@ -225,7 +226,7 @@ Exttosim_Init(interp)
 
 #endif /* EXT2SIM_AUTO */
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -257,7 +258,7 @@ CmdExtToSim(w, cmd)
     int value;
     int option = EXTTOSIM_RUN;
     int argc = cmd->tx_argc;
-    char **argv = cmd->tx_argv; 
+    char **argv = cmd->tx_argv;
     char **msg;
     bool err_result;
 
@@ -371,7 +372,7 @@ CmdExtToSim(w, cmd)
 		LocCapThreshold = INFINITE_THRESHOLD_F;
 	    else if (StrIsNumeric(cmd->tx_argv[2]))
 		LocCapThreshold = atoCap(cmd->tx_argv[2]);
-	    else 
+	    else
 		TxError("exttosim: numeric value or \"infinite\" expected.\n");
 	    break;
 	case EXTTOSIM_RTHRESH:
@@ -389,7 +390,7 @@ CmdExtToSim(w, cmd)
 		LocResistThreshold = atoi(cmd->tx_argv[2]);
 	    else if (!strncmp(cmd->tx_argv[2], "inf", 3))
 		LocResistThreshold = INFINITE_THRESHOLD;
-	    else 
+	    else
 		TxError("exttosim: integer value or \"infinite\" expected.\n");
 	    break;
 	case EXTTOSIM_FORMAT:
@@ -439,7 +440,7 @@ CmdExtToSim(w, cmd)
 		    break;
 	    }
 	    break;
-	    
+
 	case EXTTOSIM_DEFAULT:
 	    LocCapThreshold = 2;
 	    LocResistThreshold = 10;
@@ -492,7 +493,7 @@ runexttosim:
     if (inName == NULL)
     {
 	/* Assume that we want to do exttospice on the currently loaded cell */
-	
+
 	if (w == (MagWindow *) NULL)
 	    windCheckOnlyWindow(&w, DBWclientID);
 
@@ -601,7 +602,7 @@ runexttosim:
     /* Write the output file */
 
     fprintf(esSimF, "| units: %g tech: %s format: %s\n", EFScale, EFTech,
-		   (esFormat == MIT) ? "MIT" :  
+		   (esFormat == MIT) ? "MIT" :
 		      ( (esFormat == LBL) ? "LBL" : "SU" ) );
 
     /* Convert the hierarchical description to a flat one */
@@ -730,7 +731,7 @@ main(argc, argv)
     }
 
     fprintf(esSimF, "| units: %g tech: %s format: %s\n", EFScale, EFTech,
-		   (esFormat == MIT) ? "MIT" :  
+		   (esFormat == MIT) ? "MIT" :
 		      ( (esFormat == LBL) ? "LBL" : "SU" ) );
 
     /* Convert the hierarchical description to a flat one */
@@ -769,7 +770,7 @@ main(argc, argv)
 }
 
 #endif		/* MAGIC_WRAPPER */
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -891,8 +892,8 @@ simmainArgs(pargc, pargv)
 	    fetInfo[ndx].resClassSub = rClassSub;
 	    fetInfo[ndx].defSubs = (char *) mallocMagic((unsigned) (strlen(subsNode)+1));
 	    strcpy(fetInfo[ndx].defSubs,subsNode);
-	    TxError("Info: fet %s(%d) sdRclass=%d subRclass=%d dSub=%s\n", 
-	    	cp, ndx, fetInfo[ndx].resClassSD, fetInfo[ndx].resClassSub, 
+	    TxError("Info: fet %s(%d) sdRclass=%d subRclass=%d dSub=%s\n",
+	    	cp, ndx, fetInfo[ndx].resClassSD, fetInfo[ndx].resClassSub,
             	fetInfo[ndx].defSubs);
 	    break;
 	    }
@@ -924,7 +925,7 @@ usage:
     exit (1);
 #endif
 }
-
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -949,7 +950,7 @@ HierName *suffix;
 	return(((EFNodeName *) HashGetValue(he))->efnn_node);
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -981,9 +982,9 @@ HierName *suffix;
  */
 
 int
-simdevVisit(dev, hierName, scale, trans)
+simdevVisit(dev, hc, scale, trans)
     Dev *dev;		/* Device being output */
-    HierName *hierName;	/* Hierarchical path down to this device */
+    HierContext *hc;	/* Hierarchical context down to this device */
     float scale;	/* Scale transform for output */
     Transform *trans;	/* Coordinate transform */
 {
@@ -992,6 +993,7 @@ simdevVisit(dev, hierName, scale, trans)
     int l, w;
     Rect r;
     char name[12];
+    HierName *hierName = hc->hc_hierName;
 
     sprintf(name, "output");
 
@@ -1012,6 +1014,7 @@ simdevVisit(dev, hierName, scale, trans)
 
     /* Computation of length and width has been moved from efVisitDevs */
     EFGetLengthAndWidth(dev, &l, &w);
+    if (esMergeDevsA || esMergeDevsC) w *= getCurDevMult();
 
     /* If only two terminals, connect the source to the drain */
     gate = &dev->dev_terms[0];
@@ -1061,13 +1064,13 @@ simdevVisit(dev, hierName, scale, trans)
     if (dev->dev_class == DEV_BJT && subnode)
     {
         sprintf(name, "fet");
-        simdevOutNode(hierName, subnode->efnode_name->efnn_hier, name, esSimF); 
+        simdevOutNode(hierName, subnode->efnode_name->efnn_hier, name, esSimF);
     }
     else if ((dev->dev_class == DEV_DIODE || dev->dev_class == DEV_PDIODE ||
 		dev->dev_class == DEV_NDIODE) && dev->dev_nterm == 1 && subnode)
     {
         sprintf(name, "fet");
-        simdevOutNode(hierName, subnode->efnode_name->efnn_hier, name, esSimF); 
+        simdevOutNode(hierName, subnode->efnode_name->efnn_hier, name, esSimF);
     }
     else if (dev->dev_nterm > 2)
         simdevOutNode(hierName, drain->dterm_node->efnode_name->efnn_hier, name, esSimF);
@@ -1077,7 +1080,7 @@ simdevVisit(dev, hierName, scale, trans)
     {
 	putc(' ', esSimF);
 	simdevSubstrate(hierName, subnode->efnode_name->efnn_hier,
-	             dev->dev_type, 0, FALSE, esSimF);
+	             dev->dev_type, 0.0, FALSE, esSimF);
     }
 
     GeoTransRect(trans, &dev->dev_rect, &r);
@@ -1090,7 +1093,7 @@ simdevVisit(dev, hierName, scale, trans)
 	* comparator will deal with any problem arising from
 	* swapping L and W.
 	*/
-	
+
 	int n;
 	double cl, cw;
 	double chp = 0.0;
@@ -1157,7 +1160,7 @@ simdevVisit(dev, hierName, scale, trans)
 	   if ( esFormat == SU ) {
 	       fprintf(esSimF, "%s", (source->dterm_attrs) ? "," : " s=" );
 	       if (hierS)
-	         simnAPHier(source, hierName, fetInfo[dev->dev_type].resClassSD, 
+	         simnAPHier(source, hierName, fetInfo[dev->dev_type].resClassSD,
 		      scale, esSimF);
 	       else {
 	         snode= SimGetNode(hierName,
@@ -1174,13 +1177,13 @@ simdevVisit(dev, hierName, scale, trans)
 	   }
 	   if ( esFormat == SU ) {
 	       fprintf(esSimF, "%s", (drain->dterm_attrs) ? "," : " d=" );
-	       if (hierD) 
-	         simnAPHier(drain, hierName, fetInfo[dev->dev_type].resClassSD, 
+	       if (hierD)
+	         simnAPHier(drain, hierName, fetInfo[dev->dev_type].resClassSD,
 		      scale, esSimF);
 	       else {
 	         dnode = SimGetNode(hierName,
 			      drain->dterm_node->efnode_name->efnn_hier);
-	         simnAP(dnode, fetInfo[dev->dev_type].resClassSD, 
+	         simnAP(dnode, fetInfo[dev->dev_type].resClassSD,
 		      scale, esSimF);
 	       }
 	   }
@@ -1195,7 +1198,8 @@ int
 simdevSubstrate( prefix, suffix, type, scale, doAP, outf)
 HierName *prefix;
 HierName *suffix;
-int type, scale;
+int type;
+float scale;
 bool doAP;
 FILE *outf;
 {
@@ -1249,7 +1253,7 @@ FILE *outf;
  *
  * simnAP, simnAPHier --
  *
- * Output the area perimeter of the node with type type if it has not 
+ * Output the area perimeter of the node with type type if it has not
  * been visited.
  * The simnAPHier version outputs the area and perimeter only within the
  * local subcell with hierarchical name hierName.
@@ -1260,15 +1264,16 @@ FILE *outf;
  *
  * ----------------------------------------------------------------------------
  */
- 
+
 bool simnAP(node, resClass, scale, outf)
 EFNode *node;
-int  resClass, scale;
+int  resClass;
+float scale;
 FILE *outf;
 {
 	int a, p;
 
-	if ( node->efnode_client == (ClientData) NULL ) 
+	if ( node->efnode_client == (ClientData) NULL )
 		initNodeClient(node);
 	if ( resClass == NO_RESCLASS ||
 	     beenVisited((nodeClient *)node->efnode_client, resClass) ) {
@@ -1276,8 +1281,8 @@ FILE *outf;
 		return FALSE;
 	}
 	markVisited((nodeClient *)node->efnode_client, resClass);
-	a = node->efnode_pa[resClass].pa_area*scale*scale;
-	p = node->efnode_pa[resClass].pa_perim*scale;
+	a = (int)(node->efnode_pa[resClass].pa_area*scale*scale);
+	p = (int)(node->efnode_pa[resClass].pa_perim*scale);
 	if ( a < 0 ) a = 0;
 	if ( p < 0 ) p = 0;
 	fprintf(outf,"A_%d,P_%d", a, p);
@@ -1287,14 +1292,15 @@ FILE *outf;
 bool simnAPHier(dterm, hierName, resClass, scale, outf)
     DevTerm *dterm;
     HierName *hierName;
-    int  resClass, scale;
+    int  resClass;
+    float scale;
     FILE *outf;
 {
 	EFNode *node = dterm->dterm_node;
 	nodeClientHier   *nc ;
 	int a, p;
 
-	if ( node->efnode_client == (ClientData) NULL ) 
+	if ( node->efnode_client == (ClientData) NULL )
 		initNodeClientHier(node);
 	nc = (nodeClientHier *)node->efnode_client;
 	if ( nc->lastPrefix != hierName ) {
@@ -1307,15 +1313,15 @@ bool simnAPHier(dterm, hierName, resClass, scale, outf)
 		return FALSE;
 	}
 	markVisited((nodeClientHier *)node->efnode_client, resClass);
-	a = node->efnode_pa[resClass].pa_area*scale*scale;
-	p = node->efnode_pa[resClass].pa_perim*scale;
+	a = (int)(node->efnode_pa[resClass].pa_area*scale*scale);
+	p = (int)(node->efnode_pa[resClass].pa_perim*scale);
 	if ( a < 0 ) a = 0;
 	if ( p < 0 ) p = 0;
 	fprintf(outf,"A_%d,P_%d", a, p);
 	return TRUE;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1361,7 +1367,7 @@ simdevOutNode(prefix, suffix, name, outf)
 	initNodeClient(nn->efnn_node);
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1403,7 +1409,7 @@ int simcapVisit(hierName1, hierName2, cap)
     fprintf(esSimF, esCapFormat, cap);
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1443,7 +1449,7 @@ simresistVisit(hierName1, hierName2, res)
     fprintf(esSimF, " %g\n", res / 1000.);
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1622,7 +1628,7 @@ Dev    *dev;
 	} \
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1639,9 +1645,9 @@ Dev    *dev;
  * ----------------------------------------------------------------------------
  */
 int
-simmergeVisit(dev, hierName, scale, trans)
+simmergeVisit(dev, hc, scale, trans)
 Dev *dev;		/* Dev to examine */
-HierName *hierName;	/* Hierarchical path down to this dev */
+HierContext *hc;	/* Hierarchical context down to this dev */
 float scale;		/* Scale transform */
 Transform *trans;	/* Coordinate transform (not used) */
 {
@@ -1652,6 +1658,7 @@ Transform *trans;	/* Coordinate transform (not used) */
 	int      pmode, l, w;
 	float	 m;
 	devMerge *fp, *cfp;
+	HierName *hierName = hc->hc_hierName;
 
 	if (dev->dev_nterm < 2) {
 		TxError("outPremature\n");
@@ -1669,11 +1676,11 @@ Transform *trans;	/* Coordinate transform (not used) */
 	dnode = SimGetNode (hierName, drain->dterm_node->efnode_name->efnn_hier);
 
 	EFGetLengthAndWidth(dev, &l, &w);
-	fp = simmkDevMerge((int)(l*scale), (int)(w*scale), gnode, snode, 
+	fp = simmkDevMerge((int)(l*scale), (int)(w*scale), gnode, snode,
 			dnode, subnode, hierName, dev);
 
 	/*
-	 * run the list of devs. compare the current one with 
+	 * run the list of devs. compare the current one with
 	 * each one in the list. if they fullfill the matching requirements
 	 * merge them.
 	 */
@@ -1685,14 +1692,14 @@ Transform *trans;	/* Coordinate transform (not used) */
 		cg = &cfp->dev->dev_terms[0];
 		cs = cd = &cfp->dev->dev_terms[1];
 		if (cfp->dev->dev_nterm >= 3) {
-			if ( pmode == PARALLEL ) 
+			if ( pmode == PARALLEL )
 				cd = &cfp->dev->dev_terms[2];
-			else if ( pmode == PARALLEL_R ) 
+			else if ( pmode == PARALLEL_R )
 				cs = &cfp->dev->dev_terms[2];
 		}
-		
+
 		m = esFMult[cfp->esFMIndex] + ((float)fp->w/(float)cfp->w);
-		setDevMult(fp->esFMIndex, DEV_KILLED); 
+		setDevMult(fp->esFMIndex, DEV_KILLED);
 		setDevMult(cfp->esFMIndex, m);
 		esDevsMerged++;
 		freeMagic(fp);
