@@ -36,10 +36,16 @@ extern char *nodeSpiceHierName();
 extern devMerge *mkDevMerge();
 extern bool extHierSDAttr();
 
+extern bool devIsKilled();
+extern float getCurDevMult();
+extern void addDevMult();
+extern void setDevMult();
+
 /* Options specific to ext2spice */
 extern bool esDoExtResis;
 extern bool esDoPorts;
 extern bool esDoHierarchy;
+extern bool esDoBlackBox;
 extern bool esDoResistorTee;
 extern int  esDoSubckt;
 extern bool esDevNodesOnly;
@@ -73,9 +79,9 @@ extern int	 esSpiceDevsMerged;
 extern devMerge *devMergeList;
 
 /*
- * The following hash table and associated functions are used only if 
- * the format is hspice, to keep the translation between the hierarchical 
- * prefix of a node and the x num that we use to output valid hspice 
+ * The following hash table and associated functions are used only if
+ * the format is hspice, to keep the translation between the hierarchical
+ * prefix of a node and the x num that we use to output valid hspice
  * which also are meaningful.
  */
 extern HashTable subcktNameTable ; /* the hash table itself */
@@ -94,9 +100,9 @@ extern fetInfoList esFetInfo[MAXDEVTYPES];
                                 names even of the most complicated design */
 
 /* Node clients for figuring out areas and perimeters of sources and drains */
-typedef struct { 
-	long	_duml:MAXDEVTYPES; 
-} _dum; /* if you get an error here you should change 
+typedef struct {
+	long	_duml:MAXDEVTYPES;
+} _dum; /* if you get an error here you should change
            the data structures and visitMask */
 
 typedef union {
@@ -127,7 +133,7 @@ typedef struct {
 
 /*
  * Used to mark the nodes which are connected to devs. initMask  is set to
- * DEV_CONNECT_MASK only when we are in visitDevs 
+ * DEV_CONNECT_MASK only when we are in visitDevs
  */
 #define	DEV_CONNECT_MASK	((unsigned long)1<<(sizeof(long)*BITSPERCHAR-1))
 
@@ -146,7 +152,7 @@ typedef struct {
 }
 
 
-
+
 /* attributes controlling the Area/Perimeter extraction of dev terminals */
 #define ATTR_FLATAP	"*[Ee][Xx][Tt]:[Aa][Pp][Ff]*"
 #define ATTR_HIERAP	"*[Ee][Xx][Tt]:[Aa][Pp][Hh]*"
@@ -163,41 +169,18 @@ typedef struct {
 #define PARALLEL        1
 #define ANTIPARALLEL    2
 
-/* 
+/*
  *---------------------------------------------------------
- * Variables & macros used for merging parallel devs       
- * The merging of devs is based on the fact that spcdevVisit 
- * visits the devs in the same order all the time so the 
+ * Variables used for merging parallel devs
+ * The merging of devs is based on the fact that spcdevVisit
+ * visits the devs in the same order all the time so the
  * value of esFMult[i] keeps the multiplier for the ith dev
  *---------------------------------------------------------
  */
 #define	DEV_KILLED	((float) -1.0)
 #define	FMULT_SIZE	(1<<10)
-
-#define	devIsKilled(n) ( esFMult[(n)] <=(float)0.0 )
-
 #define	DEV_KILLED	((float) -1.0)
 
-
-/* macro to add a dev's multiplier to the table and grow it if necessary */
-#define addDevMult(f) \
-{  \
-	if ( esFMult == NULL ) { \
-	  esFMult = (float *) mallocMagic((unsigned) (esFMSize*sizeof(float)));  \
-	} else if ( esFMIndex >= esFMSize ) {  \
-	  int i;  \
-	  float *op = esFMult ;  \
-	  esFMult = (float *) mallocMagic((unsigned) ((esFMSize = esFMSize*2)*sizeof(float))); \
-	  for ( i = 0 ; i < esFMSize/2 ; i++ ) esFMult[i] = op[i]; \
-	  if (op) freeMagic(op); \
-	}  \
-	esFMult[esFMIndex++] = (float)(f); \
-}
-
-#define	setDevMult(i,f) { esFMult[(i)] = (float)(f); }
-
-#define	getCurDevMult() ((esFMult && (esFMIndex > 0)) ? esFMult[esFMIndex-1] : (float)1.0)
-
 #ifdef MAGIC_WRAPPER
 #define 	atoCap(s)	((EFCapValue)atof(s))
 #endif
