@@ -4,21 +4,21 @@
  * Procedures for building up the hierarchical representation
  * of a circuit.  These are all called from efReadDef() in EFread.c.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extflat/EFbuild.c,v 1.6 2010/06/24 12:37:17 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header$";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -65,7 +65,7 @@ bool efConnInitSubs();
 
 extern float locScale;
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -184,6 +184,7 @@ efBuildNode(def, isSubsnode, nodeName, nodeCap, x, y, layerName, av, ac)
     newnode->efnode_loc.r_xtop = newnode->efnode_loc.r_xbot + 1;
     newnode->efnode_loc.r_ytop = newnode->efnode_loc.r_ybot + 1;
     newnode->efnode_client = (ClientData) NULL;
+    newnode->efnode_num = 1;
     if (layerName) newnode->efnode_type =
 	    efBuildAddStr(EFLayerNames, &EFLayerNumNames, MAXTYPES, layerName);
     else newnode->efnode_type = 0;
@@ -287,7 +288,7 @@ efBuildAttr(def, nodeName, r, layerName, text)
     ap->efa_next = nn->efnn_node->efnode_attrs;
     nn->efnn_node->efnode_attrs = ap;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -370,7 +371,7 @@ efBuildDist(def, driver, receiver, min, max)
 	HashSetValue(he, (ClientData) he->h_key.h_ptr);
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -400,7 +401,7 @@ efBuildKill(def, name)
     kill->kill_next = def->def_kills;
     def->def_kills = kill;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -456,15 +457,21 @@ efBuildEquiv(def, nodeName1, nodeName2)
 			EFStrToHN((HierName *) NULL, nodeName2));
 	return;
     }
+    else if (nn2->efnn_node == (EFNode *)NULL)
+	return;		/* Repeated "equiv" statement */
 
     /* If both names exist and are for different nodes, merge them */
     if (nn1)
     {
+	if (nn1->efnn_node == (EFNode *)NULL)
+	    return;		/* Repeated "equiv" statement */
 	if (nn1->efnn_node != nn2->efnn_node)
 	{
 	    if (efWarn)
 		efReadError("Merged nodes %s and %s\n", nodeName1, nodeName2);
-	    efNodeMerge(nn1->efnn_node, nn2->efnn_node);
+	    efNodeMerge(&nn1->efnn_node, &nn2->efnn_node);
+	    if (nn1->efnn_port > 0) nn2->efnn_port = nn1->efnn_port;
+	    else if (nn2->efnn_port > 0) nn1->efnn_port = nn2->efnn_port;
 	}
 	return;
     }
@@ -473,7 +480,7 @@ efBuildEquiv(def, nodeName1, nodeName2)
     efNodeAddName(nn2->efnn_node, he1,
 			EFStrToHN((HierName *) NULL, nodeName1));
 }
-
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -529,7 +536,7 @@ efBuildDeviceParams(name, argc, argv)
 	pptr = strchr(argv[n], '=');
 	if (pptr == NULL)
 	{
-	    efReadError("Bad parameter assignment \"%s\" for device \"%s\"\n", 
+	    efReadError("Bad parameter assignment \"%s\" for device \"%s\"\n",
 			argv[n], name);
 	    continue;
 	}
@@ -802,7 +809,7 @@ efBuildDevice(def, class, type, r, argc, argv)
         newdev->dev_rect = *r;
         newdev->dev_type = efBuildAddStr(EFDevTypes, &EFDevNumTypes, MAXDEVTYPES, type);
         newdev->dev_class = class;
- 
+
         switch (class)
         {
 	    case DEV_FET:		/* old-style "fet" record */
@@ -964,7 +971,7 @@ efBuildPortNode(def, name, idx, x, y, layername, toplevel)
     if (nn != (EFNodeName *) NULL)
     {
 	nn->efnn_node->efnode_flags |= EF_PORT;
-	if (toplevel) 
+	if (toplevel)
 	    nn->efnn_node->efnode_flags |= EF_TOP_PORT;
 	nn->efnn_port = idx;
     }
@@ -1076,7 +1083,7 @@ efBuildDevNode(def, name, isSubsNode)
     }
     return nn->efnn_node;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1120,7 +1127,7 @@ efBuildAddStr(table, pMax, size, str)
 
     return max;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1209,7 +1216,7 @@ efBuildUse(def, subDefName, subUseId, ta, tb, tc, td, te, tf)
         TxError("Warning: use %s appears more than once in def!\n", newuse->use_id);
     HashSetValue(he, (ClientData)newuse);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1260,7 +1267,7 @@ efBuildConnect(def, nodeName1, nodeName2, deltaC, av, ac)
 	def->def_conns = conn;
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1297,7 +1304,7 @@ efBuildResistor(def, nodeName1, nodeName2, resistance)
 	def->def_resistors = conn;
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1334,7 +1341,7 @@ efBuildCap(def, nodeName1, nodeName2, cap)
 	def->def_caps = conn;
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1389,7 +1396,7 @@ bad:
     freeMagic((char *) conn);
     return FALSE;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1495,7 +1502,7 @@ again:
     cnp->cn_nsubs = nsubs;
     return TRUE;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1555,7 +1562,7 @@ efNodeAddName(node, he, hn)
 	oldnn->efnn_next = newnn;
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1580,70 +1587,86 @@ efNodeAddName(node, he, hn)
  */
 
 void
-efNodeMerge(node1, node2)
-    EFNode *node1, *node2;	/* Hierarchical nodes */
+efNodeMerge(node1ptr, node2ptr)
+    EFNode **node1ptr, **node2ptr;	/* Pointers to hierarchical nodes */
 {
     EFNodeName *nn, *nnlast;
     EFAttr *ap;
     int n;
+    EFNode *keeping, *removing;
 
     /* Sanity check: ignore if same node */
-    if (node1 == node2)
+    if (*node1ptr == *node2ptr)
 	return;
 
+    /* Keep the node with the greater number of entries, and merge  */
+    /* the node with fewer entries into it.			    */
+
+    if ((*node1ptr)->efnode_num >= (*node2ptr)->efnode_num)
+    {
+	keeping = *node1ptr;
+	removing = *node2ptr;
+    }
+    else
+    {
+	keeping = *node2ptr;
+	removing = *node1ptr;
+    }
+
     if (efWatchNodes)
     {
-	if (HashLookOnly(&efWatchTable, (char *) node1->efnode_name->efnn_hier)
-	    || (node2->efnode_name
+	if (HashLookOnly(&efWatchTable, (char *) keeping->efnode_name->efnn_hier)
+	    || (removing->efnode_name
 		&& HashLookOnly(&efWatchTable,
-				(char *) node2->efnode_name->efnn_hier)))
+				(char *) removing->efnode_name->efnn_hier)))
 	{
 	    printf("\ncombine: %s\n",
-		EFHNToStr(node1->efnode_name->efnn_hier));
-	    printf("  with   %s\n\n", 
-		node2->efnode_name
-		    ? EFHNToStr(node2->efnode_name->efnn_hier)
+		EFHNToStr(keeping->efnode_name->efnn_hier));
+	    printf("  with   %s\n\n",
+		removing->efnode_name
+		    ? EFHNToStr(removing->efnode_name->efnn_hier)
 		    : "(unnamed)");
 	}
     }
 
     /* Sum capacitances, perimeters, areas */
-    node1->efnode_cap += node2->efnode_cap;
+    keeping->efnode_cap += removing->efnode_cap;
     for (n = 0; n < efNumResistClasses; n++)
     {
-	node1->efnode_pa[n].pa_area += node2->efnode_pa[n].pa_area;
-	node1->efnode_pa[n].pa_perim += node2->efnode_pa[n].pa_perim;
+	keeping->efnode_pa[n].pa_area += removing->efnode_pa[n].pa_area;
+	keeping->efnode_pa[n].pa_perim += removing->efnode_pa[n].pa_perim;
     }
 
-    /* Make all EFNodeNames point to node1 */
-    if (node2->efnode_name)
+    /* Make all EFNodeNames point to "keeping" */
+    if (removing->efnode_name)
     {
-	bool topport1, topport2;
+	bool topportk, topportr;
 
-	for (nn = node2->efnode_name; nn; nn = nn->efnn_next)
+	for (nn = removing->efnode_name; nn; nn = nn->efnn_next)
 	{
 	    nnlast = nn;
-	    nn->efnn_node = node1;
+	    nn->efnn_node = keeping;
 	}
 
-	topport1 = (node1->efnode_flags & EF_TOP_PORT) ?  TRUE : FALSE;
-	topport2 = (node2->efnode_flags & EF_TOP_PORT) ?  TRUE : FALSE;
+	topportk = (keeping->efnode_flags & EF_TOP_PORT) ?  TRUE : FALSE;
+	topportr = (removing->efnode_flags & EF_TOP_PORT) ?  TRUE : FALSE;
 
 	/* Concatenate list of EFNodeNames, taking into account precedence */
-	if (!topport1 && (topport2 || EFHNBest(node2->efnode_name->efnn_hier,
-		     node1->efnode_name->efnn_hier)))
+	if ((!keeping->efnode_name) || (!topportk && (topportr
+		    || EFHNBest(removing->efnode_name->efnn_hier,
+		     keeping->efnode_name->efnn_hier))))
 	{
 	    /*
-	     * New official name is that of node2.
+	     * New official name is that of "removing".
 	     * The new list of names is:
-	     *	node2-names, node1-names
+	     *	removing-names, keeping-names
 	     */
-	    nnlast->efnn_next = node1->efnode_name;
-	    node1->efnode_name = node2->efnode_name;
+	    nnlast->efnn_next = keeping->efnode_name;
+	    keeping->efnode_name = removing->efnode_name;
 
 	    /*
-	     * Choose the new location only if node2's location is a valid one,
-	     * i.e, node2 wasn't created before it was mentioned.  This is mainly
+	     * Choose the new location only if "removing"'s location is a valid one,
+	     * i.e, "removing" wasn't created before it was mentioned.  This is mainly
 	     * to deal with new fets, resistors, and capacitors created by resistance
 	     * extraction, which appear with their full hierarchical names in the
 	     * .ext file for the root cell.
@@ -1658,18 +1681,10 @@ efNodeMerge(node1, node2)
 	     *
 	     *					Tim, 6/14/04
 	     */
-	    if (node2->efnode_type > 0)
+	    if (removing->efnode_type > 0)
 	    {
-		node1->efnode_loc = node2->efnode_loc;
-		node1->efnode_type = node2->efnode_type;
-
-		if (node2->efnode_loc.r_ybot < node1->efnode_loc.r_ybot
-		    || (node2->efnode_loc.r_ybot == node1->efnode_loc.r_ybot
-		    && node2->efnode_loc.r_xbot < node1->efnode_loc.r_xbot))
-		{
-//		    node1->efnode_loc = node2->efnode_loc;
-//		    node1->efnode_type = node2->efnode_type;
-		}
+		keeping->efnode_loc = removing->efnode_loc;
+		keeping->efnode_type = removing->efnode_type;
 	    }
 	}
 	else
@@ -1677,54 +1692,61 @@ efNodeMerge(node1, node2)
 	    /*
 	     * Keep old official name.
 	     * The new list of names is:
-	     *	node1-names[0], node2-names, node1-names[1-]
+	     *	keeping-names[0], removing-names, keeping-names[1-]
 	     */
-	    nnlast->efnn_next = node1->efnode_name->efnn_next;
-	    node1->efnode_name->efnn_next = node2->efnode_name;
+	    nnlast->efnn_next = keeping->efnode_name->efnn_next;
+	    keeping->efnode_name->efnn_next = removing->efnode_name;
 	}
     }
 
+    /* Merge list counts */
+    keeping->efnode_num += removing->efnode_num;
+
     /* Merge attribute lists */
-    if (ap = node2->efnode_attrs)
+    if (ap = removing->efnode_attrs)
     {
 	while (ap->efa_next)
 	    ap = ap->efa_next;
-	ap->efa_next = node1->efnode_attrs;
-	node1->efnode_attrs = ap;
-	node2->efnode_attrs = (EFAttr *) NULL;	/* Sanity */
+	ap->efa_next = keeping->efnode_attrs;
+	keeping->efnode_attrs = ap;
+	removing->efnode_attrs = (EFAttr *) NULL;	/* Sanity */
     }
 
-    /* Unlink node2 from list for def */
-    node2->efnode_prev->efnhdr_next = node2->efnode_next;
-    node2->efnode_next->efnhdr_prev = node2->efnode_prev;
+    /* Unlink "removing" from list for def */
+    removing->efnode_prev->efnhdr_next = removing->efnode_next;
+    removing->efnode_next->efnhdr_prev = removing->efnode_prev;
 
     /*
      * Only if both nodes were EF_DEVTERM do we keep EF_DEVTERM set
      * in the resultant node.
      */
-    if ((node2->efnode_flags & EF_DEVTERM) == 0)
-	node1->efnode_flags &= ~EF_DEVTERM;
+    if ((removing->efnode_flags & EF_DEVTERM) == 0)
+	keeping->efnode_flags &= ~EF_DEVTERM;
 
     /*
-     * If node2 has the EF_PORT flag set, then copy the port
+     * If "removing" has the EF_PORT flag set, then copy the port
      * record in the flags to node1.
      */
-    if (node2->efnode_flags & EF_PORT)
-	node1->efnode_flags |= EF_PORT;
-    if (node2->efnode_flags & EF_TOP_PORT)
-	node1->efnode_flags |= EF_TOP_PORT;
+    if (removing->efnode_flags & EF_PORT)
+	keeping->efnode_flags |= EF_PORT;
+    if (removing->efnode_flags & EF_TOP_PORT)
+	keeping->efnode_flags |= EF_TOP_PORT;
 
     /*
-     * If node2 has the EF_SUBS_NODE flag set, then copy the port
-     * record in the flags to node1.
+     * If "removing" has the EF_SUBS_NODE flag set, then copy the port
+     * record in the flags to "keeping".
      */
-    if (node2->efnode_flags & EF_SUBS_NODE)
-	node1->efnode_flags |= EF_SUBS_NODE;
+    if (removing->efnode_flags & EF_SUBS_NODE)
+	keeping->efnode_flags |= EF_SUBS_NODE;
 
-    /* Get rid of node2 */
-    freeMagic((char *) node2);
+    /* Get rid of "removing" */
+    freeMagic((char *) removing);
+
+    /* Make sure that the active node is always node1 */
+    *node1ptr = keeping;
+    *node2ptr = (EFNode *)NULL;	    /* Sanity check */
 }
-
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -1828,7 +1850,7 @@ efFreeNodeTable(table)
 	    freeMagic((char *) nn);
 	}
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1862,7 +1884,7 @@ efFreeNodeList(head)
 	freeMagic((char *) node);
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
