@@ -4,16 +4,16 @@
  * Circuit extraction.
  * Extract a single cell.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -61,7 +61,7 @@ FILE *extFileOpen();
 void extCellFile();
 void extHeader();
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -95,8 +95,11 @@ ExtCell(def, outName, doLength)
 {
     char *filename;
     FILE *f;
+    bool doLocal;
 
-    f = extFileOpen(def, outName, "w", &filename);
+    doLocal = (ExtOptions & EXT_DOLOCAL) ? TRUE : FALSE;
+
+    f = extFileOpen(def, outName, "w", doLocal, &filename);
 
     TxPrintf("Extracting %s into %s:\n", def->cd_name, filename);
 
@@ -127,7 +130,7 @@ ExtCell(def, outName, doLength)
 	TxPrintf("\n");
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -149,7 +152,7 @@ ExtCell(def, outName, doLength)
  */
 
 FILE *
-extFileOpen(def, file, mode, prealfile)
+extFileOpen(def, file, mode, doLocal, prealfile)
     CellDef *def;	/* Cell whose .ext file is to be written */
     char *file;		/* If non-NULL, open 'name'.ext; otherwise,
 			 * derive filename from 'def' as described
@@ -158,15 +161,18 @@ extFileOpen(def, file, mode, prealfile)
     char *mode;		/* Either "r" or "w", the mode in which the .ext
 			 * file is to be opened.
 			 */
+    bool  doLocal;	/* If true, always write to local directory */
     char **prealfile;	/* If this is non-NULL, it gets set to point to
 			 * a string holding the name of the .ext file.
 			 */
 {
     char namebuf[512], *name, *endp, *ends;
     int len;
-    FILE *rfile;
+    FILE *rfile, *testf;
 
     if (file) name = file;
+    else if (doLocal)
+	name = def->cd_name;	/* No path component, so save locally */
     else if (def->cd_file)
     {
 	name = def->cd_file;
@@ -191,31 +197,28 @@ extFileOpen(def, file, mode, prealfile)
 
     if (!strcmp(mode, "r")) return NULL;	/* Not even readable */
 
-    /* If mode is "w" and name has path components other than the cwd	*/
-    /* then try writing to the cwd IF there is no .mag file by the	*/
-    /* same name in the cwd that would conflict.			*/
+    /* Try writing to the cwd IF there is no .mag file by the	*/
+    /* same name in the cwd that would conflict.		*/
 
     name = strrchr(def->cd_name, '/');
     if (name != NULL)
-    {
-	FILE *testf;
-
 	name++;
-	ends = strrchr(def->cd_file, '/');
-	if (ends != NULL)
+    else
+	name = def->cd_name;
+
+    ends = strrchr(def->cd_file, '/');
+    if (ends != NULL)
+    {
+	testf = PaOpen(ends + 1, "r", ".mag", ".", ".", NULL);
+	if (testf)
 	{
-	    testf = PaOpen(ends + 1, "r", ".mag", ".", ".", NULL);
-	    if (testf)
-	    {
-		fclose(testf);
-		return NULL;
-	    }
+	    fclose(testf);
+	    return NULL;
 	}
-        return (PaOpen(name, mode, ".ext", ".", ".", prealfile));
     }
-    return NULL;
+    return (PaOpen(name, mode, ".ext", ".", ".", prealfile));
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -247,6 +250,7 @@ extCellFile(def, f, doLength)
 			 */
 {
     NodeRegion *reg;
+
     UndoDisable();
 
     /* Output the header: timestamp, technology, calls on cell uses */
@@ -258,7 +262,7 @@ extCellFile(def, f, doLength)
 
     /* Do hierarchical extraction */
     extParentUse->cu_def = def;
-    if (!SigInterruptPending) extSubtree(extParentUse, f);
+    if (!SigInterruptPending) extSubtree(extParentUse, reg, f);
     if (!SigInterruptPending) extArray(extParentUse, f);
 
     /* Clean up from basic extraction */
@@ -271,7 +275,7 @@ extCellFile(def, f, doLength)
 
     UndoEnable();
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -350,7 +354,7 @@ extHeader(def, f)
     /* Output all calls on subcells */
     (void) DBCellEnum(def, extOutputUsesFunc, (ClientData) f);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
