@@ -6,16 +6,16 @@
  * extraction, when no label for that node could be found in the
  * interaction area.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -221,8 +221,9 @@ extHardProc(scx, arg)
 	 * will have uninitialized region pointers, and so will not have labels
 	 * assigned to them.
 	 */
-	// ExtLabelRegions(def, ExtCurStyle->exts_nodeConn, &labRegList);
-	ExtLabelRegions(def, ExtCurStyle->exts_nodeConn, NULL);
+	// ExtLabelRegions(def, ExtCurStyle->exts_nodeConn, &labRegList,
+	//		&scx->scx_area);
+	ExtLabelRegions(def, ExtCurStyle->exts_nodeConn, NULL, NULL);
 
 	/* Now try to find a region with a node label */
 	for (reg = labRegList; reg; reg = reg->treg_next)
@@ -245,7 +246,7 @@ success:
 done:
     return (ret);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -334,7 +335,7 @@ extHardSetLabel(scx, reg, arg)
 	      	   /* forget it; we're never going to find the damn thing */
 		   r=oldlab->lab_rect;
 	      }
-	      
+
 	 }
     }
     GeoTransRect(&scx->scx_trans, &r, &newlab->lab_rect);
@@ -359,7 +360,7 @@ extHardSetLabel(scx, reg, arg)
 	TxPrintf("Hard way: found label = \"%s\"\n", newlab->lab_text);
     return (TRUE);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -412,9 +413,14 @@ extHardGenerateLabel(scx, reg, arg)
     // a random place.  It also allows the low-probability
     // but possible conflict between this node and another with
     // the same name in the parent cell.
+    //
+    // Reverted 10/30/2014:  Apparently this causes worse
+    // problems.
+    //
+    // GeoTransPoint(&scx->scx_trans, &reg->treg_ll, &r.r_ll);
+    // extMakeNodeNumPrint(gen, reg->treg_pnum, r.r_ll);
 
-    GeoTransPoint(&scx->scx_trans, &reg->treg_ll, &r.r_ll);
-    extMakeNodeNumPrint(gen, reg->treg_pnum, r.r_ll);
+    extMakeNodeNumPrint(gen, reg->treg_pnum, reg->treg_ll);
 
     prefixlen = tpath->tp_next - tpath->tp_first;
     len = strlen(gen) + prefixlen;
@@ -446,7 +452,7 @@ extHardGenerateLabel(scx, reg, arg)
 	TxPrintf("Hard way: generated label = \"%s\"\n", newlab->lab_text);
     return (TRUE);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
