@@ -5,16 +5,16 @@
  * This file contains the code to trace out connected Regions
  * in a layout, and to build up or tear down lists of Regions.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -35,7 +35,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include "extract/extract.h"
 #include "extract/extractInt.h"
 #include "utils/signals.h"
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -80,7 +80,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
  *
  * Non-interruptible.  It is the caller's responsibility to check
  * for interrupts.
- *	
+ *
  * ----------------------------------------------------------------------------
  */
 
@@ -126,7 +126,7 @@ ExtFindRegions(def, area, mask, connectsTo, uninit, first, each)
 
     return (arg.fra_region);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -167,7 +167,7 @@ extRegionAreaFunc(tile, arg)
     (void) ExtFindNeighbors(tile, arg->fra_pNum, arg);
     return (0);
 }
-
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -194,10 +194,11 @@ extRegionAreaFunc(tile, arg)
  */
 
 void
-ExtLabelRegions(def, connTo, nodeList)
+ExtLabelRegions(def, connTo, nodeList, clipArea)
     CellDef *def;		/* Cell definition being labelled */
     TileTypeBitMask *connTo;	/* Connectivity table (see above) */
-    NodeRegion **nodeList;
+    NodeRegion **nodeList;	/* Node list to add to (or NULL)  */
+    Rect *clipArea;		/* Area to search for sticky labels */
 {
     static Point offsets[] = { { 0, 0 }, { 0, -1 }, { -1, -1 }, { -1, 0 } };
     LabelList *ll;
@@ -248,8 +249,11 @@ ExtLabelRegions(def, connTo, nodeList)
 	    // Unconnected node label.  This may be a "sticky label".
 	    // If it is not connected to TT_SPACE, then create a new
 	    // node region for it.
+	    // (3/24/2015---changed from GEO_LABEL_IN_AREA to GEO_SURROUND)
 
-	    if (lab->lab_type != TT_SPACE)
+	    if ((GEO_SURROUND(&lab->lab_rect, clipArea) ||
+			GEO_TOUCH(&lab->lab_rect, clipArea))
+			 && (lab->lab_type != TT_SPACE))
 	    {
 		NodeRegion *newNode;
 		int n;
@@ -275,13 +279,13 @@ ExtLabelRegions(def, connTo, nodeList)
 		for (n = 0; n < nclasses; n++)
 		    newNode->nreg_pa[n].pa_perim = newNode->nreg_pa[n].pa_area = 0;
 		newNode->nreg_labels = ll;
-		
+
 		*nodeList = newNode;
 	    }
 	}
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -349,7 +353,7 @@ ExtLabelOneRegion(def, connTo, reg)
     }
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -382,7 +386,7 @@ ExtResetTiles(def, resetTo)
     for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
 	DBResetTilePlane(def->cd_planes[pNum], resetTo);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
