@@ -9,16 +9,16 @@
  *
  * The procedures in this file are not re-entrant.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -31,6 +31,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 
 #include "tcltk/tclmagic.h"
 #include "utils/magic.h"
+#include "tcltk/tclmagic.h"
 #include "utils/geometry.h"
 #include "utils/geofast.h"
 #include "tiles/tile.h"
@@ -93,7 +94,27 @@ void extSubtreeAdjustInit();
 void extSubtreeOutputCoupling();
 void extSubtreeHardSearch();
 
-
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * extClearUseFlags --
+ *
+ *  Callback function to clear the CU_SUB_EXTRACTED flag from each child
+ *  use of a CellDef.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+extClearUseFlags(use, clientData)
+    CellUse *use;
+    ClientData clientData;
+{
+    use->cu_flags &= ~CU_SUB_EXTRACTED;
+    return 0;
+}
+
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -145,6 +166,7 @@ extSubtree(parentUse, reg, f)
     bool result;
     int cuts, totcuts;
     float pdone, plast;
+    SearchContext scx;
 
     if ((ExtOptions & (EXT_DOCOUPLING|EXT_DOADJUST))
 		   != (EXT_DOCOUPLING|EXT_DOADJUST))
@@ -175,10 +197,10 @@ extSubtree(parentUse, reg, f)
      * halo has been set above to reflect the maximum distance for
      * sidewall coupling capacitance).
      */
-
     b = &def->cd_bbox;
 
-    /* Monitor progress, for large designs */
+    /* Monitor progress, for large designs, and allow display refresh at intervals */
+
     totcuts = (b->r_ytop - b->r_ybot + ExtCurStyle->exts_stepSize - 1)
 		    / ExtCurStyle->exts_stepSize;
     totcuts *= ((b->r_xtop - b->r_xbot + ExtCurStyle->exts_stepSize - 1)
@@ -214,7 +236,7 @@ extSubtree(parentUse, reg, f)
 			// If result == FALSE then ha.ha_interArea is invalid.
 			ha.ha_interArea = rlab;
 			result = TRUE;
-		    }	
+		    }
 		    else
 		        result |= GeoIncludeAll(&rlab, &ha.ha_interArea);
 		}
@@ -231,10 +253,10 @@ extSubtree(parentUse, reg, f)
 	    {
 		/* Make sure substrate connections have been handled	*/
 		/* even if there were no other interactions found.	*/
-		SearchContext scx;
 
+		ha.ha_clipArea = r;
 		scx.scx_trans = GeoIdentityTransform;
-		scx.scx_area = ha.ha_interArea;
+		scx.scx_area = r;
 		scx.scx_use = ha.ha_parentUse;
 		DBCellSrArea(&scx, extSubstrateFunc, (ClientData)&ha);
 	    }
@@ -242,8 +264,8 @@ extSubtree(parentUse, reg, f)
 	    cuts++;
 	    pdone = 100.0 * ((float)cuts / (float)totcuts);
 	    if ((((pdone - plast) > 5.0) || (cuts == totcuts)) && (cuts > 1)) {
-	        TxPrintf("Completed %d%%\n", (int)(pdone + 0.5));
-	        plast = pdone;
+		TxPrintf("Completed %d%%\n", (int)(pdone + 0.5));
+		plast = pdone;
 		TxFlushOut();
 
 #ifdef MAGIC_WRAPPER
@@ -291,6 +313,9 @@ done:
     /* Output connections and node adjustments */
     extOutputConns(&ha.ha_connHash, f);
     HashKill(&ha.ha_connHash);
+
+    /* Clear the CU_SUB_EXTRACTED flag from all children instances */
+    DBCellEnum(def, extClearUseFlags, (ClientData)NULL);
 }
 
 #ifdef	exactinteractions
@@ -336,7 +361,7 @@ extSubtreeInterFunc(tile, ha)
     return (0);
 }
 #endif	/* exactinteractions */
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -469,7 +494,7 @@ extSubtreeInteraction(ha)
 	 * hierarchy are part of ha_connHash and have adjustments
 	 * that are printed in the "merge" statement.  Nodes not in
 	 * the current cell are not considered.  Anything left over
-	 * has its adjusted value output. 
+	 * has its adjusted value output.
 	 */
 	for (reg = ha->ha_parentReg; reg; reg = reg->nreg_next)
 	{
@@ -530,7 +555,7 @@ extSubtreeInteraction(ha)
     extHierFreeLabels(cumDef);
     DBCellClearDef(cumDef);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -597,7 +622,7 @@ extSubtreeAdjustInit(ha)
 	}
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -649,7 +674,7 @@ extSubtreeOutputCoupling(ha)
 	fprintf(ha->ha_outf, "\"%s\" %lg\n", name, cap);
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -741,6 +766,8 @@ extSubtreeFunc(scx, ha)
 
 	    for (lab = cumDef->cd_labels; lab ; lab = lab->lab_next)
 	    {
+		if (!(lab->lab_flags & LABEL_STICKY)) continue;
+		
 		n = sizeof (Label) + strlen(lab->lab_text)
 			- sizeof lab->lab_text + 1;
 
@@ -780,11 +807,11 @@ extSubtreeFunc(scx, ha)
     /* connected together in the cell def, so in the case of an	*/
     /* array, just make sure that the first array entry is	*/
     /* connected.						*/
-    
+
     if (use->cu_xhi == use->cu_xlo && use->cu_yhi == use->cu_ylo)
 	extHierSubstrate(ha, use, -1, -1);
     else if (use->cu_xhi == use->cu_xlo && use->cu_yhi > use->cu_ylo)
-    {	
+    {
 	for (y = use->cu_ylo; y <= use->cu_yhi; y++)
 	    extHierSubstrate(ha, use, -1, y);
     }
@@ -799,6 +826,8 @@ extSubtreeFunc(scx, ha)
 	    for (y = use->cu_ylo; y <= use->cu_yhi; y++)
 		extHierSubstrate(ha, use, x, y);
     }
+    /* Mark substrate as having been extracted for this use. */
+    use->cu_flags |= CU_SUB_EXTRACTED;
 
     /* Free the cumulative node list we extracted above */
     if (ha->ha_cumFlat.et_nodes)
@@ -828,7 +857,7 @@ extSubtreeFunc(scx, ha)
 
     return (2);
 }
-
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -863,11 +892,11 @@ extSubstrateFunc(scx, ha)
     /* connected together in the cell def, so in the case of an	*/
     /* array, just make sure that the first array entry is	*/
     /* connected.						*/
-    
+
     if (use->cu_xhi == use->cu_xlo && use->cu_yhi == use->cu_ylo)
 	extHierSubstrate(ha, use, -1, -1);
     else if (use->cu_xhi == use->cu_xlo && use->cu_yhi > use->cu_ylo)
-    {	
+    {
 	for (y = use->cu_ylo; y <= use->cu_yhi; y++)
 	    extHierSubstrate(ha, use, -1, y);
     }
@@ -882,9 +911,10 @@ extSubstrateFunc(scx, ha)
 	    for (y = use->cu_ylo; y <= use->cu_yhi; y++)
 		extHierSubstrate(ha, use, x, y);
     }
+    use->cu_flags |= CU_SUB_EXTRACTED;
     return (2);
 }
-
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -1052,7 +1082,7 @@ extConnFindFunc(tp, preg)
 
     return (0);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
