@@ -4,16 +4,16 @@
  * Circuit extraction.
  * Generation of unique names.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -44,7 +44,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include "dbwind/dbwind.h"
 #include "utils/main.h"
 #include "utils/undo.h"
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -52,7 +52,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
  *
  * For the cell 'def', look for the same label appearing in two or more
  * distinct nodes.  For each such label found:
- * 
+ *
  *	If option is 0, then generate unique names for all
  *	    but one of the nodes by appending a unique numeric
  *	    suffix to the offending labels.
@@ -85,15 +85,22 @@ extUniqueCell(def, option)
     Label *lab;
     char *text;
     int nwarn;
+    bool isabstract;
 
     nwarn = 0;
+
+    /* Check for "LEFview", as we do not care about unique names in abstract views */
+    DBPropGet(def, "LEFview", &isabstract);
+    if (isabstract) return nwarn;
+
     HashInit(&labelHash, 32, HT_STRINGKEYS);
     TxPrintf("Processing %s\n", def->cd_name);
     TxFlush();
 
     /* Build up a list of nodes and assign them to tiles */
     lregList = (LabRegion *) ExtFindRegions(def, &TiPlaneRect,
-				&DBAllButSpaceBits, ExtCurStyle->exts_nodeConn,
+				&ExtCurStyle->exts_activeTypes,
+				ExtCurStyle->exts_nodeConn,
 				extUnInit, extHierLabFirst, (int (*)()) NULL);
 
     /* Assign the labels to their associated regions */
