@@ -1,5 +1,5 @@
 /*
- * defWrite.c --      
+ * defWrite.c --
  *
  * This module incorporates the LEF/DEF format for standard-cell place and
  * route.
@@ -11,7 +11,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/lef/defWrite.c,v 1.2 2008/02/10 19:30:21 tim Exp $";            
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/lef/defWrite.c,v 1.2 2008/02/10 19:30:21 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -66,7 +66,7 @@ char *defGetType();		/* Forward declaration */
 
 /*----------------------------------------------------------------------*/
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -84,19 +84,20 @@ char *defGetType();		/* Forward declaration */
  */
 
 void
-defWriteHeader(def, f, oscale)
+defWriteHeader(def, f, oscale, units)
     CellDef *def;	/* Def for which to generate DEF output */
     FILE *f;		/* Output to this file */
     float oscale;
+    int units;		/* Units for UNITS; could be derived from oscale */
 {
     TileType type;
 
     TxPrintf("Diagnostic:  Write DEF header for cell %s\n", def->cd_name);
 
     /* NOTE:  This routine corresponds to Envisia LEF/DEF Language	*/
-    /* Reference version 5.3 (May 31, 2000)				*/
+    /* Reference version 5.7 (November, 2009)				*/
 
-    fprintf(f, "VERSION 5.3 ;\n");
+    fprintf(f, "VERSION 5.7 ;\n");
     fprintf(f, "   NAMESCASESENSITIVE ON ;\n");
     fprintf(f, "   DIVIDERCHAR \"/\" ;\n");
 
@@ -113,13 +114,13 @@ defWriteHeader(def, f, oscale)
     /* technology).							*/
     fprintf(f, "   TECHNOLOGY %s ;\n", DBTechName);
 
-    /* As I understand it, this refers to the scalefactor of the GDS	*/
-    /* file output.  Magic does all GDS in nanometers, so the LEF	*/
-    /* scalefactor (conversion to microns) is always 1000.		*/
-    fprintf(f, "   UNITS DISTANCE MICRONS 1000 ;\n");
+    /* The DEF scalefactor (conversion to microns) is always 1000	*/
+    /* (nanometers) unless overridden on the command line.		*/
+
+    fprintf(f, "   UNITS DISTANCE MICRONS %d ;\n", units);
 
     /* Die area, taken from the cell def bounding box.			*/
-    fprintf(f, "   DIEAREA ( %.10g %.10g ) ( %.10g %.10g ) ;\n", 
+    fprintf(f, "   DIEAREA ( %.10g %.10g ) ( %.10g %.10g ) ;\n",
 	(float)def->cd_bbox.r_xbot * oscale,
 	(float)def->cd_bbox.r_ybot * oscale,
 	(float)def->cd_bbox.r_xtop * oscale,
@@ -151,7 +152,7 @@ defTransPos(Transform *t)
 {
     static char *def_orient[] = {
 	"N", "S", "E", "W", "FN", "FS", "FE", "FW"
-    }; 
+    };
 
     bool ew;  /* east-or-west identifier */
     bool sw;  /* south-or-west identifier */
@@ -173,8 +174,8 @@ defTransPos(Transform *t)
     if (flip) pos += 4;
     if (ew) pos += 2;
     if (sw) pos += 1;
-    
-    return def_orient[pos]; 
+
+    return def_orient[pos];
 }
 
 /*
@@ -232,7 +233,7 @@ defCountNets(rootDef, allSpecial)
 	EFDone();
 	total.has_nets = FALSE;
     }
-    
+
     if (allSpecial) total.regular = 0;
     return total;
 }
@@ -260,7 +261,7 @@ defnodeCount(node, res, cap, total)
 	char *pwr;
 	cp = hierName->hn_name;
 	clast = *(cp + strlen(cp) - 1);
-	
+
 	/* Global nodes are marked as "special nets" */
 	if (clast == '!')
 	    node->efnode_flags |= EF_SPECIAL;
@@ -317,7 +318,7 @@ defnodeCount(node, res, cap, total)
 	else if (node->efnode_flags & EF_PORT)
 	    total->regular++;
     }
-    
+
     return 0;	/* Keep going. . . */
 }
 
@@ -337,7 +338,7 @@ defnodeCount(node, res, cap, total)
  * ----------------------------------------------------------------------------
  */
 
-void 
+void
 defHNsprintf(str, hierName, divchar)
     char *str;
     HierName *hierName;
@@ -357,7 +358,7 @@ defHNsprintf(str, hierName, divchar)
     /* magic name anyway), or dashes, asterisks, or percent signs	*/
     /* (which are interpreted as wildcard characters by LEF/DEF).	*/
 
-    cp = hierName->hn_name; 
+    cp = hierName->hn_name;
     while (c = *cp++)
     {
 	switch (c)
@@ -415,7 +416,7 @@ nodeDefName(hname)
     EFNodeName *nn;
     HashEntry *he;
     EFNode *node;
-    static char nodeName[256]; 
+    static char nodeName[256];
 
     he = EFHNLook(hname, (char *) NULL, "nodeName");
     if (he == NULL)
@@ -444,7 +445,7 @@ nodeDefName(hname)
  *	Output to DEF file; resets defdata->outcolumn
  *
  *------------------------------------------------------------
- */ 
+ */
 
 #define MAX_DEF_COLUMNS 70
 
@@ -766,7 +767,7 @@ defPortTileFunc(tile, cx)
 		rport->r_xbot, rport->r_ybot,
 		rport->r_xtop, rport->r_ytop);
     */
-    
+
     return 1;	/* No need to check further */
 }
 
@@ -901,7 +902,7 @@ defNetGeometryFunc(tile, plane, defdata)
     /* Layer names are taken from the LEF database. */
 
     lefName = MagicToLefTable[ttype].lefName;
-    ASSERT(lefName, "Valid ttype");
+    if (lefName == NULL) return 0;	/* Do not write types not in LEF definition */
     lefType = MagicToLefTable[ttype].lefInfo;
 
     orient = GEO_EAST;
@@ -1025,7 +1026,7 @@ defNetGeometryFunc(tile, plane, defdata)
 	    r.r_ytop = r.r_ybot + 1;
 
 	/* "sameroute" is true only if rectangles touch in the		*/
-	/* direction of the route.					*/ 
+	/* direction of the route.					*/
 	/* NOTE:  We should compute this FIRST and use it to determine	*/
 	/* the current route direction!					*/
 	/* Another hack---for special nets, don't continue routes that	*/
@@ -1066,7 +1067,7 @@ defNetGeometryFunc(tile, plane, defdata)
 		defdata->orient == GEO_CENTER)
 	    sameroute = FALSE;
     }
-		
+
 
     /* Determine if we need to write a NEW (type) record.  We do this	*/
     /* if 1) this is the first tile visited (except that we don't	*/
@@ -1392,6 +1393,7 @@ defCountViaFunc(tile, cviadata)
     /* Generate a via name from the layer name and tile size */
 
     lname = MagicToLefTable[ctype].lefName;
+    if (lname == NULL) return 0;    /* Do not output undefined LEF layers */
     TiToRect(tile, &r);
 
     /* Boundary search.  WARNING:  This code is quite naive.  The	*/
@@ -1501,7 +1503,7 @@ defCountViaFunc(tile, cviadata)
 	cviadata->total++;	/* Increment the count of uses */
 	lefl = (lefLayer *)mallocMagic(sizeof(lefLayer));
 	lefl->type = ttype;
-	lefl->obsType = -1; 
+	lefl->obsType = -1;
 	lefl->lefClass = CLASS_VIA;
 	lefl->info.via.area = r;
 	lefl->info.via.cell = (CellDef *)NULL;
@@ -1550,7 +1552,8 @@ defGetType(ttype, lefptr)
 	while (he = HashNext(&LefInfo, &hs))
 	{
 	    lefl = (lefLayer *)HashGetValue(he);
-	    if (lefl && (contact == lefl->lefClass))
+	    if (lefl && ((contact == lefl->lefClass) ||
+			((contact == CLASS_ROUTE) && (lefl->lefClass == CLASS_MASTER))))
 		if ((lefl->type == ttype) || (lefl->obsType == ttype))
 		{
 		    if (lefptr) *lefptr = lefl;
@@ -1559,9 +1562,9 @@ defGetType(ttype, lefptr)
 	}
     }
 
-    /* If we got here, there is no entry;  use the database name */
+    /* If we got here, there is no entry;  return NULL. */
     if (lefptr) *lefptr = (lefLayer *)NULL;
-    return DBTypeLongNameTbl[ttype];
+    return NULL;
 }
 
 /*
@@ -1616,12 +1619,12 @@ defWriteVias(f, rootDef, oscale, lefMagicToLefLayer)
 		continue;
 
 	    if (lefl->lefClass == CLASS_VIA)
-	    { 
+	    {
 		fprintf(f, "   - %s", (char *)lefl->canonName);
 
 		/* Generate squares for the area as determined	*/
 		/* by the cifoutput section of the tech file	*/
-		
+
 		rMask = DBResidueMask(lefl->type);
 		for (ttype = TT_TECHDEPBASE; ttype < DBNumUserLayers; ttype++)
 		    if (TTMaskHasType(rMask, ttype))
@@ -1665,7 +1668,7 @@ defWriteVias(f, rootDef, oscale, lefMagicToLefLayer)
 			for (j = 0; j < nAc; j++)
 			{
 			     square.r_xtop = square.r_xbot + size;
-	
+
 			     fprintf(f, "\n      + RECT %s ( %.10g %.10g )"
 					" ( %.10g %.10g )",
 					lefMagicToLefLayer[lefl->type].lefName,
@@ -1689,7 +1692,7 @@ defWriteVias(f, rootDef, oscale, lefMagicToLefLayer)
 			(float)(lefl->info.via.area.r_ybot) * oscale / 2,
 			(float)(lefl->info.via.area.r_xtop) * oscale / 2,
 			(float)(lefl->info.via.area.r_ytop) * oscale / 2);
-	 	}	
+	 	}
 		fprintf(f, " ;\n");
 	    }
 	}
@@ -1713,7 +1716,7 @@ defWriteVias(f, rootDef, oscale, lefMagicToLefLayer)
  *
  *------------------------------------------------------------
  */
- 
+
 int
 defCountComponents(rootDef)
     CellDef *rootDef;
@@ -1820,6 +1823,7 @@ defComponentFunc(cellUse, defdata)
 {
     FILE *f = defdata->f;
     float oscale = defdata->scale;
+    char *nameroot;
 
     /* Ignore any cellUse that does not have an identifier string. */
     if (cellUse->cu_id == NULL) return 0;
@@ -1830,8 +1834,17 @@ defComponentFunc(cellUse, defdata)
 	return 0;
     }
 
+    /* In case the cd_name contains a path component (it's not supposed to),	*/
+    /* remove it.								*/
+
+    nameroot = strrchr(cellUse->cu_def->cd_name, '/');
+    if (nameroot != NULL)
+	nameroot++;
+    else
+	nameroot = cellUse->cu_def->cd_name;
+
     fprintf(f, "   - %s %s\n      + PLACED ( %.10g %.10g ) %s ;\n",
-	cellUse->cu_id, cellUse->cu_def->cd_name,
+	cellUse->cu_id, nameroot,
 	(float)cellUse->cu_bbox.r_xbot * oscale,
 	(float)cellUse->cu_bbox.r_ybot * oscale,
 	defTransPos(&cellUse->cu_transform));
@@ -1864,10 +1877,10 @@ defMakeInverseLayerMap()
     TileType i;
     char *lefname;
 
-    lefMagicToLefLayer = (LefMapping *)mallocMagic(DBNumUserLayers
+    lefMagicToLefLayer = (LefMapping *)mallocMagic(DBNumTypes
 		* sizeof(LefMapping));
     memset(lefMagicToLefLayer, 0, sizeof(LefMapping) * TT_TECHDEPBASE);
-    for (i = TT_TECHDEPBASE; i < DBNumUserLayers; i++)
+    for (i = TT_TECHDEPBASE; i < DBNumTypes; i++)
     {
 	lefname = defGetType(i, &lefl);
 	lefMagicToLefLayer[i].lefName = lefname;
@@ -1916,23 +1929,28 @@ defMakeInverseLayerMap()
  */
 
 void
-DefWriteCell(def, outName, allSpecial)
+DefWriteCell(def, outName, allSpecial, units)
     CellDef *def;		/* Cell being written */
     char *outName;		/* Name of output file, or NULL. */
     bool allSpecial;		/* Treat all nets as SPECIALNETS? */
+    int units;			/* Force units to this value (default 1000) */
 {
     char *filename;
     FILE *f;
     NetCount nets;
     int total;
-    float scale = CIFGetOutputScale(1);	/* Note that "1" here corresponds
-					 * to "1000" in the header UNITS line
-					 */
+    float scale;
+
     LefMapping *lefMagicToLefLayer;
     int i;
     lefLayer *lefl;
     HashEntry *he;
 
+    /* Note that "1" corresponds to "1000" in the header UNITS line,	*/
+    /* or units of nanometers.  10 = centimicrons, 1000 = microns.	*/
+
+    scale = CIFGetOutputScale(1000 / units);
+
     f = lefFileOpen(def, outName, ".def", "w", &filename);
 
     TxPrintf("Generating DEF output %s for cell %s:\n", filename, def->cd_name);
@@ -1949,20 +1967,20 @@ DefWriteCell(def, outName, allSpecial)
 	return;
     }
 
-    defWriteHeader(def, f, scale);
+    defWriteHeader(def, f, scale, units);
 
     lefMagicToLefLayer = defMakeInverseLayerMap();
 
     /* Vias---magic contact areas are reported as vias. */
     total = defCountVias(def, lefMagicToLefLayer, scale);
-    fprintf(f, "VIAS %d ;\n", total); 
+    fprintf(f, "VIAS %d ;\n", total);
     if (total > 0)
 	defWriteVias(f, def, scale, lefMagicToLefLayer);
     fprintf(f, "END VIAS\n\n");
 
     /* Components (i.e., cell uses) */
     total = defCountComponents(def);
-    fprintf(f, "COMPONENTS %d ;\n", total); 
+    fprintf(f, "COMPONENTS %d ;\n", total);
     if (total > 0)
 	defWriteComponents(f, def, scale);
     fprintf(f, "END COMPONENTS\n\n");
@@ -1975,14 +1993,14 @@ DefWriteCell(def, outName, allSpecial)
 
     /* "Special" nets---nets matching $GND, $VDD, or $globals(*) 	*/
 
-    fprintf(f, "SPECIALNETS %d ;\n", nets.special); 
+    fprintf(f, "SPECIALNETS %d ;\n", nets.special);
     if (nets.special > 0)
-	defWriteNets(f, def, scale, lefMagicToLefLayer, (allSpecial) ? 
+	defWriteNets(f, def, scale, lefMagicToLefLayer, (allSpecial) ?
 		ALL_SPECIAL : DO_SPECIAL);
     fprintf(f, "END SPECIALNETS\n\n");
 
     /* "Regular" nets */
-    fprintf(f, "NETS %d ;\n", nets.regular); 
+    fprintf(f, "NETS %d ;\n", nets.regular);
     if (nets.regular > 0)
 	defWriteNets(f, def, scale, lefMagicToLefLayer, DO_REGULAR);
     fprintf(f, "END NETS\n\n");
