@@ -3,24 +3,25 @@
  *
  * Commands for the plot module only.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/plot/plotCmd.c,v 1.2 2006/10/10 07:04:48 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/plot/plotCmd.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
 
@@ -38,8 +39,9 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 #include "plow/plow.h"
 #include "select/select.h"
 #include "commands/commands.h"
+#include "plot/plotInt.h"
+
 
-
 /*
  * ----------------------------------------------------------------------------
  *
@@ -65,6 +67,9 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 typedef enum {
 	POSTSCRIPT=0,
 	PLOTPNM,
+#ifdef HAVE_LIBCAIRO
+	PLOTSVG,
+#endif
 #ifdef GREMLIN
 	STYLE_GREMLIN,
 #endif
@@ -90,21 +95,30 @@ CmdPlot(w, cmd)
     CellDef *boxRootDef;
     SearchContext scx;
     float width;
-    int iwidth;
+    int iwidth, scale;
+
+#ifdef HAVE_LIBCAIRO
+    int flags;
+    extern void GrTCairoPlotSVG();
+#endif
 
     static char *cmdPlotOption[] =
-    {	
+    {
 	"postscript file [layers]    generate PostScript file for what's\n\
-                                     underneath the box",				
+                                     underneath the box",
 	"pnm file [width [layers]]   generate PNM file for what's\n\
 		                     underneath the box",
+#ifdef HAVE_LIBCAIRO
+	"svg file		     generate SVG file for the whole window",
+#endif
 #ifdef GREMLIN
 	"gremlin file [layers]	     generate gremlin file for what's\n\
 		                     underneath the box",
 #endif
 #ifdef VERSATEC
-	"versatec [size [layers]]    generate plot on Versatec printer, size\n\
-		                     inches wide for layers underneath box",
+	"versatec [scale [layers]]    generate plot for selected printer;\n\
+		                     scale is multiplying factor for parameter\n\
+		                     lambda, for layers underneath the box",
 #endif
 #ifdef	LLNL
 	"pixels [width [layers]]     generate plot in pix format, width pixels\n\
@@ -135,7 +149,10 @@ CmdPlot(w, cmd)
 #ifdef GREMLIN
 		|| (option == STYLE_GREMLIN)
 #endif
-#ifdef VERSATEX
+#ifdef HAVE_LIBCAIRO
+		|| (option == PLOTSVG)
+#endif
+#ifdef VERSATEC
 	        || (option == STYLE_VERSATEC)
 #endif
 #ifdef LLNL
@@ -151,7 +168,7 @@ CmdPlot(w, cmd)
 	    if ((window == (MagWindow *) NULL) || (window->w_client != DBWclientID))
 	    {
 		TxError("The cursor must be over a layout window to plot.\n");
-		return; 
+		return;
 	    }
 	}
 	crec = (DBWclientRec *) window->w_clientData;
@@ -189,7 +206,7 @@ CmdPlot(w, cmd)
 	    }
 	    PlotPS(cmd->tx_argv[2], &scx, &mask, crec->dbw_bitmask);
 	    return;
-	    
+
 #ifdef GREMLIN
 	case STYLE_GREMLIN:
 	    if ((cmd->tx_argc != 3) && (cmd->tx_argc != 4))
@@ -206,7 +223,7 @@ CmdPlot(w, cmd)
 	    PlotGremlin(cmd->tx_argv[2], &scx, &mask, crec->dbw_bitmask);
 	    return;
 #endif /* GREMLIN */
-	    
+
 	case HELP:
 	    TxPrintf("The \"plot\" commands are:\n");
 	    for (msg = &(cmdPlotOption[0]); *msg != NULL; msg++)
@@ -214,7 +231,7 @@ CmdPlot(w, cmd)
 		TxPrintf("    plot %s\n", *msg);
 	    }
 	    return;
-	
+
 	case PARAMETERS:
 	    if (cmd->tx_argc == 2)
 		PlotPrintParams();
@@ -226,7 +243,7 @@ CmdPlot(w, cmd)
 			cmdPlotOption[PARAMETERS]);
 	    }
 	    return;
-	
+
 #ifdef VERSATEC
 	case STYLE_VERSATEC:
 	    if (cmd->tx_argc > 4)
@@ -236,14 +253,32 @@ CmdPlot(w, cmd)
 		return;
 	    }
 	    if (cmd->tx_argc >= 3)
-		width = MagAtof(cmd->tx_argv[2]);
-	    else width = 0.0;
+		scale = atoi(cmd->tx_argv[2]);
+	    else scale = 0.0;
 	    if (cmd->tx_argc == 4)
 	    {
 		if (!CmdParseLayers(cmd->tx_argv[3], &mask))
 			return;
 	    }
-	    PlotVersatec( &scx, &mask, crec->dbw_bitmask, width);
+	    PlotVersatec( &scx, &mask, crec->dbw_bitmask, scale);
+	    return;
+#endif
+
+#ifdef HAVE_LIBCAIRO
+	case PLOTSVG:
+	    if (cmd->tx_argc > 3)
+	    {
+		TxError("Too many arguments:\n    plot %s\n",
+			cmdPlotOption[PLOTSVG]);
+		return;
+	    }
+	    flags = window->w_flags;
+	    /* In case this is called from a non-GUI wrapper window, remove */
+	    /* the window border widgets from the rendered display.	    */
+	    window->w_flags &= ~(WIND_SCROLLABLE | WIND_SCROLLBARS | WIND_CAPTION
+			| WIND_BORDER);
+	    GrTCairoPlotSVG(cmd->tx_argv[2], window);
+	    window->w_flags = flags;
 	    return;
 #endif
 
@@ -261,25 +296,42 @@ CmdPlot(w, cmd)
 	    }
 	    if (cmd->tx_argc >= 4)
 	    {
+#ifdef VERSATEC
+		if (PlotPNMRTL && StrIsNumeric(cmd->tx_argv[3]))
+		    iwidth = (int)(atof(cmd->tx_argv[3]) * (double)PlotVersDotsPerInch);
+		else
+#endif
 	        if (!StrIsInt(cmd->tx_argv[3]))
 		{
-		    TxError("Width should be an integer number of pixels\n");
-		    return;
+		    {
+			TxError("Width should be an integer number of pixels\n");
+			return;
+		    }
 		}
 		else
 		    iwidth = atoi(cmd->tx_argv[3]);
 	    }
-	    else 
+#ifdef VERSATEC
+	    /* RTL mode allows no filename.  A temporary filename is created	*/
+	    /* for the file being spooled to the printer.			*/
+
+	    else if (PlotPNMRTL && StrIsNumeric(cmd->tx_argv[2]))
+	    {
+		iwidth = (int)(atof(cmd->tx_argv[2]) * (double)PlotVersDotsPerInch);
+		PlotPNM(NULL, &scx, &mask, crec->dbw_bitmask, iwidth);
+		return;
+	    }
+#endif
+	    else
 		iwidth = 1000;		/* Default value */
 
 	    if (cmd->tx_argc == 5)
-	    {
 	      if (!CmdParseLayers(cmd->tx_argv[4], &mask))
 		return;
-	    }
-	    PlotPNM ( cmd->tx_argv[2], &scx, &mask, crec->dbw_bitmask, iwidth);
+
+	    PlotPNM(cmd->tx_argv[2], &scx, &mask, crec->dbw_bitmask, iwidth);
 	    return;
-    
+
 #ifdef	LLNL
 	case PIXELS:
 	    if (cmd->tx_argc > 4)
