@@ -150,7 +150,7 @@ proc magic::gencell {gencell_name {instname {}} args} {
     }
     set argpar [dict create {*}$args]
 
-    if {$gencell_name == {}} { 
+    if {$gencell_name == {}} {
 	# Find selected item  (to-do:  handle multiple selections)
 
 	set wlist [what -list]
@@ -171,19 +171,7 @@ proc magic::gencell {gencell_name {instname {}} args} {
 	}
         # need to incorporate argpar?
         set parameters [cellname list property $gname parameters]
-	
-	# if "nocell" is set in parameters, then overwrite additional
-        # parameters from the instance.  Reserved parameters names are
-	# nx, ny, pitchx, and pitchy
-	if {[dict exists $parameters nocell]} {
-	    set arcount [array -list count]
-	    set arpitch [array -list pitch]
-
-	    dict set parameters nx [expr [lindex $arcount 1] - [lindex $arcount 0] + 1]
-	    dict set parameters ny [expr [lindex $arcount 3] - [lindex $arcount 2] + 1]
-	    dict set parameters pitchx [lindex $arpitch 0]
-	    dict set parameters pitchy [lindex $arpitch 1]
-	}
+	set parameters [magic::gencell_defaults $gencell_type $library $parameters]
 	magic::gencell_dialog $ginst $gencell_type $library $parameters
     } else {
 	# Parse out library name from gencell_name, otherwise default
@@ -207,6 +195,7 @@ proc magic::gencell {gencell_name {instname {}} args} {
 	} else {
 	    # Check if instance exists or not in the cell
 	    set cellname [instance list celldef $instname]
+
 	    if {$cellname != ""} {
 		# Case:  Change existing instance, parameters in args (if any)
 		select cell $instname
@@ -215,7 +204,7 @@ proc magic::gencell {gencell_name {instname {}} args} {
 		if {[dict exists $parameters nocell]} {
 		    set arcount [array -list count]
 		    set arpitch [array -list pitch]
-	
+
 		    dict set parameters nx [lindex $arcount 1]
 		    dict set parameters ny [lindex $arcount 3]
 		    dict set parameters pitchx $delx
@@ -291,6 +280,12 @@ proc magic::gencell_setparams {parameters} {
       } elseif {[regexp {^.params.edits.(.*)_sel$} $s valid pname] != 0} {
 	 set value [dict get $parameters $pname]
          set magic::${pname}_val $value
+	 .params.edits.${pname}_sel configure -text $value
+      } elseif {[regexp {^.params.edits.(.*)_txt$} $s valid pname] != 0} {
+	 if {[dict exists $parameters $pname]} {
+	    set value [dict get $parameters $pname]
+	    .params.edits.${pname}_txt configure -text $value
+	 }
       }
    }
 }
@@ -305,17 +300,29 @@ proc magic::gencell_change {instname gencell_type library parameters} {
     global Opts
     suspendall
 
+    set newinstname $instname
     if {$parameters == {}} {
         # Get device defaults
 	set pdefaults [${library}::${gencell_type}_defaults]
         # Pull user-entered values from dialog
         set parameters [dict merge $pdefaults [magic::gencell_getparams]]
+	set newinstname [.params.title.ient get]
+	if {$newinstname == "(default)"} {set newinstname $instname}
+	if {$newinstname == $instname} {set newinstname $instname}
+	if {[instance list exists $newinstname] != ""} {set newinstname $instname}
+    }
+    if {[dict exists $parameters gencell]} {
+        # Setting special parameter "gencell" forces the gencell to change type
+	set gencell_type [dict get $parameters gencell]
     }
     if {[catch {set parameters [${library}::${gencell_type}_check $parameters]} \
 		checkerr]} {
 	puts stderr $checkerr
     }
     magic::gencell_setparams $parameters
+    if {[dict exists $parameters gencell]} {
+	set parameters [dict remove $parameters gencell]
+    }
 
     set gname [instance list celldef $instname]
 
@@ -337,11 +344,7 @@ proc magic::gencell_change {instname gencell_type library parameters} {
 		drawerr]} {
 	    puts stderr $drawerr
 	}
-	pushstack $gname
-	property parameters $parameters
-	popstack
         select cell $newinst
-	identify $instname
     } else {
 	pushstack $gname
 	select cell
@@ -351,15 +354,43 @@ proc magic::gencell_change {instname gencell_type library parameters} {
 	    puts stderr $drawerr
 	}
 	property parameters $parameters
+	property gencell ${gencell_type}
 	tech revert
 	popstack
+        select cell $instname
     }
+    identify $newinstname
     eval "box values $savebox"
     snap $snaptype
     resumeall
     redraw
 }
 
+#-------------------------------------------------------------
+# Assign a unique name for a gencell
+#
+# Note:  This depends on the unlikelihood of the name
+# existing in a cell on disk.  Only cells in memory are
+# checked for name collisions.  Since the names will go
+# into SPICE netlists, names must be unique when compared
+# in a case-insensitive manner.  Using base-36 (alphabet and
+# numbers), each gencell name with 6 randomized characters
+# has a 1 in 4.6E-10 chance of reappearing.
+#-------------------------------------------------------------
+
+proc magic::get_gencell_name {gencell_type} {
+    while {true} {
+        set postfix ""
+        for {set i 0} {$i < 6} {incr i} {
+	    set pint [expr 48 + int(rand() * 36)]
+	    if {$pint > 57} {set pint [expr $pint + 39]}
+	    append postfix [format %c $pint]
+	}
+	if {[cellname list exists ${gencell_type}_$postfix] == 0} {break}
+    }
+    return ${gencell_type}_$postfix
+}
+
 #-------------------------------------------------------------
 # gencell_create
 #
@@ -375,12 +406,27 @@ proc magic::gencell_create {gencell_type library parameters} {
     global Opts
     suspendall
 
+    set newinstname ""
+
     # Get device defaults
-    set pdefaults [${library}::${gencell_type}_defaults]
     if {$parameters == {}} {
         # Pull user-entered values from dialog
-        set parameters [dict merge $pdefaults [magic::gencell_getparams]]
+        set dialogparams [magic::gencell_getparams]
+	if {[dict exists $dialogparams gencell]} {
+	    # Setting special parameter "gencell" forces the gencell to change type
+	    set gencell_type [dict get $dialogparams gencell]
+	}
+	set pdefaults [${library}::${gencell_type}_defaults]
+        set parameters [dict merge $pdefaults $dialogparams]
+	set newinstname [.params.title.ient get]
+	if {$newinstname == "(default)"} {set newinstname ""}
+	if {[instance list exists $newinstname] != ""} {set newinstname ""}
     } else {
+	if {[dict exists $parameters gencell]} {
+	    # Setting special parameter "gencell" forces the gencell to change type
+	    set gencell_type [dict get $parameters gencell]
+	}
+	set pdefaults [${library}::${gencell_type}_defaults]
         set parameters [dict merge $pdefaults $parameters]
     }
 
@@ -389,6 +435,9 @@ proc magic::gencell_create {gencell_type library parameters} {
 	puts stderr $checkerr
     }
     magic::gencell_setparams $parameters
+    if {[dict exists $parameters gencell]} {
+	set parameters [dict remove $parameters gencell]
+    }
 
     set snaptype [snap list]
     snap internal
@@ -400,25 +449,15 @@ proc magic::gencell_create {gencell_type library parameters} {
 	    puts stderr $drawerr
 	}
 	set gname [instance list celldef $instname]
-	pushstack $gname
-	property library $library 
-	property gencell $gencell_type
-	property parameters $parameters
-	popstack
 	eval "box values $savebox"
     } else {
-	# Give cell a unique name
-	set pidx 1
-	while {[cellname list exists ${gencell_type}_$pidx] != 0} {
-	    incr pidx
-	}
-        set gname ${gencell_type}_$pidx
+        set gname [magic::get_gencell_name ${gencell_type}]
 	cellname create $gname
 	pushstack $gname
 	if {[catch {${library}::${gencell_type}_draw $parameters} drawerr]} {
 	    puts stderr $drawerr
 	}
-	property library $library 
+	property library $library
 	property gencell $gencell_type
 	property parameters $parameters
 	popstack
@@ -426,6 +465,10 @@ proc magic::gencell_create {gencell_type library parameters} {
 	set instname [getcell $gname]
 	expand
     }
+    if {$newinstname != ""} {
+	identify $newinstname
+	set instname $newinstname
+    }
     snap $snaptype
     resumeall
     redraw
@@ -433,7 +476,7 @@ proc magic::gencell_create {gencell_type library parameters} {
 }
 
 #-----------------------------------------------------
-#  Add a standard entry parameter to the gencell window 
+#  Add a standard entry parameter to the gencell window
 #-----------------------------------------------------
 
 proc magic::add_entry {pname ptext parameters} {
@@ -443,7 +486,7 @@ proc magic::add_entry {pname ptext parameters} {
    } else {
        set value ""
    }
-   
+
    set numrows [lindex [grid size .params.edits] 1]
    label .params.edits.${pname}_lab -text $ptext
    entry .params.edits.${pname}_ent -background white -textvariable magic::${pname}_val
@@ -456,8 +499,8 @@ proc magic::add_entry {pname ptext parameters} {
 }
 
 #----------------------------------------------------------
-# Default entry callback, without any dependencies.  Each 
-# parameter changed 
+# Default entry callback, without any dependencies.  Each
+# parameter changed
 #----------------------------------------------------------
 
 proc magic::add_check_callbacks {gencell_type library} {
@@ -484,15 +527,39 @@ proc magic::add_check_callbacks {gencell_type library} {
 # values and writes them back to the parameter dictionary.
 # The callback function must return the modified parameters
 # dictionary.
+#
+# Also handle dependencies on checkboxes and selection lists
 #----------------------------------------------------------
 
 proc magic::add_dependency {callback gencell_type library args} {
+    if {[llength $args] == 0} {
+	# If no arguments are given, do for all parameters
+	set parameters ${library}::${gencell_type}_defaults
+	magic::add_dependency $callback $gencell_type $library \
+			{*}[dict keys $parameters]
+	return
+    }
+    set clist [winfo children .params.edits]
     foreach pname $args {
-	# Add callback on enter or focus out
-	bind .params.edits.${pname}_ent <Return> \
-		"magic::update_dialog $callback $pname $gencell_type $library"
-	bind .params.edits.${pname}_ent <FocusOut> \
-		"magic::update_dialog $callback $pname $gencell_type $library"
+        if {[lsearch $clist .params.edits.${pname}_ent] >= 0} {
+	    # Add callback on enter or focus out
+	    bind .params.edits.${pname}_ent <Return> \
+			"magic::update_dialog $callback $pname $gencell_type $library"
+	    bind .params.edits.${pname}_ent <FocusOut> \
+			"magic::update_dialog $callback $pname $gencell_type $library"
+	} elseif {[lsearch $clist .params.edits.${pname}_chk] >= 0} {
+	    # Add callback on checkbox change state
+	    .params.edits.${pname}_chk configure -command \
+			"magic::update_dialog $callback $pname $gencell_type $library"
+	} elseif {[lsearch $clist .params.edits.${pname}_sel] >= 0} {
+	    set smenu .params.edits.${pname}_sel.menu
+	    set sitems [${smenu} index end]
+	    for {set idx 0} {$idx <= $sitems} {incr idx} {
+		set curcommand [${smenu} entrycget $idx -command]
+		${smenu} entryconfigure $idx -command "$curcommand ; \
+		magic::update_dialog $callback $pname $gencell_type $library"
+	    }
+	}
     }
 }
 
@@ -503,6 +570,14 @@ proc magic::add_dependency {callback gencell_type library args} {
 proc magic::update_dialog {callback pname gencell_type library} {
     set pdefaults [${library}::${gencell_type}_defaults]
     set parameters [dict merge $pdefaults [magic::gencell_getparams]]
+
+    if {[dict exists $parameters gencell]} {
+        # Setting special parameter "gencell" forces the gencell to change type
+	set gencell_type [dict get $parameters gencell]
+	set pdefaults [${library}::${gencell_type}_defaults]
+	set parameters [dict merge $pdefaults [magic::gencell_getparams]]
+    }
+
     if {$callback != {}} {
        set parameters [$callback $pname $parameters]
     }
@@ -514,7 +589,7 @@ proc magic::update_dialog {callback pname gencell_type library} {
 }
 
 #----------------------------------------------------------
-#  Add a standard checkbox parameter to the gencell window 
+#  Add a standard checkbox parameter to the gencell window
 #----------------------------------------------------------
 
 proc magic::add_checkbox {pname ptext parameters} {
@@ -524,7 +599,7 @@ proc magic::add_checkbox {pname ptext parameters} {
    } else {
        set value ""
    }
-   
+
    set numrows [lindex [grid size .params.edits] 1]
    label .params.edits.${pname}_lab -text $ptext
    checkbutton .params.edits.${pname}_chk -variable magic::${pname}_val
@@ -534,21 +609,44 @@ proc magic::add_checkbox {pname ptext parameters} {
 }
 
 #----------------------------------------------------------
-#  Add a selectable-list parameter to the gencell window 
+# Add a message box (informational, not editable) to the
+# gencell window.  Note that the text does not have to be
+# in the parameter list, as it can be upated through the
+# textvariable name.
 #----------------------------------------------------------
 
-proc magic::add_selectlist {pname ptext all_values parameters} {
+proc magic::add_message {pname ptext parameters {color blue}} {
+
+   if [dict exists $parameters $pname] {
+      set value [dict get $parameters $pname]
+   } else {
+      set value ""
+   }
+
+   set numrows [lindex [grid size .params.edits] 1]
+   label .params.edits.${pname}_lab -text $ptext
+   label .params.edits.${pname}_txt -text $value \
+		-foreground $color -textvariable magic::${pname}_val
+   grid .params.edits.${pname}_lab -row $numrows -column 0 -sticky ens
+   grid .params.edits.${pname}_txt -row $numrows -column 1 -sticky wns
+}
+
+#----------------------------------------------------------
+#  Add a selectable-list parameter to the gencell window
+#----------------------------------------------------------
+
+proc magic::add_selectlist {pname ptext all_values parameters {itext ""}} {
 
    if [dict exists $parameters $pname] {
         set value [dict get $parameters $pname]
    } else {
-       set value ""
+       set value $itext
    }
 
    set numrows [lindex [grid size .params.edits] 1]
    label .params.edits.${pname}_lab -text $ptext
    menubutton .params.edits.${pname}_sel -menu .params.edits.${pname}_sel.menu \
-		-relief groove -text ${value} 
+		-relief groove -text ${value}
    grid .params.edits.${pname}_lab -row $numrows -column 0 -sticky ens
    grid .params.edits.${pname}_sel -row $numrows -column 1 -sticky wns
    menu .params.edits.${pname}_sel.menu -tearoff 0
@@ -560,6 +658,38 @@ proc magic::add_selectlist {pname ptext all_values parameters} {
    set magic::${pname}_val $value
 }
 
+#----------------------------------------------------------
+#  Add a selectable-list parameter to the gencell window
+#  Unlike the routine above, it returns the index of the
+#  selection, not the selection itself.  This is useful for
+#  keying the selection to other parameter value lists.
+#----------------------------------------------------------
+
+proc magic::add_selectindex {pname ptext all_values parameters {ival 0}} {
+
+   if [dict exists $parameters $pname] {
+        set value [dict get $parameters $pname]
+   } else {
+       set value $ival
+   }
+
+   set numrows [lindex [grid size .params.edits] 1]
+   label .params.edits.${pname}_lab -text $ptext
+   menubutton .params.edits.${pname}_sel -menu .params.edits.${pname}_sel.menu \
+		-relief groove -text [lindex ${all_values} ${value}]
+   grid .params.edits.${pname}_lab -row $numrows -column 0 -sticky ens
+   grid .params.edits.${pname}_sel -row $numrows -column 1 -sticky wns
+   menu .params.edits.${pname}_sel.menu -tearoff 0
+   set idx 0
+   foreach item ${all_values} {
+       .params.edits.${pname}_sel.menu add radio -label $item \
+	-variable magic::${pname}_val -value $idx \
+	-command ".params.edits.${pname}_sel configure -text $item"
+       incr idx
+   }
+   set magic::${pname}_val $value
+}
+
 #-------------------------------------------------------------
 # gencell_defaults ---
 #
@@ -627,9 +757,11 @@ proc magic::gencell_dialog {instname gencell_type library parameters} {
        # shown in the parameter window.
        if {![catch {set state [wm state .params]}]} {
           if {$instname == {}} {
-	     set devstr [.params.title cget -text]
-	     if {![regexp {^Edit device \"(.*)\" library \"(.*)\".*$} \
-			$devstr valid gencell_type library]} {
+	     set devstr [.params.title.lab1 cget -text]
+	     if {$devstr == "Edit device:"} {
+		 set gencell_type [.params.title.lab2 cget -text]
+		 set library [.params.title.lab4 cget -text]
+	     } else {
 	         return
 	     }
 	  }
@@ -638,13 +770,12 @@ proc magic::gencell_dialog {instname gencell_type library parameters} {
 
    if {$instname != {}} {
       # Remove any array component of the instance name
+      set instname [string map {\\ ""} $instname]
       if {[regexp {^(.*)\[[0-9,]+\]$} $instname valid instroot]} {
 	 set instname $instroot
       }
       set gname [instance list celldef [subst $instname]]
-      if {$gencell_type == {}} {
-	 set gencell_type [cellname list property $gname gencell]
-      }
+      set gencell_type [cellname list property $gname gencell]
       if {$library == {}} {
 	 set library [cellname list property $gname library]
       }
@@ -656,34 +787,76 @@ proc magic::gencell_dialog {instname gencell_type library parameters} {
       if {$parameters == {}} {
 	 set parameters [${library}::${gencell_type}_defaults]
       }
-      set ttext "Edit device"
-   } else {
-      set pidx 1
-      while {[cellname list exists ${gencell_type}_$pidx] != 0} {
-	  incr pidx
+
+      # If the default parameters contain "nocell", then set the
+      # standard parameters for fixed devices from the instance
+      if {[dict exists $parameters nocell]} {
+	 select cell $instname
+	 set arcount [array -list count]
+	 set arpitch [array -list pitch]
+
+	 dict set parameters nx [expr [lindex $arcount 1] - [lindex $arcount 0] + 1]
+	 dict set parameters ny [expr [lindex $arcount 3] - [lindex $arcount 2] + 1]
+	 dict set parameters pitchx [lindex $arpitch 0]
+	 dict set parameters pitchy [lindex $arpitch 1]
       }
+      set ttext "Edit device"
+      set itext $instname
+   } else {
       set parameters [magic::gencell_defaults $gencell_type $library $parameters]
+      set gname "(default)"
+      set itext "(default)"
       set ttext "New device"
    }
 
    # Destroy children, not the top-level window, or else window keeps
    # bouncing around every time something is changed.
    if {[catch {toplevel .params}]} {
-       foreach child [winfo children .params] {
+       .params.title.lab1 configure -text "${ttext}:"
+       .params.title.lab2 configure -text "$gencell_type"
+       .params.title.lab4 configure -text "$library"
+       .params.title.glab configure -foreground blue -text "$gname"
+       .params.title.ient delete 0 end
+       .params.title.ient insert 0 "$itext"
+       foreach child [winfo children .params.edits] {
 	  destroy $child
        }
-   }
-   label .params.title -text "$ttext \"$gencell_type\" library \"$library\"" \
-		-foreground blue
-   ttk::separator .params.sep
-   frame .params.edits
-   frame .params.buttons
-   pack .params.title
-   pack .params.sep -fill x -expand true
-   pack .params.edits -side top -fill both -expand true -ipadx 5
-   pack .params.buttons -fill x
+       foreach child [winfo children .params.buttons] {
+	  destroy $child
+       }
+   } else {
+       frame .params.title
+       label .params.title.lab1 -text "${ttext}:"
+       label .params.title.lab2 -foreground blue -text "$gencell_type"
+       label .params.title.lab3 -text "Library:"
+       label .params.title.lab4 -foreground blue -text "$library"
+       label .params.title.clab -text "Cellname:"
+       label .params.title.glab -foreground blue -text "$gname"
+       label .params.title.ilab -text "Instance:"
+       entry .params.title.ient -foreground brown -background white
+       .params.title.ient insert 0 "$itext"
+       ttk::separator .params.sep
+       frame .params.edits
+       frame .params.buttons
 
-   grid columnconfigure .params.edits 1 -weight 1
+       grid .params.title.lab1 -padx 5 -row 0 -column 0
+       grid .params.title.lab2 -padx 5 -row 0 -column 1 -sticky w
+       grid .params.title.lab3 -padx 5 -row 0 -column 2
+       grid .params.title.lab4 -padx 5 -row 0 -column 3 -sticky w
+
+       grid .params.title.clab -padx 5 -row 1 -column 0
+       grid .params.title.glab -padx 5 -row 1 -column 1 -sticky w
+       grid .params.title.ilab -padx 5 -row 1 -column 2
+       grid .params.title.ient -padx 5 -row 1 -column 3 -sticky ew
+       grid columnconfigure .params.title 3 -weight 1
+
+       pack .params.title -fill x -expand true
+       pack .params.sep -fill x -expand true
+       pack .params.edits -side top -fill both -expand true -ipadx 5
+       pack .params.buttons -fill x
+
+       grid columnconfigure .params.edits 1 -weight 1
+   }
 
    if {$instname == {}} {
 	button .params.buttons.apply -text "Create" -command \
@@ -702,11 +875,14 @@ proc magic::gencell_dialog {instname gencell_type library parameters} {
 		"magic::gencell_change $instname $gencell_type $library {} ;\
 		 destroy .params"
    }
+   button .params.buttons.reset -text "Reset" -command \
+		"magic::gencell_dialog {} ${gencell_type} ${library} {}"
    button .params.buttons.close -text "Close" -command {destroy .params}
 
    pack .params.buttons.apply -padx 5 -ipadx 5 -ipady 2 -side left
    pack .params.buttons.okay  -padx 5 -ipadx 5 -ipady 2 -side left
    pack .params.buttons.close -padx 5 -ipadx 5 -ipady 2 -side right
+   pack .params.buttons.reset -padx 5 -ipadx 5 -ipady 2 -side right
 
    # Invoke the callback procedure that creates the parameter entries
 
