@@ -55,7 +55,7 @@ proc magic::resumeall {} {
 proc magic::makecrashbackup {} {
    global Opts
 
-   crash save
+   *bypass crash save
    if {$Opts(backupinterval) > 0} {
       after $Opts(backupinterval) magic::makecrashbackup
    }
@@ -83,7 +83,7 @@ proc magic::crashbackups {{option start}} {
 #--------------------------------------------------------------------------
 
 proc magic::pushstack {{name ""}} {
-   global editstack  
+   global editstack
    if {$name == ""} {
       # no cell selected, so see if we can select one
       set selected [what -list]
@@ -121,8 +121,8 @@ proc magic::popstack {} {
    } else {
       set ltag [tag load]
       tag load {}
-      load [lindex $editstack end]             
-      view [lindex $editstack end-1]             
+      load [lindex $editstack end]
+      view [lindex $editstack end-1]
       tag load $ltag
       set editstack [lrange $editstack 0 end-2]
       catch {magic::cellmanager}
@@ -467,7 +467,7 @@ proc magic::enable_tools {} {
    # Set keystrokes for push and pop
    magic::macro XK_greater {magic::pushstack [cellname list self]}
    magic::macro XK_less {magic::popstack}
- 
+
    # Set keystrokes for the "tool" command.
    magic::macro space		{magic::tool}
    magic::macro Shift_space	{magic::tool box}
@@ -485,16 +485,17 @@ proc magic::enable_tools {} {
 
 proc magic::trackwire {window {option {}}} {
    global Opts
+
    if {$Opts(motion) == {}} {
       if {$option == "done"} {
 	 wire switch
       } elseif {$option == "pick"} {
 	 puts stdout $window
 	 wire type
-         set Opts(motion) [bind ${window} <Motion>]
-         bind ${window} <Motion> [subst {$Opts(motion); *bypass wire show}]
+	 set Opts(motion) [bind ${window} <Motion>]
+	 bind ${window} <Motion> [subst {$Opts(motion); *bypass wire show}]
 	 if {$Opts(motion) == {}} {set Opts(motion) "null"}
-         cursor 21
+	 cursor 21
       }
    } else {
       if {$option != "cancel"} {
@@ -654,12 +655,12 @@ proc magic::tool {{type next}} {
 	 macro  Button1          "magic::trackwire %W pick"
 	 macro  Button2          "magic::trackwire %W done"
 	 macro  Button3          "magic::trackwire %W cancel"
-         macro  Shift_Button1    "wire incr type"
+         macro  Shift_Button1    "wire incr type ; wire show"
 	 macro  Shift_Button2    "wire switch"
-	 macro  Shift_Button3    "wire decr type"
-	 macro  Button4 "wire incr width"
-	 macro  Button5 "wire decr width"
-	
+	 macro  Shift_Button3    "wire decr type ; wire show"
+	 macro  Button4 "wire incr width ; wire show"
+	 macro  Button5 "wire decr width ; wire show"
+
       }
       netlist {
 	 puts stdout {Switching to NETLIST tool.}
