@@ -1,15 +1,15 @@
 /* path.c
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  *
  * This file contains routines that implement a path mechanism, whereby
@@ -17,12 +17,13 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/utils/path.c,v 1.2 2008/06/01 18:37:47 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/utils/path.c,v 1.3 2009/05/13 15:03:18 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <errno.h>
 #include <pwd.h>
 #include <ctype.h>
 #include <sys/param.h>
@@ -43,7 +44,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
  */
 
 static HashTable expansionTable;
-static noTable = TRUE;
+static bool noTable = TRUE;
 
 /* Limit on how long a single file name may be: */
 
@@ -54,7 +55,7 @@ static noTable = TRUE;
  *	Add a string to the designated path variable.
  *
  *	pathptr is a pointer to the path variable;  srcptr is the
- *	newstring is the new string to append to the path. 
+ *	newstring is the new string to append to the path.
  */
 void
 PaAppend(char **pathptr, char *newstring)
@@ -72,7 +73,7 @@ PaAppend(char **pathptr, char *newstring)
     *pathptr = new;
 }
 
-
+
 /*-------------------------------------------------------------------
  * PaExpand --
  *	This routine converts tilde notation and environment variable
@@ -156,7 +157,7 @@ PaExpand(psource, pdest, size)
 	 * home directory.
 	 */
 
-	if (strcmp(expandName, "") == 0) 
+	if (strcmp(expandName, "") == 0)
 	    string = getenv("HOME");
 	else
 	{
@@ -283,7 +284,7 @@ noexpand:
     return size;
 }
 
-
+
 /* ----------------------------------------------------------------------------
  * nextName --
  *	This local procedure is used to step through a path, adding a
@@ -357,7 +358,7 @@ nextName(ppath, file, dest, size)
  *	the given file name.  If the first character of the
  *	file name is "~" or "/" or if nosearch is TRUE, then we try
  *	to look up the file with the original name, doing tilde
- *	expansion of course and returning that result.  If none of 
+ *	expansion of course and returning that result.  If none of
  *	these conditions is met, we go through the path	trying to
  *	look up the file once for each path entry by prepending the
  *	path entry to the original file name. This concatenated name
@@ -414,17 +415,26 @@ PaLockOpen(file, mode, ext, path, library, pRealName, is_locked)
     /* See if we must supply an extension. */
 
     length = strlen(file);
-    if (length >= MAXSIZE) length = MAXSIZE-1;
+    if (length >= MAXSIZE) length = MAXSIZE - 1;
     if (ext != NULL)
     {
-	(void) strncpy(extendedName, file, length+1);
+	(void) strncpy(extendedName, file, length + 1);
 	i = MAXSIZE - 1 - length;
 	extLength = strlen(ext);
 	if (extLength > i) extLength = i;
 
-	/* if the extension is already on the name, don't add it */
-	if ( (length < extLength) || ((extLength > 0) && (strcmp(ext, file + length - extLength))) )
-	    (void) strncpy(&(extendedName[length]), ext, extLength+1);
+	/* (Modified by Tim, 1/13/2015;  assume that "file" has	*/
+	/* the extension already stripped, therefore always add	*/
+	/* the extension if one is specified.  This allows the	*/
+	/* code to distinguish between, say, "a.mag" and	*/
+	/* "a.mag.mag".)					*/
+
+	/* If the extension is already on the name, don't add it */
+	// if ((length < extLength) || ((extLength > 0)
+	//		&& (strcmp(ext, file + length - extLength))))
+
+	    (void) strncpy(&(extendedName[length]), ext, extLength + 1);
+
 	extendedName[MAXSIZE-1] = '\0';
 	file = extendedName;
     }
@@ -477,6 +487,10 @@ PaLockOpen(file, mode, ext, path, library, pRealName, is_locked)
 	f = flock_open(realName, mode, is_locked);
 #endif
 	if (f != NULL) return f;
+
+	// If any error other than "file not found" occurred,
+	// then halt immediately.
+	if (errno != ENOENT) return NULL;
     }
 
     /* We've tried the path and that didn't work.  Now go through
@@ -492,6 +506,10 @@ PaLockOpen(file, mode, ext, path, library, pRealName, is_locked)
 	f = flock_open(realName, mode, is_locked);
 #endif
 	if (f != NULL) return f;
+
+	// If any error other than "file not found" occurred,
+	// then halt immediately.
+	if (errno != ENOENT) return NULL;
     }
 
     return NULL;
@@ -580,7 +598,7 @@ char *newWD;			/* New working directory to be used.  Must
 	/* Scan through the old path, copying separators to the new
 	 * path until getting the first character of the directory.
 	 */
-	
+
 	while (isspace(*pOld) || (*pOld == ':'))
 	{
 	    if (spaceLeft <= 0) goto subsDone;
@@ -591,7 +609,7 @@ char *newWD;			/* New working directory to be used.  Must
 	/* If the first character of the directory isn't a "/", "$", or "~",
 	 * then add in the new directory name in front of it in newPath.
 	 */
-	
+
 	if ((*pOld != '/') && (*pOld != '~') && (*pOld != '$') && (*pOld != 0))
 	{
 	    pWD = newWD;
@@ -621,7 +639,7 @@ char *newWD;			/* New working directory to be used.  Must
     else newPath[NEWPATHSIZE-1] = 0;
     return newPath;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *	PaEnum --
