@@ -4,19 +4,19 @@
  * Contains definitions for things that are exported by the
  * wiring module.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  *
- * rcsid $Header: /usr/cvsroot/magic-8.0/wiring/wiring.h,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $
+ * rcsid $Header$
  */
 
 #ifndef _WIRING_H
@@ -48,10 +48,15 @@ typedef struct _Contact
 				 * con_layer1 must be painted around the
 				 * edge of the contact.
 				 */
+    int con_extend1;		/* How much additional material of type
+				 * con_layer1 must extend beyond the
+				 * contact in the orientation of the route.
+				 */
     TileType con_layer2;	/* Same information for second layer that
 				 * the contact connects.
 				 */
     int con_surround2;
+    int con_extend2;
 
     ContactPtr con_next;	/* Pointer to next contact record */
 } Contact;
@@ -71,6 +76,8 @@ extern void WirePickType();
 extern void WireAddLeg();
 extern void WireAddContact();
 extern void WireShowLeg();
+extern int WireGetWidth();
+extern TileType WireGetType();
 
 /* Legal values for the "direction" parameter to WireAddLeg: */
 
